@@ -25,6 +25,8 @@
 
 const gchar *cEnvName[4] = {"other", "gnome", "kde", "xfce"};  // "other" allows to handle all the other DE (put your own actions into the corresponding file).
 
+// TODO idea: cairo_dock_cut_string => g_strstrip + cut X first char, then add '...', then the X last char + removed '\n'
+
 static int _cd_clipper_compare_item (CDClipperItem *pItem1, CDClipperItem *pItem2)
 {
 	if (pItem1->iType < pItem2->iType)  // on insere en 1er les items de la primary.
@@ -35,7 +37,7 @@ static int _cd_clipper_compare_item (CDClipperItem *pItem1, CDClipperItem *pItem
 		return 0;
 }
 
-static GList *_cd_clipper_get_last_item (CDClipperItemType iItemType)
+GList *cd_clipper_get_last_item (CDClipperItemType iItemType)
 {
 	CDClipperItem *pItem = NULL;
 	GList *pElement;
@@ -136,10 +138,10 @@ void _on_text_received (GtkClipboard *pClipBoard, const gchar *text, gpointer us
 		myData.pItems = g_list_delete_link (myData.pItems, pElement);
 		myData.iNbItems[pItem->iType] --;
 		
-		if (pItem->iType != iType && myData.iNbItems[iType] == myConfig.iNbItems[iType])
+		if (pItem->iType != iType && myData.iNbItems[iType] >= myConfig.iNbItems[iType])
 		{
 			cd_debug ("Clipper : %s remplace le dernier de l'autre selection", text);
-			pElement = _cd_clipper_get_last_item (iType);
+			pElement = cd_clipper_get_last_item (iType);
 			CD_APPLET_LEAVE_IF_FAIL (pElement != NULL);
 			cd_clipper_free_item (pElement->data);
 			myData.pItems = g_list_delete_link (myData.pItems, pElement);
@@ -152,10 +154,10 @@ void _on_text_received (GtkClipboard *pClipBoard, const gchar *text, gpointer us
 	{
 		bExistingItem = FALSE;
 		cd_debug ("%d items / %d", myData.iNbItems[iType], myConfig.iNbItems[iType]);
-		if (myData.iNbItems[iType] == myConfig.iNbItems[iType])
+		if (myData.iNbItems[iType] >= myConfig.iNbItems[iType])
 		{
 			cd_debug ("Clipper : %s remplace le dernier", text);
-			pElement = _cd_clipper_get_last_item (iType);
+			pElement = cd_clipper_get_last_item (iType);
 			CD_APPLET_LEAVE_IF_FAIL (pElement != NULL);
 			cd_clipper_free_item (pElement->data);
 			myData.pItems = g_list_delete_link (myData.pItems, pElement);
@@ -168,7 +170,7 @@ void _on_text_received (GtkClipboard *pClipBoard, const gchar *text, gpointer us
 		pItem = g_new0 (CDClipperItem, 1);
 		pItem->iType = iType;
 		pItem->cText = g_strdup (text);
-		pItem->cDisplayedText = cairo_dock_cut_string (text, 50);
+		pItem->cDisplayedText = cairo_dock_cut_string (g_strstrip (g_strdup (text)), 50); // g_strstrip: Removes leading and trailing whitespace from a string
 	}
 	myData.pItems = g_list_insert_sorted (myData.pItems, pItem, (GCompareFunc)_cd_clipper_compare_item);
 	myData.iNbItems[pItem->iType] ++;
@@ -425,13 +427,14 @@ GtkWidget *cd_clipper_build_action_menu (CDClipperAction *pAction)
 	CDClipperCommand *pCommand;
 	gchar *cIconFilePath;
 	GList *pElement;
+	gint iDesiredIconSize = cairo_dock_search_icon_size (GTK_ICON_SIZE_LARGE_TOOLBAR); // 24px by default
 	for (pElement = pAction->pCommands; pElement != NULL; pElement = pElement->next)
 	{
 		pCommand = pElement->data;
 		if (pCommand->cIconFileName != NULL)
 		{
 			cd_debug (" icone %s", pCommand->cIconFileName);
-			cIconFilePath = cairo_dock_search_icon_s_path (pCommand->cIconFileName);
+			cIconFilePath = cairo_dock_search_icon_s_path (pCommand->cIconFileName, iDesiredIconSize);
 		}
 		else
 		{
@@ -440,17 +443,20 @@ GtkWidget *cd_clipper_build_action_menu (CDClipperAction *pAction)
 				tmp ++;
 			gchar *cIconName = g_strndup (pCommand->cFormat, tmp - pCommand->cFormat);
 			cd_debug (" icone %s", cIconName);
-			cIconFilePath = cairo_dock_search_icon_s_path (cIconName);
+			cIconFilePath = cairo_dock_search_icon_s_path (cIconName, iDesiredIconSize);
 			g_free (cIconName);
 		}
 		
 		pMenuItem = gtk_image_menu_item_new_with_mnemonic (pCommand->cDescription);
 		if (cIconFilePath != NULL)
 		{
-			GdkPixbuf *pixbuf = gdk_pixbuf_new_from_file_at_size (cIconFilePath, 24, 24, NULL);
+			GdkPixbuf *pixbuf = gdk_pixbuf_new_from_file_at_size (cIconFilePath, iDesiredIconSize, iDesiredIconSize, NULL);
 			pImage = gtk_image_new_from_pixbuf (pixbuf);
 			g_free (cIconFilePath);
 			g_object_unref (pixbuf);
+#if (GTK_MAJOR_VERSION > 2 || GTK_MINOR_VERSION >= 16)
+			gtk_image_menu_item_set_always_show_image (GTK_IMAGE_MENU_ITEM (pMenuItem), TRUE);
+#endif
 			gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (pMenuItem), pImage);
 		}
 		gtk_menu_shell_append  (GTK_MENU_SHELL (pMenu), pMenuItem);
@@ -549,7 +555,7 @@ GtkWidget *cd_clipper_build_items_menu (void)
 		CD_APPLET_ADD_IN_MENU_WITH_DATA ((pItem->cDisplayedText != NULL ? pItem->cDisplayedText : pItem->cText), _cd_clipper_activate_item, pMenu, pItem);
 		if (pElement->next != NULL && ((CDClipperItem *)pElement->next->data)->iType != pItem->iType)
 		{
-			CD_APPLET_ADD_SEPARATOR (pMenu);
+			CD_APPLET_ADD_SEPARATOR_IN_MENU (pMenu);
 		}
 	}
 	return pMenu;
@@ -600,7 +606,10 @@ gchar *cd_clipper_concat_items_of_type (CDClipperItemType iType, const gchar *cS
 		pItem = it->data;
 		if ((pItem->iType & iType) == 0)
 			continue;
-		g_string_append_printf (sText, "%s%s", pItem->cText, it->next ? cSeparator : "");
+		g_string_prepend (sText, pItem->cText); // we prepend the string in order to keep the same order
+		if (it->next)
+			g_string_prepend (sText, cSeparator);
+		// g_string_append_printf (sText, "%s%s", pItem->cText, it->next ? cSeparator : "");
 	}
 	gchar *cText = sText->str;
 	g_string_free (sText, FALSE);
@@ -610,16 +619,20 @@ gchar *cd_clipper_concat_items_of_type (CDClipperItemType iType, const gchar *cS
 void cd_clipper_load_items (const gchar *cItems)
 {
 	CDClipperItem *pItem;
+	int iClipperItemType = myConfig.bSeparateSelections ? CD_CLIPPER_CLIPBOARD : CD_CLIPPER_BOTH;
 	gchar **cItemList = g_strsplit (cItems, CD_ITEMS_DELIMITER, -1);
 	int i;
 	for (i = 0; cItemList[i] != NULL; i ++)
 	{
+		// if we have reduced the number of items to display
+		if (i == myConfig.iNbItems[iClipperItemType])
+			break;
 		pItem = g_new0 (CDClipperItem, 1);
-		pItem->iType = CD_CLIPPER_CLIPBOARD;
+		pItem->iType = iClipperItemType;
 		pItem->cText = cItemList[i];
-		pItem->cDisplayedText = cairo_dock_cut_string (pItem->cText, 50);
+		pItem->cDisplayedText = cairo_dock_cut_string (g_strstrip (pItem->cText), 50); // g_strstrip: Removes leading and trailing whitespace from a string
 		myData.pItems = g_list_insert_sorted (myData.pItems, pItem, (GCompareFunc)_cd_clipper_compare_item);
-		myData.iNbItems[pItem->iType] ++;
+		myData.iNbItems[iClipperItemType] ++;
 	}
 	g_free (cItemList);
 }
