@@ -105,9 +105,10 @@ struct _AppletData {
 	dbusMainObject *pMainObject;
 	const gchar *cProgName;
 	gchar *cBasePath;
-	GtkWidget *pModuleSubMenu;
+	///GtkWidget *pModuleSubMenu;
 	GtkWidget *pModuleMainMenu;
 	dbusApplet *pCurrentMenuDbusApplet;
+	gint iMenuPosition;
 	Window xActiveWindow;
 	CairoDockTask *pGetListTask;
 	GList *pUpdateTasksList;
