@@ -240,6 +240,7 @@ static void _on_text_changed (GtkWidget *pEntry, GtkWidget *pLabel)
 	else
 		cLabel = g_strdup_printf ("<span color=\"red\"><b>%d</b></span>", iNbChars);
 	gtk_label_set_markup (GTK_LABEL (pLabel), cLabel);
+	cairo_dock_set_dialog_widget_text_color (pLabel);
 	g_free (cLabel);
 }
 static void _on_dialog_destroyed (dbusApplet *pDbusApplet)
@@ -268,7 +269,7 @@ static gboolean _applet_popup_dialog (dbusApplet *pDbusApplet, GHashTable *hDial
 	v = g_hash_table_lookup (hDialogAttributes, "icon");
 	if (v && G_VALUE_HOLDS_STRING (v))
 	{
-		cImageFilePath = cairo_dock_search_icon_s_path (g_value_get_string (v));
+		cImageFilePath = cairo_dock_search_icon_s_path (g_value_get_string (v), MAX (pIcon->iImageWidth, pIcon->iImageHeight));
 		attr.cImageFilePath = cImageFilePath;
 	}
 	else
@@ -346,7 +347,7 @@ static gboolean _applet_popup_dialog (dbusApplet *pDbusApplet, GHashTable *hDial
 						GtkWidget *pScrolledWindow = gtk_scrolled_window_new (NULL, NULL);
 						gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (pScrolledWindow), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
 						gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (pScrolledWindow), pOneWidget);
-						gtk_widget_set (pScrolledWindow, "width-request", 230, "height-request", 130, NULL);
+						g_object_set (pScrolledWindow, "width-request", 230, "height-request", 130, NULL);
 						pInteractiveWidget = pScrolledWindow;
 						
 						if (! bEditable)
@@ -366,7 +367,7 @@ static gboolean _applet_popup_dialog (dbusApplet *pDbusApplet, GHashTable *hDial
 						pOneWidget = gtk_entry_new ();
 						pInteractiveWidget = pOneWidget;
 						gtk_entry_set_has_frame (GTK_ENTRY (pOneWidget), FALSE);
-						gtk_widget_set (pOneWidget, "width-request", CAIRO_DIALOG_MIN_ENTRY_WIDTH, NULL);
+						g_object_set (pOneWidget, "width-request", CAIRO_DIALOG_MIN_ENTRY_WIDTH, NULL);
 						if (cInitialText != NULL)
 							gtk_entry_set_text (GTK_ENTRY (pOneWidget), cInitialText);
 						if (! bEditable)
@@ -383,7 +384,7 @@ static gboolean _applet_popup_dialog (dbusApplet *pDbusApplet, GHashTable *hDial
 						GtkWidget *pLabel = gtk_label_new (cLabel);
 						g_free (cLabel);
 						gtk_label_set_use_markup (GTK_LABEL (pLabel), TRUE);
-						GtkWidget *pBox = gtk_hbox_new (FALSE, 3);
+						GtkWidget *pBox = _gtk_hbox_new (3);
 						gtk_box_pack_start (GTK_BOX (pBox), pInteractiveWidget, TRUE, TRUE, 0);
 						gtk_box_pack_start (GTK_BOX (pBox), pLabel, FALSE, FALSE, 0);
 						pInteractiveWidget = pBox;
@@ -398,6 +399,7 @@ static gboolean _applet_popup_dialog (dbusApplet *pDbusApplet, GHashTable *hDial
 						{
 							g_signal_connect (pOneWidget, "changed", G_CALLBACK (_on_text_changed), pLabel);
 							g_object_set_data (G_OBJECT (pOneWidget), "nb-chars-max", GINT_TO_POINTER (iNbCharsMax));
+							gtk_entry_set_width_chars (GTK_ENTRY (pOneWidget), MIN (iNbCharsMax/2, 100));  // a rough estimate is: 140 chars ~ 1024 pixels
 						}
 					}
 				}
@@ -436,18 +438,22 @@ static gboolean _applet_popup_dialog (dbusApplet *pDbusApplet, GHashTable *hDial
 					v = g_hash_table_lookup (hWidgetAttributes, "max-label");
 					if (v && G_VALUE_HOLDS_STRING (v))
 						cMaxLabel = g_value_get_string (v);
-					
+
+					#if (GTK_MAJOR_VERSION < 3)
 					pScale = gtk_hscale_new_with_range (fMinValue, fMaxValue, (fMaxValue - fMinValue) / 100.);
+					#else
+					pScale = gtk_scale_new_with_range (GTK_ORIENTATION_HORIZONTAL, fMinValue, fMaxValue, (fMaxValue - fMinValue) / 100.);
+					#endif
 					pOneWidget = pScale;
 					gtk_scale_set_digits (GTK_SCALE (pScale), iNbDigit);
 					gtk_range_set_value (GTK_RANGE (pScale), fInitialValue);
 					
-					gtk_widget_set (pScale, "width-request", 150, NULL);
+					g_object_set (pScale, "width-request", 150, NULL);
 					cairo_dock_set_dialog_widget_text_color (pScale);
 					
 					if (cMinLabel || cMaxLabel)
 					{
-						GtkWidget *pExtendedWidget = gtk_hbox_new (FALSE, 0);
+						GtkWidget *pExtendedWidget = _gtk_hbox_new (0);
 						GtkWidget *label = gtk_label_new (cMinLabel);
 						GtkWidget *pAlign = gtk_alignment_new (1., 1., 0., 0.);
 						gtk_container_add (GTK_CONTAINER (pAlign), label);
@@ -484,10 +490,17 @@ static gboolean _applet_popup_dialog (dbusApplet *pDbusApplet, GHashTable *hDial
 					if (cValues != NULL)
 						cValuesList = g_strsplit (cValues, ";", -1);
 					
+					#if (GTK_MAJOR_VERSION < 3 && GTK_MINOR_VERSION < 24)
 					if (bEditable)
 						pOneWidget = gtk_combo_box_entry_new_text ();
 					else
 						pOneWidget = gtk_combo_box_new_text ();
+					#else
+					if (bEditable)
+						pOneWidget = gtk_combo_box_text_new_with_entry ();
+					else
+						pOneWidget = gtk_combo_box_text_new ();
+					#endif
 					pInteractiveWidget = pOneWidget;
 					
 					if (cValuesList != NULL)
@@ -495,7 +508,11 @@ static gboolean _applet_popup_dialog (dbusApplet *pDbusApplet, GHashTable *hDial
 						int i;
 						for (i = 0; cValuesList[i] != NULL; i ++)
 						{
+							#if (GTK_MAJOR_VERSION < 3 && GTK_MINOR_VERSION < 24)
 							gtk_combo_box_append_text (GTK_COMBO_BOX (pInteractiveWidget), cValuesList[i]);
+							#else
+							gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (pInteractiveWidget), cValuesList[i]);
+							#endif
 						}
 					}
 					
@@ -645,42 +662,8 @@ gboolean cd_dbus_sub_applet_add_sub_icons (dbusSubApplet *pDbusSubApplet, const
 		cd_warning ("the number of argument is incorrect\nThis may result in an incorrect number of loaded icons.");
 	}
 	
-	if (pInstance->pDock)
-	{
-		if (pIcon->pSubDock == NULL)
-		{
-			if (pIcon->cName == NULL)
-				cairo_dock_set_icon_name (pInstance->pModule->pVisitCard->cModuleName, pIcon, pContainer);
-			if (cairo_dock_check_unique_subdock_name (pIcon))
-				cairo_dock_set_icon_name (pIcon->cName, pIcon, pContainer);
-			pIcon->pSubDock = cairo_dock_create_subdock_from_scratch (pIconsList, pIcon->cName, pInstance->pDock);
-			//cairo_dock_set_renderer (pIcon->pSubDock, cRenderer);
-			cairo_dock_update_dock_size (pIcon->pSubDock);
-		}
-		else
-		{
-			GList *ic;
-			for (ic = pIconsList; ic != NULL; ic = ic->next)
-			{
-				pOneIcon = ic->data;
-				cairo_dock_load_icon_buffers (pOneIcon, CAIRO_CONTAINER (pIcon->pSubDock));
-				cairo_dock_insert_icon_in_dock (pOneIcon, pIcon->pSubDock, ! CAIRO_DOCK_UPDATE_DOCK_SIZE, ! CAIRO_DOCK_ANIMATE_ICON);
-			}
-			cairo_dock_update_dock_size (pIcon->pSubDock);
-			g_list_free (pIconsList);
-		}
-	}
-	else
-	{
-		if (pIcon->pSubDock != NULL)  // precaution.
-		{
-			cairo_dock_destroy_dock (pIcon->pSubDock, pIcon->cName);
-			pIcon->pSubDock = NULL;
-		}
-		pInstance->pDesklet->icons = g_list_concat (pInstance->pDesklet->icons, pIconsList);
-		gpointer data[3] = {GINT_TO_POINTER (0), GINT_TO_POINTER (TRUE), NULL};
-		cairo_dock_set_desklet_renderer_by_name (pInstance->pDesklet, "Panel", (CairoDeskletRendererConfigPtr) data);
-	}
+	gpointer data[3] = {GINT_TO_POINTER (0), GINT_TO_POINTER (TRUE), NULL};
+	cairo_dock_insert_icons_in_applet (pInstance, pIconsList, NULL, "Panel", (CairoDeskletRendererConfigPtr) data);  // NULL <=> default sub-docks renderer
 	
 	return TRUE;
 }
@@ -698,43 +681,13 @@ gboolean cd_dbus_sub_applet_remove_sub_icon (dbusSubApplet *pDbusSubApplet, cons
 	
 	if (cIconID == NULL || strcmp (cIconID, "any") == 0)  // remove all
 	{
-		if (pInstance->pDesklet && pInstance->pDesklet->icons != NULL)
-		{
-			g_list_foreach (pInstance->pDesklet->icons, (GFunc) cairo_dock_free_icon, NULL);
-			g_list_free (pInstance->pDesklet->icons);
-			pInstance->pDesklet->icons = NULL;
-		}
-		if (pIcon->pSubDock != NULL)
-		{
-			if (pInstance->pDesklet)
-			{
-				cairo_dock_destroy_dock (pIcon->pSubDock, pIcon->cName);
-				pIcon->pSubDock = NULL;
-			}
-			else
-			{
-				g_list_foreach (pIcon->pSubDock->icons, (GFunc) cairo_dock_free_icon, NULL);
-				g_list_free (pIcon->pSubDock->icons);
-				pIcon->pSubDock->icons = NULL;
-				pIcon->pSubDock->pFirstDrawnElement = NULL;
-			}
-		}
+		cairo_dock_remove_all_icons_from_applet (pInstance);
 	}
 	else
 	{
 		GList *pIconsList = (pInstance->pDock ? (pIcon->pSubDock ? pIcon->pSubDock->icons : NULL) : pInstance->pDesklet->icons);
 		Icon *pOneIcon = cairo_dock_get_icon_with_command (pIconsList, cIconID);
-		if (pInstance->pDock)
-		{
-			cairo_dock_detach_icon_from_dock (pOneIcon, pIcon->pSubDock, FALSE);
-			cairo_dock_update_dock_size (pIcon->pSubDock);
-		}
-		else
-		{
-			pInstance->pDesklet->icons = g_list_remove (pInstance->pDesklet->icons, pOneIcon);
-			gtk_widget_queue_draw (pInstance->pDesklet->container.pWidget);
-		}
-		cairo_dock_free_icon (pOneIcon);
+		cairo_dock_remove_icon_from_applet (pInstance, pOneIcon);
 	}
 	
 	return TRUE;
@@ -970,9 +923,50 @@ gboolean cd_dbus_applet_show_appli (dbusApplet *pDbusApplet, gboolean bShow, GEr
 	return TRUE;
 }
 
+gboolean cd_dbus_applet_act_on_appli (dbusApplet *pDbusApplet, const gchar *cAction, GError **error)
+{
+	CairoDockModuleInstance *pInstance = _get_module_instance_from_dbus_applet (pDbusApplet);
+	g_return_val_if_fail (pInstance != NULL, FALSE);
+	
+	Icon *pIcon = pInstance->pIcon;
+	g_return_val_if_fail (pIcon != NULL && pIcon->Xid != 0, FALSE);
+	
+	g_return_val_if_fail (cAction != NULL, FALSE);
+	
+	if (strcmp (cAction, "minimize") == 0)
+		cairo_dock_minimize_xwindow (pIcon->Xid);
+	else if (strcmp (cAction, "show") == 0)
+		cairo_dock_show_xwindow (pIcon->Xid);
+	else if (strcmp (cAction, "toggle-visibility") == 0)
+	{
+		if (pIcon->bIsHidden)
+			cairo_dock_show_xwindow (pIcon->Xid);
+		else
+			cairo_dock_minimize_xwindow (pIcon->Xid);
+	}	
+	else if (strcmp (cAction, "maximize") == 0)
+		cairo_dock_maximize_xwindow (pIcon->Xid, TRUE);
+	else if (strcmp (cAction, "restaure") == 0)
+		cairo_dock_maximize_xwindow (pIcon->Xid, FALSE);
+	else if (strcmp (cAction, "toggle-size") == 0)
+	{
+		cairo_dock_maximize_xwindow (pIcon->Xid, ! pIcon->bIsMaximized);
+	}
+	else if (strcmp (cAction, "close") == 0)
+		cairo_dock_close_xwindow (pIcon->Xid);
+	else if (strcmp (cAction, "kill") == 0)
+		cairo_dock_kill_xwindow (pIcon->Xid);
+	else
+	{
+		cd_warning ("invalid action '%s' on window %s", cAction, pIcon->cName);
+	}
+	
+	return TRUE;
+}
+
 gboolean cd_dbus_applet_populate_menu (dbusApplet *pDbusApplet, const gchar **pLabels, GError **error)
 {
-	if (myData.pModuleSubMenu == NULL || pDbusApplet != myData.pCurrentMenuDbusApplet)
+	if (myData.pModuleMainMenu == NULL || pDbusApplet != myData.pCurrentMenuDbusApplet)
 	{
 		cd_warning ("the 'PopulateMenu' method can only be used to populate the menu that was summoned from a right-click on your applet !\nthat is to say, after you received a 'build-menu' event.");
 		return FALSE;
@@ -983,43 +977,35 @@ gboolean cd_dbus_applet_populate_menu (dbusApplet *pDbusApplet, const gchar **pL
 	{
 		if (*pLabels[i] == '\0')
 		{
-			gtk_menu_shell_append (GTK_MENU_SHELL (myData.pModuleSubMenu), gtk_separator_menu_item_new ());
+			gtk_menu_shell_append (GTK_MENU_SHELL (myData.pModuleMainMenu), gtk_separator_menu_item_new ());
 		}
 		else
 		{
 			cairo_dock_add_in_menu_with_stock_and_data (pLabels[i],
 				NULL,
-				(GFunc) cd_dbus_emit_on_menu_select,
-				myData.pModuleSubMenu,
+				G_CALLBACK (cd_dbus_emit_on_menu_select),
+				myData.pModuleMainMenu,
 				GINT_TO_POINTER (i));
 		}
 	}
-	gtk_widget_show_all (myData.pModuleSubMenu);
+	gtk_widget_show_all (myData.pModuleMainMenu);
 	
 	return TRUE;
 }
 
 gboolean cd_dbus_applet_add_menu_items (dbusApplet *pDbusApplet, GPtrArray *pItems, GError **error)
 {
-	if (myData.pModuleMainMenu == NULL || myData.pModuleSubMenu == NULL || pDbusApplet != myData.pCurrentMenuDbusApplet)
+	if (myData.pModuleMainMenu == NULL/** || myData.pModuleSubMenu == NULL*/ || pDbusApplet != myData.pCurrentMenuDbusApplet)
 	{
 		cd_warning ("the 'AddMenuItems' method can only be used to populate the menu that was summoned from a right-click on your applet !\nthat is to say, after you received a 'build-menu' event.");
 		return FALSE;
 	}
 	
-	// on recupere la position du sous-menu par defaut, afin d'inserer les items apres lui dans le menu principal.
-	GList *pChildren = gtk_container_get_children (GTK_CONTAINER (myData.pModuleMainMenu));
-	GList *c = g_list_find (pChildren, myData.pModuleSubMenu);
-	GtkMenuItem *item;
-	for (c = pChildren; c != NULL; c = c->next)
-	{
-		item = c->data;
-		if (gtk_menu_item_get_submenu (item) == myData.pModuleSubMenu)
-			break;
-	}
-	g_return_val_if_fail (c, FALSE);
-	int iPosition = g_list_position (pChildren, c) + 1;
-	g_list_free (pChildren);
+	// get the position of our items in the menu.
+	int iPosition = myData.iMenuPosition;
+	
+	// insert a separator
+	gtk_menu_shell_insert (GTK_MENU_SHELL (myData.pModuleMainMenu), gtk_separator_menu_item_new (), iPosition++);
 	
 	// table des menus et groupes de radio-boutons.
 	GHashTable *pSubMenus = g_hash_table_new_full (g_int_hash,
@@ -1155,21 +1141,16 @@ gboolean cd_dbus_applet_add_menu_items (dbusApplet *pDbusApplet, GPtrArray *pIte
 		v = g_hash_table_lookup (pItem, "menu");
 		if (v && G_VALUE_HOLDS_INT (v))
 			iMenuID = g_value_get_int (v);
-		if (iMenuID == 0)
+		if (iMenuID <= 0)
 			pMenu = myData.pModuleMainMenu;
-		else if (iMenuID == -1)
-			pMenu = myData.pModuleSubMenu;
 		else
 		{
 			pMenu = g_hash_table_lookup (pSubMenus, &iMenuID);
 			if (pMenu == NULL)
-				pMenu = myData.pModuleSubMenu;
+				pMenu = myData.pModuleMainMenu;
 		}
 		
-		if (pMenu == myData.pModuleMainMenu)
-			gtk_menu_shell_insert (GTK_MENU_SHELL (pMenu), pMenuItem, iPosition++);
-		else
-			gtk_menu_shell_append (GTK_MENU_SHELL (pMenu), pMenuItem);
+		gtk_menu_shell_insert (GTK_MENU_SHELL (pMenu), pMenuItem, iPosition++);
 	}
 	
 	g_hash_table_destroy (pSubMenus);
@@ -1188,12 +1169,36 @@ gboolean cd_dbus_applet_bind_shortkey (dbusApplet *pDbusApplet, const gchar **cS
 	CairoDockModuleInstance *pInstance = _get_module_instance_from_dbus_applet (pDbusApplet);
 	g_return_val_if_fail (pInstance != NULL, FALSE);
 	
-	const gchar *cShortkey;
+	const gchar *cShortkey, *cDescription = "Pouet", *cGroupName = "Configuration", *cKeyName = "shortkey";
+	CairoKeyBinding *pKeyBinding;
 	int i;
-	GList *sk, *next_sk;
-	gchar *key;
+	GList *kb;
 	
-	// on enleve les vieux raccourcis dont l'applet ne veut plus.
+	if (pDbusApplet->pShortkeyList == NULL)
+	{
+		for (i = 0; cShortkeys[i] != NULL; i ++)
+		{
+			cShortkey = cShortkeys[i];
+			pKeyBinding = cd_keybinder_bind (cShortkey,
+				pInstance->pModule->pVisitCard->cTitle,
+				cDescription,
+				pInstance->pModule->pVisitCard->cIconFilePath,
+				pInstance->cConfFilePath,
+				cGroupName, cKeyName,
+				(CDBindkeyHandler) cd_dbus_applet_emit_on_shortkey, pDbusApplet),
+			pDbusApplet->pShortkeyList = g_list_append (pDbusApplet->pShortkeyList, pKeyBinding);
+		}
+	}
+	else  // just rebind, we consider that the applet wants to rebind the same shortkeys.
+	{
+		for (i = 0, kb = pDbusApplet->pShortkeyList; cShortkeys[i] != NULL, kb != NULL; i ++, kb = kb->next)
+		{
+			cShortkey = cShortkeys[i];
+			pKeyBinding = kb->data;
+			cd_keybinder_rebind (pKeyBinding, cShortkey, NULL);
+		}
+	}
+	/** // on enleve les vieux raccourcis dont l'applet ne veut plus.
 	sk = pDbusApplet->pShortkeyList;
 	while (sk != NULL)
 	{
@@ -1235,7 +1240,7 @@ gboolean cd_dbus_applet_bind_shortkey (dbusApplet *pDbusApplet, const gchar **cS
 				pDbusApplet->pShortkeyList = g_list_prepend (pDbusApplet->pShortkeyList, g_strdup (cShortkey));
 			cd_debug ("*** bind %s: %d", cShortkey, bCouldBind);
 		}
-	}
+	}*/
 	return TRUE;
 }
 
@@ -1292,16 +1297,10 @@ gboolean cd_dbus_applet_get (dbusApplet *pDbusApplet, const gchar *cProperty, GV
 		g_value_init (v, G_TYPE_UINT);
 		g_value_set_uint (v, pContainer->iType);
 	}
-	else if (strcmp (cProperty, "width") == 0)
+	else if (strcmp (cProperty, "width") == 0)  // this is the dimension of the icon when it's hovered.
 	{
 		int iWidth, iHeight;
 		cairo_dock_get_icon_extent (pIcon, &iWidth, &iHeight);
-		if (pInstance->pDock)
-		{
-			double a = cairo_dock_get_max_scale (pContainer);
-			double s = pInstance->pDock->fMagnitudeMax;
-			iWidth /= (1 + a) / (1 + s*a);
-		}
 		g_value_init (v, G_TYPE_INT);
 		g_value_set_int (v, iWidth);
 	}
@@ -1309,12 +1308,6 @@ gboolean cd_dbus_applet_get (dbusApplet *pDbusApplet, const gchar *cProperty, GV
 	{
 		int iWidth, iHeight;
 		cairo_dock_get_icon_extent (pIcon, &iWidth, &iHeight);
-		if (pInstance->pDock)
-		{
-			double a = cairo_dock_get_max_scale (pContainer);
-			double s = pInstance->pDock->fMagnitudeMax;
-			iHeight /= (1 + a) / (1 + s*a);
-		}
 		g_value_init (v, G_TYPE_INT);
 		g_value_set_int (v, iHeight);
 	}
@@ -1364,13 +1357,6 @@ gboolean cd_dbus_applet_get_all (dbusApplet *pDbusApplet, GHashTable **hProperti
 	CairoDockPositionType iScreenBorder = ((! pContainer->bIsHorizontal) << 1) | (! pContainer->bDirectionUp);
 	int iWidth, iHeight;
 	cairo_dock_get_icon_extent (pIcon, &iWidth, &iHeight);
-	if (pInstance->pDock)
-	{
-		double a = cairo_dock_get_max_scale (pContainer);
-		double s = pInstance->pDock->fMagnitudeMax;
-		iWidth /= (1 + a) / (1 + s*a);
-		iHeight /= (1 + a) / (1 + s*a);
-	}
 	
 	Window Xid = pIcon->Xid;
 	gboolean bHasFocus = (pIcon->Xid != 0 && pIcon->Xid == cairo_dock_get_current_active_window ());
