@@ -81,13 +81,13 @@ static void _on_got_events (ZeitgeistResultSet *pEvents, GtkListStore *pModel)
 	gint64 iTimeStamp;
 	const gchar *cEventURI;
 	guint id;
-	gchar *cName = NULL, *cURI = NULL, *cIconName = NULL, *cPath = NULL;
+	gchar *cName = NULL, *cURI = NULL, *cIconName = NULL, *cIconPath, *cPath = NULL;
 	double fOrder;
 	int iVolumeID;
 	gboolean bIsDirectory;
 	GdkPixbuf *pixbuf;
 	GtkTreeIter iter;
-	GHashTable *pHashTable = g_hash_table_new_full (g_str_hash, g_str_equal, NULL, NULL);
+	GHashTable *pHashTable = g_hash_table_new_full (g_str_hash, g_str_equal, NULL, NULL);  // used to prevent doubles
 	
 	//\_____________ parse all the events.
 	while (zeitgeist_result_set_has_next (pEvents))
@@ -116,16 +116,18 @@ static void _on_got_events (ZeitgeistResultSet *pEvents, GtkListStore *pModel)
 			//\_____________ find the icon.
 			if (strncmp (cEventURI, "http", 4) == 0)  // gvfs is deadly slow to get info on distant URI...
 			{
-				cIconName = cairo_dock_search_icon_s_path ("text-html");
+				cIconName = cairo_dock_search_icon_s_path ("text-html", myData.iDesiredIconSize);
 			}
 			else
 			{
 				cairo_dock_fm_get_file_info (cEventURI, &cName, &cURI, &cIconName, &bIsDirectory, &iVolumeID, &fOrder, CAIRO_DOCK_FM_SORT_BY_DATE);
-				g_free (cName);
-				g_free (cURI);
 			}
 			if (cIconName != NULL)
-				pixbuf = gdk_pixbuf_new_from_file_at_size (cIconName, 32, 32, NULL);
+			{
+				cIconPath = cairo_dock_search_icon_s_path (cIconName, myData.iDesiredIconSize);
+				pixbuf = gdk_pixbuf_new_from_file_at_size (cIconPath, myData.iDesiredIconSize, myData.iDesiredIconSize, NULL);
+				g_free (cIconPath);
+			}
 			else
 				pixbuf = NULL;
 			
@@ -166,7 +168,13 @@ static void _on_got_events (ZeitgeistResultSet *pEvents, GtkListStore *pModel)
 				CD_MODEL_ICON, pixbuf,
 				CD_MODEL_DATE, iTimeStamp,
 				CD_MODEL_ID, id, -1);
+			
 			g_free (cIconName);
+			cIconName = NULL;
+			g_free (cName);
+			cName = NULL;
+			g_free (cURI);
+			cURI = NULL;
 			if (pixbuf)
 				g_object_unref (pixbuf);
 			g_free (cPath);
@@ -266,7 +274,7 @@ static gboolean _on_click_module_tree_view (GtkTreeView *pTreeView, GdkEventButt
 					myData.pAppList = g_list_prepend (myData.pAppList, pAppInfo[1]);
 					
 					if (pAppInfo[2] != NULL)
-						cIconPath = cairo_dock_search_icon_s_path (pAppInfo[2]);
+						cIconPath = cairo_dock_search_icon_s_path (pAppInfo[2], cairo_dock_search_icon_size (GTK_ICON_SIZE_LARGE_TOOLBAR));
 					else
 						cIconPath = NULL;
 					CD_APPLET_ADD_IN_MENU_WITH_STOCK_AND_DATA (pAppInfo[0], cIconPath, _cd_launch_with, pSubMenu, pAppInfo[1]);
@@ -344,17 +352,17 @@ static inline GtkToolItem *_add_category_button (GtkWidget *pToolBar, const gcha
 #define MARGIN 3
 static GtkWidget *cd_build_events_widget (void)
 {
-	GtkWidget *pMainBox = gtk_vbox_new (FALSE, MARGIN);
+	GtkWidget *pMainBox = _gtk_vbox_new (MARGIN);
 	
 	// category toolbar.
 	GtkWidget *pToolBar = gtk_toolbar_new ();
-	gtk_toolbar_set_orientation (GTK_TOOLBAR (pToolBar), GTK_ORIENTATION_HORIZONTAL);
+	///gtk_toolbar_set_orientation (GTK_TOOLBAR (pToolBar), GTK_ORIENTATION_HORIZONTAL);
 	gtk_toolbar_set_style (GTK_TOOLBAR (pToolBar), GTK_TOOLBAR_BOTH);  // overwrite system preference (GTK_TOOLBAR_ICONS)
 	gtk_toolbar_set_show_arrow (GTK_TOOLBAR (pToolBar), FALSE);  // force to display all the entries.
 	gtk_box_pack_start (GTK_BOX (pMainBox), pToolBar, TRUE, TRUE, MARGIN);
 	
 	int i = 0;
-	GtkToolItem *group = _add_category_button (pToolBar, D_("All"), "stock_all", i++, NULL);
+	GtkToolItem *group = _add_category_button (pToolBar, D_("All"), "stock_search", i++, NULL);
 	_add_category_button (pToolBar, D_("Document"), "document", i++, group);
 	///_add_category_button (pToolBar, D_("Folder"), "folder", i++, group);
 	_add_category_button (pToolBar, D_("Image"), "image", i++, group);
@@ -365,10 +373,11 @@ static GtkWidget *cd_build_events_widget (void)
 	_add_category_button (pToolBar, D_("Top Results"), "gtk-about", i++, group);
 	
 	// search entry.
-	GtkWidget *pFilterBox = gtk_hbox_new (FALSE, CAIRO_DOCK_GUI_MARGIN);
+	GtkWidget *pFilterBox = _gtk_hbox_new (CAIRO_DOCK_GUI_MARGIN);
 	gtk_box_pack_start (GTK_BOX (pMainBox), pFilterBox, FALSE, FALSE, MARGIN);
 	
 	GtkWidget *pFilterLabel = gtk_label_new (D_("Look for events"));
+	cairo_dock_set_dialog_widget_text_color (GTK_WIDGET (pFilterLabel));
 	gtk_box_pack_start (GTK_BOX (pFilterBox), pFilterLabel, FALSE, FALSE, MARGIN);
 	
 	GtkWidget *pEntry = gtk_entry_new ();
@@ -431,10 +440,15 @@ static GtkWidget *cd_build_events_widget (void)
 	gtk_tree_view_append_column (GTK_TREE_VIEW (pOneWidget), col);
 	
 	// barres de defilement
+	#if (GTK_MAJOR_VERSION < 3)
 	GtkObject *adj = gtk_adjustment_new (0., 0., 100., 1, 10, 10);
 	gtk_tree_view_set_vadjustment (GTK_TREE_VIEW (pOneWidget), GTK_ADJUSTMENT (adj));
+	#else
+	GtkAdjustment *adj = gtk_adjustment_new (0., 0., 100., 1, 10, 10);
+	gtk_scrollable_set_vadjustment (GTK_SCROLLABLE (pOneWidget), GTK_ADJUSTMENT (adj));
+	#endif
 	GtkWidget *pScrolledWindow = gtk_scrolled_window_new (NULL, NULL);
-	gtk_widget_set (pScrolledWindow, "height-request", 300, NULL);
+	g_object_set (pScrolledWindow, "height-request", 300, NULL);
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (pScrolledWindow), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
 	gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (pScrolledWindow), pOneWidget);
 	gtk_box_pack_start (GTK_BOX (pMainBox), pScrolledWindow, FALSE, FALSE, MARGIN);
@@ -442,13 +456,6 @@ static GtkWidget *cd_build_events_widget (void)
 	return pMainBox;
 }
 
-/**static gboolean on_button_press_dialog (GtkWidget *widget, GdkEventButton *pButton, CairoDockModuleInstance *myApplet)
-{
-	CD_APPLET_ENTER;
-	cairo_dock_dialog_unreference (myData.pDialog);
-	myData.pDialog = NULL;
-	CD_APPLET_LEAVE(FALSE);
-}*/
 static void _on_dialog_destroyed (CairoDockModuleInstance *myApplet)
 {
 	myData.pDialog = NULL;
@@ -456,15 +463,55 @@ static void _on_dialog_destroyed (CairoDockModuleInstance *myApplet)
 	myData.iCurrentCaterogy = CD_EVENT_ALL;
 	myData.pModel = NULL;	
 }
+static gboolean _show_dialog_delayed (gpointer data)
+{
+	cd_toggle_dialog ();
+	if (myData.pDialog != NULL)  // dialog built with success, quit.
+	{
+		myData.iSidTryDialog = 0;
+		return FALSE;
+	}
+	else  // failed, retry up to 3 times.
+	{
+		myData.iNbTries ++;
+		cd_debug (" %d tries", myData.iNbTries);
+		if (myData.iNbTries >= 3)  // definitely no hope -> show a message to the user.
+		{
+			cairo_dock_remove_dialog_if_any (myIcon);
+			cairo_dock_show_temporary_dialog_with_icon (D_("You need to install the Zeitgeist data engine."), myIcon, myContainer, 6000, "same icon");
+			myData.iSidTryDialog = 0;
+			return FALSE;
+		}
+	}
+	return TRUE;
+}
 void cd_toggle_dialog (void)
 {
-	if (myData.pDialog != NULL)
+	if (myData.pDialog != NULL)  // the dialog can be opened in the case it was called from the shortkey.
 	{
 		cairo_dock_dialog_unreference (myData.pDialog);
 		myData.pDialog = NULL;
 	}
 	else
 	{
+		// establish the connection to Zeitgesit.
+		if (myData.pLog == NULL)  // first search.
+		{
+			cd_debug ("first search");
+			myData.pLog = zeitgeist_log_new ();  // may launch the Zeitgeist daemon if it's not yet running.
+		}
+		if (! zeitgeist_log_is_connected (myData.pLog))
+		{
+			cd_debug ("not yet connected");
+			if (myData.iSidTryDialog == 0)
+			{
+				myData.iNbTries = 0;
+				myData.iSidTryDialog = g_timeout_add_seconds (1, _show_dialog_delayed, NULL);
+			}
+			return;
+		}
+		
+		// build the dialog and the tree model.
 		GtkWidget *pInteractiveWidget = cd_build_events_widget ();
 		myData.pDialog = cairo_dock_show_dialog_full (D_("Browse and search in recent events"),
 			myIcon,
@@ -476,11 +523,8 @@ void cd_toggle_dialog (void)
 			myApplet,
 			(GFreeFunc) _on_dialog_destroyed);
 		gtk_widget_grab_focus (myData.pEntry);
-		/**g_signal_connect (G_OBJECT (myData.pDialog->container.pWidget),
-			"button-press-event",
-			G_CALLBACK (on_button_press_dialog),
-			myApplet);*/
 		
+		// trigger the search that will fill the model.
 		cd_trigger_search ();
 	}
 }
