@@ -94,7 +94,9 @@ static void _cd_cclosure_marshal_VOID__STRING_INT_STRING_STRING_STRING_STRING_ST
 		data2);
 }
 #else  // Natty
-#if (INDICATOR_APPLICATIONADDED_HAS_HINT == 1)
+#if (INDICATOR_APPLICATIONADDED_HAS_TITLE == 1)
+static void _cd_cclosure_marshal_VOID__STRING_INT_STRING_STRING_STRING_STRING_STRING_STRING_STRING_STRING (
+#elif (INDICATOR_APPLICATIONADDED_HAS_HINT == 1)
 static void _cd_cclosure_marshal_VOID__STRING_INT_STRING_STRING_STRING_STRING_STRING_STRING_STRING (
 #else
 static void _cd_cclosure_marshal_VOID__STRING_INT_STRING_STRING_STRING_STRING_STRING_STRING (
@@ -107,7 +109,9 @@ static void _cd_cclosure_marshal_VOID__STRING_INT_STRING_STRING_STRING_STRING_ST
 	gpointer marshal_data)
 {
 	//cd_debug ("=== %s ()\n", __func__);
-	#if (INDICATOR_APPLICATIONADDED_HAS_HINT == 1)
+	#if (INDICATOR_APPLICATIONADDED_HAS_TITLE == 1)
+	typedef void (*GMarshalFunc_VOID__STRING_INT_STRING_STRING_STRING_STRING_STRING_STRING_STRING_STRING) (
+	#elif (INDICATOR_APPLICATIONADDED_HAS_HINT == 1)
 	typedef void (*GMarshalFunc_VOID__STRING_INT_STRING_STRING_STRING_STRING_STRING_STRING_STRING) (
 	#else
 	typedef void (*GMarshalFunc_VOID__STRING_INT_STRING_STRING_STRING_STRING_STRING_STRING) (
@@ -123,9 +127,14 @@ static void _cd_cclosure_marshal_VOID__STRING_INT_STRING_STRING_STRING_STRING_ST
 		gchar      *arg_8,
 		#if (INDICATOR_APPLICATIONADDED_HAS_HINT == 1)
 		gchar      *arg_9,
+		#if (INDICATOR_APPLICATIONADDED_HAS_TITLE == 1)
+		gchar      *arg_10,
+		#endif
 		#endif
 		gpointer     data2);
-	#if (INDICATOR_APPLICATIONADDED_HAS_HINT == 1)
+	#if (INDICATOR_APPLICATIONADDED_HAS_TITLE == 1)
+	register GMarshalFunc_VOID__STRING_INT_STRING_STRING_STRING_STRING_STRING_STRING_STRING_STRING callback;
+	#elif (INDICATOR_APPLICATIONADDED_HAS_HINT == 1)
 	register GMarshalFunc_VOID__STRING_INT_STRING_STRING_STRING_STRING_STRING_STRING_STRING callback;
 	#else
 	register GMarshalFunc_VOID__STRING_INT_STRING_STRING_STRING_STRING_STRING_STRING callback;
@@ -133,7 +142,9 @@ static void _cd_cclosure_marshal_VOID__STRING_INT_STRING_STRING_STRING_STRING_ST
 	register GCClosure *cc = (GCClosure*) closure;
 	register gpointer data1, data2;
 	// return_value est NULL ici, car la callback ne renvoit rien.
-	#if (INDICATOR_APPLICATIONADDED_HAS_HINT == 1)
+	#if (INDICATOR_APPLICATIONADDED_HAS_TITLE == 1)
+	g_return_if_fail (n_param_values == 11);
+	#elif (INDICATOR_APPLICATIONADDED_HAS_HINT == 1)
 	g_return_if_fail (n_param_values == 10);
 	#else
 	g_return_if_fail (n_param_values == 9);
@@ -149,7 +160,9 @@ static void _cd_cclosure_marshal_VOID__STRING_INT_STRING_STRING_STRING_STRING_ST
 		data1 = g_value_peek_pointer (param_values + 0);
 		data2 = closure->data;
 	}
-	#if (INDICATOR_APPLICATIONADDED_HAS_HINT == 1)
+	#if (INDICATOR_APPLICATIONADDED_HAS_TITLE == 1)
+	callback = (GMarshalFunc_VOID__STRING_INT_STRING_STRING_STRING_STRING_STRING_STRING_STRING_STRING)
+	#elif (INDICATOR_APPLICATIONADDED_HAS_HINT == 1)
 	callback = (GMarshalFunc_VOID__STRING_INT_STRING_STRING_STRING_STRING_STRING_STRING_STRING)
 	#else
 	callback = (GMarshalFunc_VOID__STRING_INT_STRING_STRING_STRING_STRING_STRING_STRING)
@@ -167,6 +180,9 @@ static void _cd_cclosure_marshal_VOID__STRING_INT_STRING_STRING_STRING_STRING_ST
 		(char*) g_value_get_string (param_values + 8),
 		#if (INDICATOR_APPLICATIONADDED_HAS_HINT == 1)
 		(char*) g_value_get_string (param_values + 9),
+		#if (INDICATOR_APPLICATIONADDED_HAS_TITLE == 1)
+		(char*) g_value_get_string (param_values + 10),
+		#endif
 		#endif
 		data2);
 }
@@ -176,21 +192,27 @@ static void _cd_cclosure_marshal_VOID__STRING_INT_STRING_STRING_STRING_STRING_ST
  /// Signals ///
 ///////////////
 
-static void on_new_application (DBusGProxy *proxy_watcher, const gchar *cIconName, gint iPosition, const gchar *cAdress, const gchar *cObjectPath, const gchar *cIconThemePath, const gchar *cLabel, const gchar *cLabelGuide,
+static void on_new_application (DBusGProxy *proxy_watcher, const gchar *cIconName, gint iPosition, const gchar *cAddress, const gchar *cObjectPath, const gchar *cIconThemePath, const gchar *cLabel, const gchar *cLabelGuide,
 #if (INDICATOR_OLD_NAMES == 0)  // Natty
 const gchar *cAccessbleDesc,  // WTF is this new param ??
 #if (INDICATOR_APPLICATIONADDED_HAS_HINT == 1)
-const gchar *cHint,
+const gchar *cHint,  // <irony> is this a hint to work around a clumsy API ? </irony>
+#if (INDICATOR_APPLICATIONADDED_HAS_TITLE == 1)
+const gchar *cTitle,
+#endif
 #endif
 #endif
 CairoDockModuleInstance *myApplet)
 {
 	CD_APPLET_ENTER;
-	cd_debug ("=== %s (%s, %s, %s, %s, %d)", __func__, cAdress, cObjectPath, cIconName, cIconThemePath, iPosition);
+	cd_debug ("=== %s (%s, %s, %s, %s, %d)", __func__, cAddress, cObjectPath, cIconName, cIconThemePath, iPosition);
 	#if (INDICATOR_OLD_NAMES == 0)  // Natty
 	cd_debug ("    %s", cAccessbleDesc);
 	#if (INDICATOR_APPLICATIONADDED_HAS_HINT == 1)
 	cd_debug ("    %s", cHint);
+	#if (INDICATOR_APPLICATIONADDED_HAS_TITLE == 1)
+	cd_debug ("    %s", cTitle);
+	#endif
 	#endif
 	#endif
 	
@@ -207,7 +229,21 @@ CairoDockModuleInstance *myApplet)
 		}
 	}
 	
-	cd_satus_notifier_add_new_item (cAdress, cObjectPath, iPosition);
+	cd_satus_notifier_add_new_item_with_default (cAddress, cObjectPath, iPosition,
+		#if (INDICATOR_APPLICATIONADDED_HAS_HINT == 1)
+		cIconName ? cIconName : cHint,
+		#else
+		cIconName,
+		#endif
+		cIconThemePath,
+		#if (INDICATOR_OLD_NAMES == 0)
+		cAccessbleDesc && *cAccessbleDesc != '\0' ? cAccessbleDesc :
+		#if (INDICATOR_APPLICATIONADDED_HAS_TITLE == 1)
+		cTitle && *cTitle != '\0' ? cTitle :
+		#endif
+		#endif
+		cLabel
+		);
 	
 	CD_APPLET_LEAVE ();
 }
@@ -248,7 +284,7 @@ static void _on_get_applications_from_service (DBusGProxy *proxy, DBusGProxyCall
 	//\______________________ get the applications list from the service.
 	GPtrArray *pApplications = NULL;
 	GError *erreur = NULL;
-	GType g_type_ptrarray = g_type_ptrarray = dbus_g_type_get_collection ("GPtrArray",
+	GType g_type_ptrarray = dbus_g_type_get_collection ("GPtrArray",
 		dbus_g_type_get_struct("GValueArray",
 			G_TYPE_STRING,  // iconname
 			G_TYPE_INT,  // position
@@ -257,6 +293,15 @@ static void _on_get_applications_from_service (DBusGProxy *proxy, DBusGProxyCall
 			G_TYPE_STRING,  // iconpath
 			G_TYPE_STRING,  // label
 			G_TYPE_STRING,  // labelguide
+			#if (INDICATOR_OLD_NAMES == 0)  // Natty
+			G_TYPE_STRING,  // accessibledesc
+			#if (INDICATOR_APPLICATIONADDED_HAS_HINT == 1)
+			G_TYPE_STRING,  // hint
+			#if (INDICATOR_APPLICATIONADDED_HAS_TITLE == 1)
+			G_TYPE_STRING,  // title
+			#endif
+			#endif
+			#endif
 			G_TYPE_INVALID));
 	gboolean bSuccess = dbus_g_proxy_end_call (proxy,
 		call_id,
@@ -273,14 +318,15 @@ static void _on_get_applications_from_service (DBusGProxy *proxy, DBusGProxyCall
 		CD_APPLET_LEAVE ();
 	
 	//\______________________ build each items.
-	cd_debug ("=== got %d aplications", pApplications->len);
+	cd_debug ("=== got %d applications", pApplications->len);
 	guint i, j;
 	GValueArray *va;
 	GValue *v;
-	CDStatusNotifierItem *pItem=NULL;
+	CDStatusNotifierItem *pItem = NULL;
 	//cd_debug ("=== %d apps in the systray", pApplications->len);
 	for (i = 0; i < pApplications->len; i ++)
 	{
+		// get the properties of the item
 		cd_debug ("=== %d) %p", i, pApplications->pdata[i]);
 		va = pApplications->pdata[i];
 		if (! va)
@@ -288,11 +334,14 @@ static void _on_get_applications_from_service (DBusGProxy *proxy, DBusGProxyCall
 		
 		const gchar *cIconName = NULL;
 		gint iPosition = -1;
-		const gchar *cAdress = NULL;
+		const gchar *cAddress = NULL;
 		const gchar *cObjectPath = NULL;
 		const gchar *cIconThemePath = NULL;
 		const gchar *cLabel = NULL;
 		const gchar *cLabelGuide = NULL;
+		const gchar *cAccessibleDesc = NULL; // natty
+		// const gchar *cHint = NULL; // oneiric
+		const gchar *cTitle = NULL;  // precise
 		
 		v = g_value_array_get_nth (va, 0);
 		if (v && G_VALUE_HOLDS_STRING (v))
@@ -304,10 +353,10 @@ static void _on_get_applications_from_service (DBusGProxy *proxy, DBusGProxyCall
 		
 		v = g_value_array_get_nth (va, 2);
 		if (v && G_VALUE_HOLDS_STRING (v))
-			cAdress = g_value_get_string (v);
+			cAddress = g_value_get_string (v);
 		
 		v = g_value_array_get_nth (va, 3);
-		if (v && G_VALUE_HOLDS_BOXED (v))
+		if (v && G_VALUE_HOLDS (v, DBUS_TYPE_G_OBJECT_PATH))
 			cObjectPath = (gchar*)g_value_get_boxed (v);
 
 		/*cd_debug ("=== cObjectPath : %s", cObjectPath);
@@ -330,22 +379,59 @@ static void _on_get_applications_from_service (DBusGProxy *proxy, DBusGProxyCall
 		if (v && G_VALUE_HOLDS_STRING (v))
 			cLabelGuide = g_value_get_string (v);
 		
-		cd_debug ("===  + item {%s ; %d ; %s ; %s ; %s ; %s ; %s}",
+		#if (INDICATOR_OLD_NAMES == 0)
+		v = g_value_array_get_nth (va, 7);
+		if (v && G_VALUE_HOLDS_STRING (v))
+			cAccessibleDesc = g_value_get_string (v);
+		
+		#if (INDICATOR_APPLICATIONADDED_HAS_TITLE == 1)
+		v = g_value_array_get_nth (va, 9);
+		if (v && G_VALUE_HOLDS_STRING (v))
+			cTitle = g_value_get_string (v);
+		#endif
+		#endif
+		
+		cd_debug ("===  + item {%s ; %d ; %s ; %s ; %s ; %s ; %s ; %s ; %s}",
 			cIconName,
 			iPosition,
-			cAdress,
+			cAddress,
 			cObjectPath,
 			cIconThemePath,
 			cLabel,
-			cLabelGuide);
+			cLabelGuide,
+			cAccessibleDesc,
+			cTitle);
 		
-		pItem = cd_satus_notifier_create_item (cAdress, cObjectPath);
+		if (!cAddress)
+			continue;
+		
+		// ensure we're not duplicating an existing item (this should never happen if the service is ok, but since it doesn't depend on us, let's be careful).
+		GList *it;
+		for (it = myData.pItems; it != NULL; it = it->next)
+		{
+			pItem = it->data;
+			if (strcmp (cAddress, pItem->cService) == 0)  // pItem->cService is never NULL
+			{
+				cd_warning ("Duplicated item: %s (%s)", cIconName, cAddress);
+				return;
+			}
+			if (iPosition != -1 && pItem->iPosition == iPosition)
+			{
+				cd_warning ("Possible duplicated item: %s/%s/%d , %s/%s/%d)", cIconName, cAddress, iPosition, pItem->cIconName, pItem->cService, pItem->iPosition);
+			}
+		}
+		
+		// make a new item based on these properties.
+		pItem = cd_satus_notifier_create_item (cAddress, cObjectPath);
 		if (! pItem)
 			continue;
 		if (pItem->iPosition == -1)
 			pItem->iPosition = iPosition;
-		if (pItem->cTitle == NULL && pItem->cLabel == NULL)
-			pItem->cLabel = g_strdup (cLabel && *cLabel != '\0' ? cLabel : pItem->cId);
+		if (pItem->cTitle == NULL && pItem->cLabel == NULL && pItem->cAccessibleDesc == NULL)
+			pItem->cLabel = g_strdup (cAccessibleDesc && *cAccessibleDesc != '\0' ? cAccessibleDesc :
+			                          cLabel && *cLabel != '\0' ? cLabel :
+			                          cTitle && *cTitle != '\0' ? cTitle :
+			                          NULL);  // don't use cId as a fallback, because it often has cryptic names (nm-applet ; dropbox-xxxx). If the appli doesn't provide a title, it's its fault.
 		myData.pItems = g_list_prepend (myData.pItems, pItem);
 	}
 	
@@ -389,7 +475,9 @@ void cd_satus_notifier_get_items_from_ias (void)
 			G_TYPE_NONE, G_TYPE_STRING, G_TYPE_INT, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INVALID);
 	#else  // Natty
 	dbus_g_object_register_marshaller(
-	#if (INDICATOR_APPLICATIONADDED_HAS_HINT == 1)
+	#if (INDICATOR_APPLICATIONADDED_HAS_TITLE == 1)
+	_cd_cclosure_marshal_VOID__STRING_INT_STRING_STRING_STRING_STRING_STRING_STRING_STRING_STRING,
+	#elif (INDICATOR_APPLICATIONADDED_HAS_HINT == 1)
 	_cd_cclosure_marshal_VOID__STRING_INT_STRING_STRING_STRING_STRING_STRING_STRING_STRING,
 	#else
 	_cd_cclosure_marshal_VOID__STRING_INT_STRING_STRING_STRING_STRING_STRING_STRING,
@@ -399,6 +487,9 @@ void cd_satus_notifier_get_items_from_ias (void)
 			G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING,
 			#if (INDICATOR_APPLICATIONADDED_HAS_HINT == 1)
 			G_TYPE_STRING,
+			#if (INDICATOR_APPLICATIONADDED_HAS_TITLE == 1)
+			G_TYPE_STRING,
+			#endif
 			#endif
 			G_TYPE_INVALID);
 	#endif
@@ -418,6 +509,9 @@ void cd_satus_notifier_get_items_from_ias (void)
 		G_TYPE_STRING,  // accessibledesc
 		#if (INDICATOR_APPLICATIONADDED_HAS_HINT == 1)
 		G_TYPE_STRING, // hint => only with indicator-0.4 (Oneiric)
+		#if (INDICATOR_APPLICATIONADDED_HAS_TITLE == 1)
+		G_TYPE_STRING, // title => only with indicator-0.4.90 (Precise)
+		#endif
 		#endif
 		#endif
 		G_TYPE_INVALID);
@@ -429,6 +523,7 @@ void cd_satus_notifier_get_items_from_ias (void)
 		G_TYPE_INVALID);
 	dbus_g_proxy_connect_signal(myData.pProxyIndicatorApplicationService, "ApplicationRemoved",
 		G_CALLBACK(on_removed_application), myApplet, NULL);
+	// TODO? ApplicationIconChanged, ApplicationIconThemePathChanged, ApplicationLabelChanged, ApplicationTitleChanged
 }
 
   //////////////////
