@@ -50,6 +50,7 @@ static CDCategoryEnum _find_category (const gchar *cCategory)
 
 static CDStatusEnum _find_status (const gchar *cStatus)
 {
+	cd_debug ("STATUS: %s", cStatus);
 	if (!cStatus)
 		return CD_STATUS_ACTIVE;
 	if (*cStatus == 'N')
@@ -114,7 +115,7 @@ static void _show_item_tooltip (Icon *pIcon, CDStatusNotifierItem *pItem)
 	g_free (cText);
 }
 
-static void _show_item_status (Icon *pIcon, CDStatusNotifierItem *pItem)
+/*static void _show_item_status (Icon *pIcon, CDStatusNotifierItem *pItem)
 {
 	switch (pItem->iStatus)
 	{
@@ -134,7 +135,7 @@ static void _show_item_status (Icon *pIcon, CDStatusNotifierItem *pItem)
 			cairo_dock_request_icon_attention (pIcon, myIcon->pSubDock, "rotate", 60);
 		break;
 	}
-}
+}*/
 
 
 static void on_new_item_icon (DBusGProxy *proxy_item, CDStatusNotifierItem *pItem)
@@ -144,6 +145,8 @@ static void on_new_item_icon (DBusGProxy *proxy_item, CDStatusNotifierItem *pIte
 	
 	g_free (pItem->cIconName);
 	pItem->cIconName = cairo_dock_dbus_get_property_as_string (pItem->pProxyProps, CD_STATUS_NOTIFIER_ITEM_IFACE, "IconName");
+	g_free (pItem->cAccessibleDesc);
+	pItem->cAccessibleDesc = cairo_dock_dbus_get_property_as_string (pItem->pProxyProps, CD_STATUS_NOTIFIER_ITEM_IFACE, "IconAccessibleDesc");
 	//g_print ("===  new icon : %s\n", pItem->cIconName);
 	
 	if (pItem->iStatus != CD_STATUS_NEEDS_ATTENTION)
@@ -181,7 +184,8 @@ static void on_new_item_status (DBusGProxy *proxy_item, const gchar *cStatus, CD
 		CD_APPLET_LEAVE ();
 	
 	// update the item
-	if (iPrevStatus == CD_STATUS_PASSIVE || pItem->iStatus == CD_STATUS_PASSIVE)  // hide/show the item.
+	if ((iPrevStatus == CD_STATUS_PASSIVE || pItem->iStatus == CD_STATUS_PASSIVE)
+	&& myConfig.bHideInactive)  // status was/is passive => hide/show the item.
 	{
 		if (myConfig.bCompactMode)
 		{
@@ -326,7 +330,7 @@ static void _on_item_proxy_destroyed (DBusGProxy *proxy_item, CDStatusNotifierIt
 	CD_APPLET_LEAVE ();
 }
 
-gchar *cd_satus_notifier_search_item_icon_s_path (CDStatusNotifierItem *pItem)
+gchar *cd_satus_notifier_search_item_icon_s_path (CDStatusNotifierItem *pItem, gint iSize)
 {
 	g_return_val_if_fail (pItem != NULL, NULL);
 	gchar *cImageName = (pItem->iStatus == CD_STATUS_NEEDS_ATTENTION ? pItem->cAttentionIconName: pItem->cIconName);
@@ -334,7 +338,7 @@ gchar *cd_satus_notifier_search_item_icon_s_path (CDStatusNotifierItem *pItem)
 	gchar *cIconPath = NULL;
 	if (pItem->cIconThemePath != NULL)  // workaround pour des applis telles que dropbox qui trouvent malin de specifier des icones avec des noms hyper generiques (idle.png).
 	{
-		cIconPath = g_strdup_printf ("%s/%s", pItem->cIconThemePath, pItem->cIconName);
+		cIconPath = g_strdup_printf ("%s/%s", pItem->cIconThemePath, cImageName);
 		if (! g_file_test (cIconPath, G_FILE_TEST_EXISTS))
 		{
 			g_free (cIconPath);
@@ -344,16 +348,19 @@ gchar *cd_satus_notifier_search_item_icon_s_path (CDStatusNotifierItem *pItem)
 	
 	if (cIconPath == NULL)
 	{
-		cIconPath = cairo_dock_search_icon_s_path (pItem->cIconName);
+		cIconPath = cairo_dock_search_icon_s_path (cImageName, iSize);
 	}
-	
+
+	if (cIconPath == NULL)
+		cIconPath = g_strdup (MY_APPLET_SHARE_DATA_DIR"/"MY_APPLET_ICON_FILE);
+
 	return cIconPath;
 }
 
 CDStatusNotifierItem *cd_satus_notifier_create_item (const gchar *cService, const gchar *cObjectPath)
 {
 	g_return_val_if_fail  (cService != NULL, NULL);
-	//g_print ("=== %s (%s)\n", __func__, cService);
+	cd_debug ("=== %s (%s, %s)", __func__, cService, cObjectPath);
 	
 	gchar *str = strchr (cService, '/');  // just to be sure.
 	if (str)
@@ -361,19 +368,21 @@ CDStatusNotifierItem *cd_satus_notifier_create_item (const gchar *cService, cons
 	
 	// special case for Ubuntu indicators: we don't know their object path.
 	gchar *cRealObjectPath = NULL;
-	if (cObjectPath != NULL && strncmp (cObjectPath, CD_INDICATOR_APPLICATION_ITEM_OBJ, strlen (CD_INDICATOR_APPLICATION_ITEM_OBJ)) == 0)
+	if (cObjectPath != NULL && strncmp (cObjectPath, CD_INDICATOR_APPLICATION_ITEM_OBJ, strlen (CD_INDICATOR_APPLICATION_ITEM_OBJ)) == 0 && g_str_has_suffix (cObjectPath, "/Menu"))
 	{
 		// I think this is because this path is actually the menu path, and fortunately it's just under the item object's path.
 		const gchar *str = strrchr (cObjectPath, '/');
 		if (str)
+		{
 			cRealObjectPath = g_strndup (cObjectPath, str - cObjectPath);
+		}
 	}
 	else if (cObjectPath == NULL || *cObjectPath == '\0')  // no path, let's assume it's the common one.
 	{
 		cObjectPath = CD_STATUS_NOTIFIER_ITEM_OBJ;
 	}
 	
-	//g_print ("=== %s (cObjectPath: %s)\n", __func__, cObjectPath);
+	//g_print ("=== %s (cObjectPath: %s)\n", __func__, cRealObjectPath ? cRealObjectPath : cObjectPath);
 	
 	//\_________________ get the properties of the item.
 	DBusGProxy *pProxyItemProp = cairo_dock_create_new_session_proxy (
@@ -384,7 +393,7 @@ CDStatusNotifierItem *cd_satus_notifier_create_item (const gchar *cService, cons
 		return NULL;
 	//g_print ("=== owner : %s\n", dbus_g_proxy_get_bus_name (pProxyItemProp));
 
-	cd_debug ("%s, %s, %s", cService, cObjectPath, dbus_g_proxy_get_bus_name (pProxyItemProp));
+	//cd_debug ("%s, %s, %s", cService, cObjectPath, dbus_g_proxy_get_bus_name (pProxyItemProp));
 
 	//g_print ("=== getting properties ...\n");
 	GHashTable *hProps = cairo_dock_dbus_get_all_properties (pProxyItemProp, CD_STATUS_NOTIFIER_ITEM_IFACE);
@@ -462,7 +471,7 @@ CDStatusNotifierItem *cd_satus_notifier_create_item (const gchar *cService, cons
 	
 	const gchar *cLabel = NULL;
 	v = g_hash_table_lookup (hProps, "XAyatanaLabel");
-	if (v && G_VALUE_HOLDS_BOXED(v))
+	if (v && G_VALUE_HOLDS_STRING(v))
 	{
 		cLabel = g_value_get_string (v);
 	}
@@ -470,11 +479,18 @@ CDStatusNotifierItem *cd_satus_notifier_create_item (const gchar *cService, cons
 	
 	const gchar *cLabelGuide = NULL;
 	v = g_hash_table_lookup (hProps, "XAyatanaLabelGuide");
-	if (v && G_VALUE_HOLDS_BOXED(v))
+	if (v && G_VALUE_HOLDS_STRING(v))
 	{
 		cLabelGuide = g_value_get_string (v);
 	}
 	//g_print ("===   cLabelGuide '%s'\n", cLabelGuide);
+
+	const gchar *cAccessibleDesc = NULL;
+	v = g_hash_table_lookup (hProps, "IconAccessibleDesc");
+	if (v && G_VALUE_HOLDS_STRING(v))
+	{
+		cAccessibleDesc = g_value_get_string (v);
+	} // Updated with ApplicationIconChanged
 	
 	// properties supported by KDE.
 	const gchar *cTitle = NULL;
@@ -483,7 +499,7 @@ CDStatusNotifierItem *cd_satus_notifier_create_item (const gchar *cService, cons
 	{
 		cTitle = g_value_get_string (v);
 	}
-	cd_debug ("===   Title '%s", cTitle);
+	cd_debug ("===   Title '%s'", cTitle);
 	
 	v = g_hash_table_lookup (hProps, "WindowId");
 	guint iWindowId = 0;
@@ -516,6 +532,13 @@ CDStatusNotifierItem *cd_satus_notifier_create_item (const gchar *cService, cons
 		pToolTipTab = g_value_get_boxed (v);
 	}
 	
+	gboolean bItemIsMenu = FALSE;  // "when is true the dbusmenu will be shown instead of emitting Activate()"
+	v = g_hash_table_lookup (hProps, "ItemIsMenu");
+	if (v && G_VALUE_HOLDS_BOOLEAN (v))
+	{
+		bItemIsMenu = g_value_get_boolean (v);
+	}
+	
 	DBusGProxy *pProxyItem = cairo_dock_create_new_session_proxy (
 		cService,
 		cRealObjectPath ? cRealObjectPath : cObjectPath,
@@ -533,7 +556,9 @@ CDStatusNotifierItem *cd_satus_notifier_create_item (const gchar *cService, cons
 	pItem->cTitle = g_strdup (cTitle);
 	pItem->cLabel = g_strdup (cLabel);
 	pItem->cLabelGuide = g_strdup (cLabelGuide);
-	pItem->cMenuPath = (cMenuPath ? g_strdup (cMenuPath) : g_strdup (cObjectPath));
+	pItem->cAccessibleDesc = g_strdup (cAccessibleDesc);
+	///pItem->cMenuPath = (cMenuPath ? g_strdup (cMenuPath) : g_strdup (cObjectPath));
+	pItem->cMenuPath = g_strdup (cMenuPath);  // if NULL, we'll just send the ContextMenu() signal.
 	pItem->iWindowId = iWindowId;
 	pItem->iCategory = _find_category (cCategory);
 	pItem->iStatus = _find_status (cStatus);
@@ -542,6 +567,7 @@ CDStatusNotifierItem *cd_satus_notifier_create_item (const gchar *cService, cons
 	pItem->cAttentionIconName = g_strdup (cAttentionIconName);
 	pItem->cAttentionMovieName = g_strdup (cAttentionMovieName);
 	pItem->cOverlayIconName = g_strdup (cOverlayIconName);
+	pItem->bItemIsMenu = bItemIsMenu;
 	if (pToolTipTab)
 	{
 		pItem->pToolTip = _make_tooltip_from_dbus_struct (pToolTipTab);
@@ -622,6 +648,7 @@ void cd_free_item (CDStatusNotifierItem *pItem)
 	g_free (pItem->cAttentionIconName);
 	g_free (pItem->cLabel);
 	g_free (pItem->cLabelGuide);
+	g_free (pItem->cAccessibleDesc);
 	g_free (pItem->cTitle);
 	g_free (pItem->cAttentionMovieName);
 	g_free (pItem->cOverlayIconName);
@@ -637,7 +664,7 @@ static void _load_item_image (Icon *icon)
 	int iHeight = icon->iImageHeight;
 	
 	CDStatusNotifierItem *pItem = cd_satus_notifier_get_item_from_icon (icon);
-	gchar *cIconPath = cd_satus_notifier_search_item_icon_s_path (pItem);
+	gchar *cIconPath = cd_satus_notifier_search_item_icon_s_path (pItem, MAX (iWidth, iHeight));
 	if (cIconPath != NULL && *cIconPath != '\0')
 		icon->pIconBuffer = cairo_dock_create_surface_from_image_simple (cIconPath,
 			iWidth,
