@@ -87,7 +87,7 @@ static inline CDStatusNotifierItem *_get_item (Icon *pClickedIcon, CairoContaine
 static inline gboolean _popup_menu (CDStatusNotifierItem *pItem, Icon *pIcon, CairoContainer *pContainer)
 {
 	gboolean r = FALSE;
-	if (pItem->cMenuPath != NULL)
+	if (pItem->cMenuPath != NULL && *pItem->cMenuPath != '\0' && strcmp (pItem->cMenuPath, "/NO_DBUSMENU") != 0)  // hopefully, if the item doesn't have a dbusmenu, it will not set something different as these 2 choices.
 	{
 		if (pItem->pMenu == NULL)
 			pItem->pMenu = dbusmenu_gtkmenu_new ((gchar *)pItem->cService, (gchar *)pItem->cMenuPath);
@@ -98,15 +98,17 @@ static inline gboolean _popup_menu (CDStatusNotifierItem *pItem, Icon *pIcon, Ca
 		}
 	}
 
-	if (!r)
+	if (!r)  // no menu available, send the corresponding action
 	{
 		r = _emit_click (pItem, pIcon, pContainer, "ContextMenu");
 	}
 
-	if (!r)
+	if (!r)  // no luck, try to fallback on 'activate()'
 	{
 		r = _emit_click (pItem, pIcon, pContainer, "Activate");
 	}
+	
+	return r;
 }
 
 CD_APPLET_ON_CLICK_BEGIN
@@ -114,19 +116,22 @@ CD_APPLET_ON_CLICK_BEGIN
 	//g_print ("click on item '%s'\n", pItem?pItem->cService:"none");
 	if (pItem != NULL)
 	{
-		if (myConfig.bMenuOnLeftClick)
+		// Ubuntu-like: show the menu on left click as the sole action (right-click = usual Cairo-Dock menu).
+		if (myConfig.bMenuOnLeftClick || pItem->bItemIsMenu)  // if bItemIsMenu: "The item only support the context menu, the visualization should prefer sending ContextMenu() instead of Activate()"
 		{
 			_popup_menu (pItem, CD_APPLET_CLICKED_ICON, CD_APPLET_CLICKED_CONTAINER);
 		}
-		else
+		else  // KDE-like: activate the item on left click, and show its menu on right-click.
 		{
-			gboolean r = _emit_click (pItem, CD_APPLET_CLICKED_ICON, CD_APPLET_CLICKED_CONTAINER, "Activate");
+			gboolean r;
+			r = _emit_click (pItem, CD_APPLET_CLICKED_ICON, CD_APPLET_CLICKED_CONTAINER, "Activate");
+			
 			if (!r)
 			{
 				if (pItem->cId != NULL)
 				{
 					/// TODO: try to get the icon in the taskbar, because launch the command doesn't raise the window if it was already visible (but it does pop up it if it was hidden, usually).
-					cairo_dock_launch_command (pItem->cId);  // lancer une nouvelle fois l'appli montre sa fenetre (enfin, generalement).
+					cairo_dock_launch_command (pItem->cId);  // try to launch the application because generally this click shows its item's window.
 				}
 			}
 		}
@@ -164,8 +169,7 @@ CD_APPLET_ON_SCROLL_END
 
 //\___________ Define here the entries you want to add to the menu when the user right-clicks on your icon or on its subdock or your desklet. The icon and the container that were clicked are available through the macros CD_APPLET_CLICKED_ICON and CD_APPLET_CLICKED_CONTAINER. CD_APPLET_CLICKED_ICON may be NULL if the user clicked in the container but out of icons. The menu where you can add your entries is available throught the macro CD_APPLET_MY_MENU; you can add sub-menu to it if you want.
 CD_APPLET_ON_BUILD_MENU_BEGIN
-	GtkWidget *pSubMenu = CD_APPLET_CREATE_MY_SUB_MENU ();
-		CD_APPLET_ADD_ABOUT_IN_MENU (pSubMenu);
+
 CD_APPLET_ON_BUILD_MENU_END
 
 
@@ -187,7 +191,7 @@ gboolean cd_status_notifier_on_right_click (CairoDockModuleInstance *myApplet, I
 }
 
 
-static gboolean _popup_tooltip (Icon *pIcon)
+/*static gboolean _popup_tooltip (Icon *pIcon)
 {
 	CDStatusNotifierItem *pItemData = CD_APPLET_GET_MY_ICON_DATA (pIcon);
 	if (pItemData != NULL && pItemData->pToolTip != NULL)
@@ -198,7 +202,7 @@ static gboolean _popup_tooltip (Icon *pIcon)
 		gchar *cIconPath = NULL;
 		if (pItemData->pToolTip->cIconName)
 		{
-			cIconPath = cairo_dock_search_icon_s_path (pItemData->pToolTip->cIconName);
+			cIconPath = cairo_dock_search_icon_s_path (pItemData->pToolTip->cIconName, cairo_dock_search_icon_size (GTK_ICON_SIZE_DND)); // dialog
 		}
 		
 		cairo_dock_show_temporary_dialog_with_icon (cText, pIcon, CAIRO_CONTAINER (myIcon->pSubDock), 3000, cIconPath ? cIconPath : "same icon");
@@ -207,7 +211,7 @@ static gboolean _popup_tooltip (Icon *pIcon)
 		pItemData->iSidPopupTooltip = 0;
 	}
 	return FALSE;
-}
+}*/
 gboolean cd_status_notifier_on_enter_icon (CairoDockModuleInstance *myApplet, Icon *pIcon, CairoDock *pDock, gboolean *bStartAnimation)
 {
 	if (pDock == myIcon->pSubDock && myIcon->pSubDock != NULL)
@@ -252,7 +256,13 @@ gboolean on_mouse_moved (CairoDockModuleInstance *myApplet, CairoContainer *pCon
 		if (pItem == NULL)
 			CD_APPLET_SET_NAME_FOR_MY_ICON (NULL);
 		else
-			CD_APPLET_SET_NAME_FOR_MY_ICON (pItem->cLabel ? pItem->cLabel : pItem->cTitle ? pItem->cTitle : pItem->cId);
+			CD_APPLET_SET_NAME_FOR_MY_ICON (
+				pItem->cAccessibleDesc && *pItem->cAccessibleDesc != '\0' ? pItem->cAccessibleDesc :
+				pItem->cLabel && *pItem->cLabel != '\0' ? pItem->cLabel :
+				pItem->cTitle && *pItem->cTitle != '\0' ? pItem->cTitle :
+				NULL);
+				// pItem->cId && *pItem->cId != '\0' ? pItem->cId :
+				// maybe better to not display cId, e.g: nm-applet ; dropbox-xxxx ; etc.
 		
 		if (myDock)
 			CAIRO_DOCK_REDRAW_MY_CONTAINER;
