@@ -403,7 +403,7 @@ void cd_sysmonitor_start_top_dialog (CairoDockModuleInstance *myApplet)
 	cairo_dock_remove_dialog_if_any (myIcon);
 	// build an interactive widget that will be used to display the top list.
 	gchar *cTitle = g_strdup_printf ("  [ Top %d ] :", myConfig.iNbDisplayedProcesses);
-	GtkWidget *pInteractiveWidget = gtk_vbox_new (FALSE, 0);
+	GtkWidget *pInteractiveWidget = _gtk_vbox_new (0);
 	gtk_widget_set_size_request (pInteractiveWidget,
 		myConfig.pTopTextDescription->iSize * 15,
 		myConfig.pTopTextDescription->iSize * myConfig.iNbDisplayedProcesses);  // approximatif au depart.
