@@ -22,7 +22,7 @@
 #include <cairo-dock.h>
 
 #include "applet-struct.h"
-#include "applet-mixer.h"
+#include "applet-backend-alsamixer.h"  // mixer_get_elements_list
 #include "applet-notifications.h"
 #include "applet-config.h"
 
@@ -77,8 +77,6 @@ CD_APPLET_RESET_CONFIG_BEGIN
 	g_free (myConfig.cMixerElementName);
 	g_free (myConfig.cMixerElementName2);
 	g_free (myConfig.cShowAdvancedMixerCommand);
-	if (myConfig.cShortcut)
-		cd_keybinder_unbind(myConfig.cShortcut, (CDBindkeyHandler) mixer_on_keybinding_pull);
 	g_free (myConfig.cShortcut);
 	g_free (myConfig.cDefaultIcon);
 	g_free (myConfig.cBrokenIcon);
@@ -93,14 +91,10 @@ CD_APPLET_RESET_DATA_BEGIN
 		gtk_widget_destroy (myData.pScale);
 		myData.pScale = NULL;
 	}
-	mixer_stop ();
 	cairo_surface_destroy (myData.pSurface);
 	if (myData.pMuteSurface)
 		cairo_surface_destroy (myData.pMuteSurface);
 	cairo_dock_dialog_unreference (myData.pDialog);
-	g_free (myData.cErrorMessage);
-	g_free (myData.mixer_card_name);
-	g_free (myData.mixer_device_name);
 CD_APPLET_RESET_DATA_END
 
 
