@@ -24,7 +24,6 @@
 #include <math.h>
 
 #include "applet-struct.h"
-//#include "applet-digital.h" //Digital html like renderer
 #include "applet-draw.h"
 
 #define CD_CLOCK_DATE_BUFFER_LENGTH 50
@@ -37,6 +36,7 @@ static char s_cDateBuffer[CD_CLOCK_DATE_BUFFER_LENGTH+1];
 void cd_clock_draw_text (CairoDockModuleInstance *myApplet, int iWidth, int iHeight, struct tm *pTime)
 {
 	g_return_if_fail (myDrawContext != NULL);
+	//\______________ On efface le dessin courant.
 	cairo_dock_erase_cairo_context (myDrawContext);
 	if (myData.pNumericBgSurface != NULL)
 	{
@@ -45,8 +45,10 @@ void cd_clock_draw_text (CairoDockModuleInstance *myApplet, int iWidth, int iHei
 	}
 	cairo_set_source_rgba (myDrawContext, myConfig.fTextColor[0], myConfig.fTextColor[1], myConfig.fTextColor[2], myConfig.fTextColor[3]);
 	
+	//\______________ On defini le texte a dessiner.
+	// layout
 	PangoFontDescription *pDesc = pango_font_description_new ();
-	pango_font_description_set_absolute_size (pDesc, myIcon->fHeight * PANGO_SCALE);
+	pango_font_description_set_absolute_size (pDesc, myIcon->fHeight * 72 / myData.fDpi * PANGO_SCALE); // pixel converted to point, converted to pango dimension.
 	pango_font_description_set_family_static (pDesc, myConfig.cFont);
 	pango_font_description_set_weight (pDesc, myConfig.iWeight);
 	pango_font_description_set_style (pDesc, myConfig.iStyle);
@@ -54,6 +56,7 @@ void cd_clock_draw_text (CairoDockModuleInstance *myApplet, int iWidth, int iHei
 	PangoLayout *pLayout = pango_cairo_create_layout (myDrawContext);
 	pango_layout_set_font_description (pLayout, pDesc);
 	
+	// format de l'heure
 	const gchar *cFormat;
 	if (myConfig.b24Mode)
 	{
@@ -72,9 +75,10 @@ void cd_clock_draw_text (CairoDockModuleInstance *myApplet, int iWidth, int iHei
 	
 	strftime (s_cDateBuffer, CD_CLOCK_DATE_BUFFER_LENGTH, cFormat, pTime);
 	pango_layout_set_text (pLayout, s_cDateBuffer, -1);
-	PangoRectangle ink, log;
-	pango_layout_get_pixel_extents (pLayout, &ink, &log);
+	PangoRectangle log;
+	pango_layout_get_pixel_extents (pLayout, NULL, &log);
 	
+	//\______________ On dessine le texte.
 	cairo_save (myDrawContext);
 	if (myConfig.iShowDate == CAIRO_DOCK_INFO_ON_ICON)
 	{
@@ -83,45 +87,46 @@ void cd_clock_draw_text (CairoDockModuleInstance *myApplet, int iWidth, int iHei
 		
 		strftime (s_cDateBuffer, CD_CLOCK_DATE_BUFFER_LENGTH, "%a %d %b", pTime);
 		pango_layout_set_text (pLayout2, s_cDateBuffer, -1);
-		PangoRectangle ink2, log2;
-		pango_layout_get_pixel_extents (pLayout2, &ink2, &log2);
+		PangoRectangle log2;
+		pango_layout_get_pixel_extents (pLayout2, NULL, &log2);
 		
-		double h=0, w=0, fZoomX=0, fZoomY=0;
-		double h_=0, w_=0, fZoomX_=0, fZoomY_=0;
+		double h=0, w=0, fZoomX=0, fZoomY=0;  // parametres d'affichage 2 lignes
+		double h_=0, w_=0, fZoomX_=0, fZoomY_=0;  // parametres d'affichage 1 ligne
 		if (myData.iTextLayout == CD_TEXT_LAYOUT_2_LINES || myData.iTextLayout == CD_TEXT_LAYOUT_AUTO)
 		{
-			h = ink.height + ink2.height + GAPY * iHeight;
-			w = MAX (ink.width, ink2.width);
+			h = log.height + log2.height + GAPY * iHeight;
+			w = MAX (log.width, log2.width);
 			fZoomX = (double) iWidth / w;
 			fZoomY = (double) iHeight / h;
-			if (myDock && fZoomY > MAX_RATIO * fZoomX)  // on ne garde pas le ratio car ca ferait un texte trop petit en hauteur, toutefois on limite un peu la deformation en hauteur.
+			/**if (myDock && fZoomY > MAX_RATIO * fZoomX)  // we limit the deformation
 				fZoomY = MAX_RATIO * fZoomX;
 			
-			if (fZoomX * w > myConfig.fTextRatio * iWidth)
-			{
-				fZoomY *= myConfig.fTextRatio * iWidth / w * fZoomX;
-				fZoomX = myConfig.fTextRatio * iWidth / w;
-			}
+			if (myConfig.fTextRatio < 1)
+				fZoomY *= myConfig.fTextRatio;*/
+			// keep the ratio of the text
+			fZoomX = MIN (fZoomX, fZoomY) * myConfig.fTextRatio;
+			fZoomY = fZoomX;
 		}
 		if (myData.iTextLayout == CD_TEXT_LAYOUT_1_LINE || myData.iTextLayout == CD_TEXT_LAYOUT_AUTO)
 		{
-			h_ = MAX (ink.height, ink2.height);
-			w_ = ink.width + ink2.width + GAPX * iWidth;
+			h_ = MAX (log.height, log2.height);
+			w_ = log.width + log2.width + GAPX * iWidth;
 			fZoomX_ = (double) iWidth / w_;
 			fZoomY_ = (double) iHeight / h_;
-			if (myDock && fZoomY_ > MAX_RATIO * fZoomX_)  // on ne garde pas le ratio car ca ferait un texte trop petit en hauteur, toutefois on limite un peu la deformation en hauteur.
+			/**if (myDock && fZoomY_ > MAX_RATIO * fZoomX_)  // we limit the deformation
 				fZoomY_ = MAX_RATIO * fZoomX_;
 			
-			if (fZoomX_ * w_ > myConfig.fTextRatio * iWidth)
-			{
-				fZoomY_ *= myConfig.fTextRatio * iWidth / w_ * fZoomX_;
-				fZoomX_ = myConfig.fTextRatio * iWidth / w_;
-			}
+			if (myConfig.fTextRatio < 1)
+				fZoomY_ *= myConfig.fTextRatio;
+			
 			if (fZoomY_ > fZoomX_)
 			{
-				double fMaxScale = cairo_dock_get_max_scale (myContainer);
+				double fMaxScale = cairo_dock_get_icon_max_scale (myIcon);
 				fZoomY_ = MAX (fZoomX_, 16. * fMaxScale / h_);  // en mode horizontal, on n'a pas besoin que le texte remplisse toute la hauteur de l'icone. 16 pixels de haut sont suffisant pour etre lisible.
-			}
+			}*/
+			// keep the ratio of the text
+			fZoomX_ = MIN (fZoomX_, fZoomY_) * myConfig.fTextRatio;
+			fZoomY_ = fZoomX_;
 		}
 		
 		if (myData.iTextLayout == CD_TEXT_LAYOUT_AUTO)  // si l'orientation n'est pas encore definie, on la definit de facon a ne pas changer (si on est tres proche de la limite, la taille du texte pourrait changer suffisamment pour nous faire passer d'une orientation a l'autre.
@@ -136,52 +141,49 @@ void cd_clock_draw_text (CairoDockModuleInstance *myApplet, int iWidth, int iHei
 		
 		if (myData.iTextLayout == CD_TEXT_LAYOUT_1_LINE)  // mode 1 ligne
 		{
-			cairo_translate (myDrawContext, (iWidth - fZoomX_ * w_) / 2, (iHeight - fZoomY_ * h_)/2);  // centre verticalement.
+			cairo_translate (myDrawContext, (iWidth - fZoomX_ * w_) / 2, (iHeight - fZoomY_ * h_)/2);  // text will be centred.
 			cairo_scale (myDrawContext, fZoomX_, fZoomY_);
-			cairo_translate (myDrawContext, -ink2.x, -ink2.y);
 			pango_cairo_show_layout (myDrawContext, pLayout2);
 			
 			cairo_restore (myDrawContext);
 			cairo_save (myDrawContext);
 			
-			cairo_translate (myDrawContext, (iWidth + fZoomX_ * w_) / 2 - fZoomX_ * ink.width, (iHeight - fZoomY_ * h_)/2);
+			cairo_translate (myDrawContext, (iWidth + fZoomX_ * w_) / 2 - fZoomX_ * log.width, (iHeight - fZoomY_ * h_)/2);
 			cairo_scale (myDrawContext, fZoomX_, fZoomY_);
-			cairo_translate (myDrawContext, -ink.x, -ink.y);
 			pango_cairo_show_layout (myDrawContext, pLayout);
 		}
 		else  // mode 2 lignes
 		{
-			cairo_translate (myDrawContext, (iWidth - fZoomX * ink.width) / 2, (iHeight - fZoomY * h)/2);  // centre verticalement.
+			cairo_translate (myDrawContext, (iWidth - fZoomX * log.width) / 2, (iHeight - fZoomY * h)/2);  // text will be centred.
 			cairo_scale (myDrawContext, fZoomX, fZoomY);
-			cairo_translate (myDrawContext, -ink.x, -ink.y);
 			pango_cairo_show_layout (myDrawContext, pLayout);
 			
 			cairo_restore (myDrawContext);
 			cairo_save (myDrawContext);
 			
-			cairo_translate (myDrawContext, (iWidth - fZoomX * ink2.width) / 2, (iHeight + fZoomY * GAPY)/2);
+			cairo_translate (myDrawContext, (iWidth - fZoomX * log2.width) / 2, (iHeight + fZoomY * GAPY)/2);
 			cairo_scale (myDrawContext, fZoomX, fZoomY);
-			cairo_translate (myDrawContext, -ink2.x, -ink2.y);
 			pango_cairo_show_layout (myDrawContext, pLayout2);
 		}
 		g_object_unref (pLayout2);
 	}
-	else
+	else  // only the hour with 1 line.
 	{
-		double fZoomX = (double) (iWidth - 1) / ink.width;
-		double fZoomY = (double) iHeight / ink.height;
-		if (myDock && fZoomY > MAX_RATIO * fZoomX)  // on ne garde pas le ratio car ca ferait un texte trop petit en hauteur, toutefois on limite un peu la deformation en hauteur.
+		double fZoomX = (double) iWidth / log.width;
+		double fZoomY = (double) iHeight / log.height;
+		/**if (myDock && fZoomY > MAX_RATIO * fZoomX)  // we limit the deformation
 			fZoomY = MAX_RATIO * fZoomX;
 		
-		if (fZoomX * ink.width > myConfig.fTextRatio * iWidth)
-		{
-			fZoomY *= myConfig.fTextRatio * iWidth / (ink.width * fZoomX);
-			fZoomX = myConfig.fTextRatio * iWidth / ink.width;
-		}
+		if (myConfig.fTextRatio < 1)
+			fZoomY *= myConfig.fTextRatio;*/
+		// keep the ratio of the text
+		fZoomX = MIN (fZoomX, fZoomY) * myConfig.fTextRatio;
+		fZoomY = fZoomX;
 		
-		cairo_translate (myDrawContext, 0., (iHeight - fZoomY * ink.height)/2);  // centre verticalement.
+		cairo_translate (myDrawContext,
+			(iWidth - fZoomX * log.width)/2,
+			(iHeight - fZoomY * log.height)/2);  // text will be centred.
 		cairo_scale (myDrawContext, fZoomX, fZoomY);
-		cairo_translate (myDrawContext, -ink.x, -ink.y);
 		pango_cairo_show_layout (myDrawContext, pLayout);
 	}
 	cairo_restore (myDrawContext);
