@@ -153,7 +153,7 @@ static gboolean _on_click_tree_view (GtkTreeView *pTreeView, GdkEventButton* pBu
 	if (pButton->button == 3 && pButton->type == GDK_BUTTON_RELEASE)
 	{
 		GtkWidget *pMenu = gtk_menu_new ();
-		cairo_dock_add_in_menu_with_stock_and_data (D_("Add a new task"), GTK_STOCK_ADD, (GFunc)_cd_clock_add_new_task, pMenu, myApplet);
+		cairo_dock_add_in_menu_with_stock_and_data (D_("Add a new task"), GTK_STOCK_ADD, G_CALLBACK (_cd_clock_add_new_task), pMenu, myApplet);
 		
 		GtkTreeSelection *pSelection = gtk_tree_view_get_selection (pTreeView);
 		GtkTreeModel *pModel;
@@ -174,7 +174,7 @@ static gboolean _on_click_tree_view (GtkTreeView *pTreeView, GdkEventButton* pBu
 				data[0] = myApplet;
 				data[1] = pTask;
 				data[2] = pModel;
-				cairo_dock_add_in_menu_with_stock_and_data (D_("Delete this task"), GTK_STOCK_REMOVE, (GFunc)_cd_clock_delete_task, pMenu, data);
+				cairo_dock_add_in_menu_with_stock_and_data (D_("Delete this task"), GTK_STOCK_REMOVE, G_CALLBACK (_cd_clock_delete_task), pMenu, data);
 			}
 		}
 		gtk_widget_show_all (pMenu);
@@ -377,6 +377,7 @@ void cd_clock_build_task_editor (guint iDay, guint iMonth, guint iYear, CairoDoc
 	if (myData.pTaskWindow == NULL)
 	{
 		myData.pTaskWindow = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+		gtk_window_set_modal (GTK_WINDOW (myData.pTaskWindow), TRUE);  // set as modal, since the dialog it comes from (the one containing the calendar) is modal (as any interactive dialog).
 		
 		//\______________ On construit le treeview.
 		GtkWidget *pTreeView = gtk_tree_view_new ();
