@@ -25,7 +25,6 @@
 
 #include "applet-struct.h"
 #include "applet-config.h"
-#include "applet-digital.h" //Digital html like renderer
 #include "applet-calendar.h"
 #include "applet-draw.h"
 #include "applet-timer.h"
@@ -45,13 +44,17 @@ static gboolean _task_warning (CDClockTask *pTask, const gchar *cMessage)
 {
 	cd_debug ("%s (%s)", __func__, cMessage);
 	CairoDockModuleInstance *myApplet = pTask->pApplet;
-	
+
+	#if (GTK_MAJOR_VERSION < 3)
 	GtkWidget *pScale = gtk_hscale_new_with_range (1, 60, 1);  // 1mn-60mn et 1 cran/mn.
+	#else
+	GtkWidget *pScale = gtk_scale_new_with_range (GTK_ORIENTATION_HORIZONTAL, 1, 60, 1);  // 1mn-60mn et 1 cran/mn.
+	#endif
 	gtk_scale_set_digits (GTK_SCALE (pScale), 0);
 	gtk_range_set_value (GTK_RANGE (pScale), pTask->iWarningDelay != 0 ? pTask->iWarningDelay : 15);  // 15mn par defaut.
-	gtk_widget_set (pScale, "width-request", CAIRO_DIALOG_MIN_SCALE_WIDTH, NULL);
-	
-	GtkWidget *pExtendedWidget = gtk_hbox_new (FALSE, 0);
+	g_object_set (pScale, "width-request", CAIRO_DIALOG_MIN_SCALE_WIDTH, NULL);
+
+	GtkWidget *pExtendedWidget = _gtk_hbox_new (0);
 	GtkWidget *label = gtk_label_new (D_("1mn"));
 	GtkWidget *pAlign = gtk_alignment_new (1., 1., 0., 0.);
 	gtk_container_add (GTK_CONTAINER (pAlign), label);
@@ -273,8 +276,7 @@ gboolean cd_clock_update_with_time (CairoDockModuleInstance *myApplet)
 				&labelDescription,
 				fScale,
 				iWidth,
-				&myData.iDateWidth, &myData.iDateHeight,
-				NULL, NULL);
+				&myData.iDateWidth, &myData.iDateHeight);
 			myData.iDateWidth *= fScale;
 			myData.iDateHeight *= fScale;
 			//g_print ("date : %dx%d\n", myData.iDateWidth, myData.iDateHeight);
@@ -303,10 +305,10 @@ gboolean cd_clock_update_with_time (CairoDockModuleInstance *myApplet)
 			cairo_dock_update_icon_texture (myIcon);
 	}
 	
-	if (myDock && ! CD_APPLET_MY_CONTAINER_IS_OPENGL)  // les reflets pour cairo.
+	/**if (myDock && ! CD_APPLET_MY_CONTAINER_IS_OPENGL)  // les reflets pour cairo.
 	{
 		CD_APPLET_UPDATE_REFLECT_ON_MY_ICON;
-	}
+	}*/
 	
 	CD_APPLET_REDRAW_MY_ICON;
 	
