@@ -160,6 +160,8 @@ static void calculate_icons (CairoDesklet *pDesklet)
 	{
 		pIcon->fWidth = pPanel->iMainIconSize;
 		pIcon->fHeight = pPanel->iMainIconSize;
+		pIcon->iImageWidth = pIcon->fWidth;
+		pIcon->iImageHeight = pIcon->fHeight;
 		
 		pIcon->fScale = 1.;
 		pIcon->fAlpha = 1.;
@@ -193,6 +195,8 @@ static void calculate_icons (CairoDesklet *pDesklet)
 		{
 			pIcon->fWidth = pPanel->iIconSize;
 			pIcon->fHeight = pPanel->iIconSize;
+			pIcon->iImageWidth = pIcon->fWidth;
+			pIcon->iImageHeight = pIcon->fHeight;
 		
 			pIcon->fScale = 1.;
 			pIcon->fAlpha = 1.;
@@ -332,7 +336,6 @@ static void render (cairo_t *pCairoContext, CairoDesklet *pDesklet)
 				cairo_paint (pCairoContext);
 			else
 				cairo_paint_with_alpha (pCairoContext, pIcon->fAlpha);
-			///cairo_dock_render_one_icon_in_desklet (pIcon, pCairoContext, FALSE, FALSE, pDesklet->container.iWidth);
 			
 			cairo_restore (pCairoContext);
 			
