@@ -27,7 +27,6 @@
 #include <cairo.h>
 
 #ifdef HAVE_GLITZ
-#include <gdk/gdkx.h>
 #include <glitz-glx.h>
 #include <cairo-glitz.h>
 #endif
@@ -125,8 +124,8 @@ void cd_rendering_render_diapo (cairo_t *pCairoContext, CairoDock *pDock)
         
 	//\____________________ On dessine la ficelle qui les joint.
 	//TODO Rendre joli !
-	if (myIcons.iStringLineWidth > 0)
-		cairo_dock_draw_string (pCairoContext, pDock, myIcons.iStringLineWidth, FALSE, FALSE);
+	if (myIconsParam.iStringLineWidth > 0)
+		cairo_dock_draw_string (pCairoContext, pDock, myIconsParam.iStringLineWidth, FALSE, FALSE);
 	//\____________________ On dessine les icones avec leurs etiquettes.
 	
 	
