@@ -31,12 +31,17 @@
 extern gdouble my_fPanelRadius;
 extern gdouble my_fPanelInclination;
 extern gdouble my_fPanelRatio;
+extern gboolean my_bPanelPhysicalSeparator;
 const int iNbCurveSteps = 10;
 
+typedef struct {
+	gdouble fGroupGap;  // gap between 2 groups of icons (= width of a separator), at rest.
+	} CDPanelData;
+
 static void cd_compute_size (CairoDock *pDock)
 {
 	//\_____________ On calcule le nombre de groupes et la place qu'ils occupent.
-	int iNbGroups = 1, iCurrentOrder = -1;
+	int iNbGroups = 1;
 	double fCurrentGroupWidth = - myIconsParam.iIconGap, fGroupsWidth = 0.;
 	GList *ic;
 	Icon *pIcon;
@@ -44,20 +49,6 @@ static void cd_compute_size (CairoDock *pDock)
 	{
 		pIcon = ic->data;
 		if (CAIRO_DOCK_ICON_TYPE_IS_SEPARATOR (pIcon))
-		{
-			if (CAIRO_DOCK_IS_USER_SEPARATOR (pIcon))  // si c'est un separateur automatique, le changement de groupe incrementera le compteur a l'icone suivante.
-			{
-				if (fCurrentGroupWidth > 0)  // le groupe courant est non vide, sinon c'est juste 2 separateurs cote a cote.
-				{
-					iNbGroups ++;
-					fGroupsWidth += MAX (0, fCurrentGroupWidth);
-					//g_print ("fGroupsWidth += %.2f\n", fCurrentGroupWidth);
-					fCurrentGroupWidth = - myIconsParam.iIconGap;
-				}
-			}
-			continue;
-		}
-		if (iCurrentOrder != (int)cairo_dock_get_icon_order (pIcon))
 		{
 			if (fCurrentGroupWidth > 0)  // le groupe courant est non vide, sinon c'est juste 2 separateurs cote a cote.
 			{
@@ -66,9 +57,10 @@ static void cd_compute_size (CairoDock *pDock)
 				//g_print ("fGroupsWidth += %.2f\n", fCurrentGroupWidth);
 				fCurrentGroupWidth = - myIconsParam.iIconGap;
 			}
+			
+			continue;
 		}
-		iCurrentOrder = cairo_dock_get_icon_order (pIcon);
-		fCurrentGroupWidth += pIcon->fWidth * my_fPanelRatio + myIconsParam.iIconGap;
+		fCurrentGroupWidth += pIcon->fWidth/** * my_fPanelRatio*/ + myIconsParam.iIconGap;
 		//g_print ("fCurrentGroupWidth <- %.2f\n", fCurrentGroupWidth);
 	}
 	if (fCurrentGroupWidth > 0)  // le groupe courant est non vide, sinon c'est juste un separateur a la fin.
@@ -90,33 +82,20 @@ static void cd_compute_size (CairoDock *pDock)
 	//\_____________ On calcule la position au repos des icones et la taille du dock.
 	double xg = fScreenBorderGap;  // abscisse de l'icone courante, et abscisse du debut du groupe courant.
 	double x = xg;
-	if (iNbGroups == 1 && pDock->icons)  // if the first icon is a separator, place icons to the right.
+	/*if (iNbGroups == 1 && pDock->icons)  // if the first icon is a separator, place icons to the right.
 	{
 		pIcon = pDock->icons->data;
 		if (CAIRO_DOCK_ICON_TYPE_IS_SEPARATOR (pIcon))
 			x += fGroupGap;
-	}
+	}*/
 	fCurrentGroupWidth = - myIconsParam.iIconGap;
-	iCurrentOrder = -1;
 	for (ic = pDock->icons; ic != NULL; ic = ic->next)
 	{
 		pIcon = ic->data;
 		if (CAIRO_DOCK_ICON_TYPE_IS_SEPARATOR (pIcon))
 		{
-			if (CAIRO_DOCK_IS_USER_SEPARATOR (pIcon))  // si c'est un separateur automatique, le changement de groupe incrementera le compteur a l'icone suivante.
-			{
-				if (fCurrentGroupWidth > 0)  // le groupe courant est non vide, sinon c'est juste 2 separateurs cote a cote.
-				{
-					xg += fCurrentGroupWidth + fGroupGap;
-					x = xg;
-					//g_print ("jump to %.2f\n", x);
-					fCurrentGroupWidth = - myIconsParam.iIconGap;
-				}
-			}
-			continue;
-		}
-		if (iCurrentOrder != (int)cairo_dock_get_icon_order (pIcon))
-		{
+			pIcon->fXAtRest = x;
+			//g_print ("*** separator at %d\n", (int)x);
 			if (fCurrentGroupWidth > 0)  // le groupe courant est non vide, sinon c'est juste 2 separateurs cote a cote.
 			{
 				xg += fCurrentGroupWidth + fGroupGap;
@@ -124,20 +103,18 @@ static void cd_compute_size (CairoDock *pDock)
 				//g_print ("jump to %.2f\n", x);
 				fCurrentGroupWidth = - myIconsParam.iIconGap;
 			}
+			
+			continue;
 		}
-		iCurrentOrder = cairo_dock_get_icon_order (pIcon);
-		fCurrentGroupWidth += pIcon->fWidth * my_fPanelRatio + myIconsParam.iIconGap;
+		fCurrentGroupWidth += pIcon->fWidth/** * my_fPanelRatio*/ + myIconsParam.iIconGap;
 		
-		//g_print ("icon at %.2f\n", x);
 		pIcon->fXAtRest = x;
-		x += pIcon->fWidth * my_fPanelRatio + myIconsParam.iIconGap;
+		x += pIcon->fWidth/** * my_fPanelRatio*/ + myIconsParam.iIconGap;
 	}
 	
 	pDock->fMagnitudeMax = 0.;  // pas de vague.
 	
-	pDock->pFirstDrawnElement = pDock->icons;
-	
-	double hicon = pDock->iMaxIconHeight * my_fPanelRatio;
+	double hicon = pDock->iMaxIconHeight/** * my_fPanelRatio*/;
 	pDock->iDecorationsHeight = hicon * pDock->container.fRatio + 2 * myDocksParam.iFrameMargin;
 	
 	pDock->iMaxDockWidth = pDock->fFlatDockWidth = pDock->iMinDockWidth = MAX (W, x);
@@ -155,6 +132,14 @@ static void cd_compute_size (CairoDock *pDock)
 	pDock->iActiveHeight = pDock->iMinDockHeight;
 	if (! pDock->container.bIsHorizontal && myIconsParam.bTextAlwaysHorizontal)
 		pDock->iMaxDockHeight += 8*myIconsParam.iLabelSize;  // vertical dock, add some padding to draw the labels
+	
+	CDPanelData *pData = pDock->pRendererData;
+	if (pData == NULL)
+	{
+		pData = g_new0 (CDPanelData, 1);
+		pDock->pRendererData = pData;
+	}
+	pData->fGroupGap = fGroupGap;
 }
 
 
@@ -176,7 +161,7 @@ static void cd_render (cairo_t *pCairoContext, CairoDock *pDock)
 	else
 	{
 		fDockWidth = cairo_dock_get_current_dock_width_linear (pDock);
-		Icon *pFirstIcon = cairo_dock_get_first_drawn_icon (pDock);
+		Icon *pFirstIcon = cairo_dock_get_first_icon (pDock->icons);
 		fDockOffsetX = (pFirstIcon != NULL ? pFirstIcon->fX - fMargin : fExtraWidth / 2);
 		if (fDockOffsetX < fExtraWidth / 2)
 			fDockOffsetX = fExtraWidth / 2;
@@ -195,7 +180,7 @@ static void cd_render (cairo_t *pCairoContext, CairoDock *pDock)
 	}
 
 	cairo_save (pCairoContext);
-	double fDeltaXTrapeze = cairo_dock_draw_frame (pCairoContext, fRadius, fLineWidth, fDockWidth, pDock->iDecorationsHeight, fDockOffsetX, fDockOffsetY, sens, 0., pDock->container.bIsHorizontal, myDocksParam.bRoundedBottomCorner);
+	double fDeltaXTrapeze = cairo_dock_draw_frame (pCairoContext, fRadius, fLineWidth, fDockWidth, pDock->iDecorationsHeight, fDockOffsetX, fDockOffsetY, sens, 0., pDock->container.bIsHorizontal, FALSE);  // FALSE <=> ignore 'myDocksParam.bRoundedBottomCorner', as bottom rounded corners looks bad for a panel.
 
 	//\____________________ On dessine les decorations dedans.
 	fDockOffsetY = (pDock->container.bDirectionUp ? pDock->container.iHeight - pDock->iDecorationsHeight - fLineWidth : fLineWidth);
@@ -213,93 +198,96 @@ static void cd_render (cairo_t *pCairoContext, CairoDock *pDock)
 	cairo_restore (pCairoContext);
 	
 	//\____________________ On dessine les separateurs physiques.
-	cairo_save (pCairoContext);
-	if (pDock->container.bIsHorizontal)
-	{
-		if (! pDock->container.bDirectionUp)
-		{
-			cairo_translate (pCairoContext, 0., pDock->container.iHeight);
-			cairo_scale (pCairoContext, 1., -1.);
-		}
-	}
-	else
-	{
-		cairo_translate (pCairoContext, pDock->container.iHeight/2., pDock->container.iWidth/2.);
-		cairo_rotate (pCairoContext, G_PI/2);
-		if (pDock->container.bDirectionUp)
-			cairo_scale (pCairoContext, 1., -1.);
-		cairo_translate (pCairoContext, -pDock->container.iWidth/2., -pDock->container.iHeight/2.);
-	}
-	
-	double x1, x2, dx, delta, h = pDock->iDecorationsHeight + 2*fLineWidth, h_ = h - fLineWidth;
 	GList *ic;
 	Icon *pIcon;
-	for (ic = pDock->icons; ic != NULL; ic = ic->next)
+	if (my_bPanelPhysicalSeparator)
 	{
-		pIcon = ic->data;
-		if (CAIRO_DOCK_ICON_TYPE_IS_SEPARATOR (pIcon))
+		cairo_save (pCairoContext);
+		if (pDock->container.bIsHorizontal)
 		{
-			x1 = pIcon->fDrawX = pIcon->fX;
-			
-			pIcon = NULL;
-			for (ic = ic->next; ic != NULL; ic = ic->next)
+			if (! pDock->container.bDirectionUp)
 			{
-				pIcon = ic->data;
-				if (!CAIRO_DOCK_ICON_TYPE_IS_SEPARATOR (pIcon))
+				cairo_translate (pCairoContext, 0., pDock->container.iHeight);
+				cairo_scale (pCairoContext, 1., -1.);
+			}
+		}
+		else
+		{
+			cairo_translate (pCairoContext, pDock->container.iHeight/2., pDock->container.iWidth/2.);
+			cairo_rotate (pCairoContext, G_PI/2);
+			if (pDock->container.bDirectionUp)
+				cairo_scale (pCairoContext, 1., -1.);
+			cairo_translate (pCairoContext, -pDock->container.iWidth/2., -pDock->container.iHeight/2.);
+		}
+		
+		double x1, x2, dx, delta, h = pDock->iDecorationsHeight + 2*fLineWidth, h_ = h - fLineWidth;
+		for (ic = pDock->icons; ic != NULL; ic = ic->next)
+		{
+			pIcon = ic->data;
+			if (CAIRO_DOCK_ICON_TYPE_IS_SEPARATOR (pIcon))
+			{
+				x1 = pIcon->fDrawX = pIcon->fX;
+				
+				pIcon = NULL;
+				for (ic = ic->next; ic != NULL; ic = ic->next)
+				{
+					pIcon = ic->data;
+					if (!CAIRO_DOCK_ICON_TYPE_IS_SEPARATOR (pIcon))
+						break;
+				}
+				if (ic != NULL)
+				{
+					pIcon = ic->data;
+					x2 = pIcon->fDrawX;
+				}
+				else
 					break;
-			}
-			if (ic != NULL)
-			{
-				pIcon = ic->data;
-				x2 = pIcon->fDrawX;
-			}
-			else
-				break;
-			
-			dx = MIN (my_fPanelRadius, (x2 - x1) / 2);
-			delta = dx + h*tan(my_fPanelInclination)/2;
-			if (delta > (x2 - x1) / 2)
-				delta = (x2 - x1) / 2;
-			
-			cairo_move_to (pCairoContext, x1, pDock->iMaxDockHeight - h);
-			cairo_rel_curve_to (pCairoContext,
-				dx, 0.,
-				delta - dx, h,
-				delta, h);
-			cairo_rel_line_to (pCairoContext,
-				x2 - x1 - 2*delta, 0.);
-			cairo_rel_curve_to (pCairoContext,
-				dx, 0.,
-				delta - dx, -h,
-				delta, -h);
-			cairo_close_path (pCairoContext);
-			
-			cairo_set_operator (pCairoContext, CAIRO_OPERATOR_DEST_OUT);
-			cairo_set_source_rgba (pCairoContext, 0.0, 0.0, 0.0, 1.0);
-			cairo_fill (pCairoContext);
-			
-			if (fLineWidth > 0)
-			{
-				cairo_move_to (pCairoContext, x1, pDock->iMaxDockHeight - h_ - fLineWidth/2);
+				
+				dx = MIN (my_fPanelRadius, (x2 - x1) / 2);
+				delta = dx + h*tan(my_fPanelInclination)/2;
+				if (delta > (x2 - x1) / 2)
+					delta = (x2 - x1) / 2;
+				
+				cairo_move_to (pCairoContext, x1, pDock->iMaxDockHeight - h);
 				cairo_rel_curve_to (pCairoContext,
 					dx, 0.,
-					delta - dx, h_,
-					delta, h_);
+					delta - dx, h,
+					delta, h);
 				cairo_rel_line_to (pCairoContext,
 					x2 - x1 - 2*delta, 0.);
 				cairo_rel_curve_to (pCairoContext,
 					dx, 0.,
-					delta - dx, -h_,
-					delta, -h_);
+					delta - dx, -h,
+					delta, -h);
+				cairo_close_path (pCairoContext);
 				
-				cairo_set_operator (pCairoContext, CAIRO_OPERATOR_OVER);
-				cairo_set_line_width (pCairoContext, fLineWidth);
-				cairo_set_source_rgba (pCairoContext, myDocksParam.fLineColor[0], myDocksParam.fLineColor[1], myDocksParam.fLineColor[2], myDocksParam.fLineColor[3]);
-				cairo_stroke (pCairoContext);
+				cairo_set_operator (pCairoContext, CAIRO_OPERATOR_DEST_OUT);
+				cairo_set_source_rgba (pCairoContext, 0.0, 0.0, 0.0, 1.0);
+				cairo_fill (pCairoContext);
+				
+				if (fLineWidth > 0)
+				{
+					cairo_move_to (pCairoContext, x1, pDock->iMaxDockHeight - h_ - fLineWidth/2);
+					cairo_rel_curve_to (pCairoContext,
+						dx, 0.,
+						delta - dx, h_,
+						delta, h_);
+					cairo_rel_line_to (pCairoContext,
+						x2 - x1 - 2*delta, 0.);
+					cairo_rel_curve_to (pCairoContext,
+						dx, 0.,
+						delta - dx, -h_,
+						delta, -h_);
+					
+					cairo_set_operator (pCairoContext, CAIRO_OPERATOR_OVER);
+					cairo_set_line_width (pCairoContext, fLineWidth);
+					cairo_set_source_rgba (pCairoContext, myDocksParam.fLineColor[0], myDocksParam.fLineColor[1], myDocksParam.fLineColor[2], myDocksParam.fLineColor[3]);
+					cairo_stroke (pCairoContext);
+				}
 			}
 		}
+		cairo_restore (pCairoContext);
 	}
-	cairo_restore (pCairoContext);
 	
 	//\____________________ On dessine la ficelle qui les joint.
 	if (myIconsParam.iStringLineWidth > 0)
@@ -366,7 +354,7 @@ static void cd_render_optimized (cairo_t *pCairoContext, CairoDock *pDock, GdkRe
 	}
 	else
 	{
-		Icon *pFirstIcon = cairo_dock_get_first_drawn_icon (pDock);
+		Icon *pFirstIcon = cairo_dock_get_first_icon (pDock->icons);
 		fOffsetX = (pFirstIcon != NULL ? pFirstIcon->fX - fMargin : fRadius + fLineWidth / 2);
 	}
 	double fDockWidth = cairo_dock_get_current_dock_width_linear (pDock);
@@ -411,7 +399,7 @@ static void cd_render_optimized (cairo_t *pCairoContext, CairoDock *pDock, GdkRe
 	//\____________________ On dessine les icones impactees.
 	cairo_set_operator (pCairoContext, CAIRO_OPERATOR_OVER);
 
-	GList *pFirstDrawnElement = (pDock->pFirstDrawnElement != NULL ? pDock->pFirstDrawnElement : pDock->icons);
+	GList *pFirstDrawnElement = pDock->icons;
 	if (pFirstDrawnElement != NULL)
 	{
 		double fXMin = (pDock->container.bIsHorizontal ? pArea->x : pArea->y), fXMax = (pDock->container.bIsHorizontal ? pArea->x + pArea->width : pArea->y + pArea->height);
@@ -461,7 +449,7 @@ static void cd_render_opengl (CairoDock *pDock)
 	double fFrameHeight = pDock->iDecorationsHeight + fLineWidth;
 	
 	double fDockOffsetX, fDockOffsetY;  // Offset du coin haut gauche du cadre.
-	GList *pFirstDrawnElement = (pDock->pFirstDrawnElement != NULL ? pDock->pFirstDrawnElement : pDock->icons);
+	GList *pFirstDrawnElement = pDock->icons;
 	if (pFirstDrawnElement == NULL)
 		return ;
 	if (cairo_dock_is_extended_dock (pDock))  // mode panel etendu.
@@ -485,7 +473,7 @@ static void cd_render_opengl (CairoDock *pDock)
 	double fDockMagnitude = cairo_dock_calculate_magnitude (pDock->iMagnitudeIndex);
 	
 	//\_____________ On genere les coordonnees du contour.
-	const CairoDockGLPath *pFramePath = cairo_dock_generate_rectangle_path (fDockWidth, fFrameHeight, fRadius, myDocksParam.bRoundedBottomCorner);
+	const CairoDockGLPath *pFramePath = cairo_dock_generate_rectangle_path (fDockWidth, fFrameHeight, fRadius, FALSE);  // same remark as in cairo rendering
 	
 	//\_____________ On remplit avec le fond.
 	glPushMatrix ();
@@ -508,82 +496,84 @@ static void cd_render_opengl (CairoDock *pDock)
 	
 	
 	//\_____________ On trace les separateurs physiques.
-	glPushMatrix ();
-	cairo_dock_set_container_orientation_opengl (CAIRO_CONTAINER (pDock));
-	
-	double x1, x2, dx, delta, h = pDock->iDecorationsHeight + 2*fLineWidth, h_ = h - fLineWidth;
 	GList *ic;
 	Icon *pIcon;
-	for (ic = pDock->icons; ic != NULL; ic = ic->next)
+	if (my_bPanelPhysicalSeparator)
 	{
-		pIcon = ic->data;
-		if (CAIRO_DOCK_ICON_TYPE_IS_SEPARATOR (pIcon))
+		glPushMatrix ();
+		cairo_dock_set_container_orientation_opengl (CAIRO_CONTAINER (pDock));
+		
+		double x1, x2, dx, delta, h = pDock->iDecorationsHeight + 2*fLineWidth, h_ = h - fLineWidth;
+		for (ic = pDock->icons; ic != NULL; ic = ic->next)
 		{
-			x1 = pIcon->fDrawX = pIcon->fX;
-			
-			pIcon = NULL;
-			for (ic = ic->next; ic != NULL; ic = ic->next)
+			pIcon = ic->data;
+			if (CAIRO_DOCK_ICON_TYPE_IS_SEPARATOR (pIcon))
 			{
-				pIcon = ic->data;
-				if (!CAIRO_DOCK_ICON_TYPE_IS_SEPARATOR (pIcon))
+				x1 = pIcon->fDrawX = pIcon->fX;
+				
+				pIcon = NULL;
+				for (ic = ic->next; ic != NULL; ic = ic->next)
+				{
+					pIcon = ic->data;
+					if (!CAIRO_DOCK_ICON_TYPE_IS_SEPARATOR (pIcon))
+						break;
+				}
+				if (ic != NULL)
+				{
+					pIcon = ic->data;
+					x2 = pIcon->fDrawX;
+				}
+				else
 					break;
-			}
-			if (ic != NULL)
-			{
-				pIcon = ic->data;
-				x2 = pIcon->fDrawX;
-			}
-			else
-				break;
-			
-			CairoDockGLPath *pPath = cairo_dock_new_gl_path (2*(iNbCurveSteps+1) + 1, (x1+x2)/2, h, 0., 0.);  // on part du milieu en haut pour que les triangles soient contenus dans l'enveloppe.
-			
-			dx = MIN (my_fPanelRadius, (x2 - x1) / 2);
-			delta = dx + h*tan(my_fPanelInclination)/2;
-			if (delta > (x2 - x1) / 2)
-				delta = (x2 - x1) / 2;
-			
-			cairo_dock_gl_path_rel_line_to (pPath,
-				-(x2-x1)/2, 0.);
-			cairo_dock_gl_path_rel_curve_to (pPath, iNbCurveSteps,
-				dx, 0.,
-				delta - dx, -h,
-				delta, -h);
-			cairo_dock_gl_path_rel_line_to (pPath,
-				x2 - x1 - 2*delta, 0.);
-			cairo_dock_gl_path_rel_curve_to (pPath, iNbCurveSteps,
-				dx, 0.,
-				delta - dx, h,
-				delta, h);
-			
-			glBlendFunc (GL_ONE, GL_ZERO);
-			glColor4f (0., 0., 0., 0.);
-			cairo_dock_fill_gl_path (pPath, 0);
-			glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
-			
-			if (fLineWidth > 0)
-			{
-				cairo_dock_gl_path_move_to (pPath, x1, h - fLineWidth/2);  // on part du haut/gauche, le nombre de points est ok.
+				
+				CairoDockGLPath *pPath = cairo_dock_new_gl_path (2*(iNbCurveSteps+1) + 1, (x1+x2)/2, h, 0., 0.);  // on part du milieu en haut pour que les triangles soient contenus dans l'enveloppe.
+				
+				dx = MIN (my_fPanelRadius, (x2 - x1) / 2);
+				delta = dx + h*tan(my_fPanelInclination)/2;
+				if (delta > (x2 - x1) / 2)
+					delta = (x2 - x1) / 2;
+				
+				cairo_dock_gl_path_rel_line_to (pPath,
+					-(x2-x1)/2, 0.);
 				cairo_dock_gl_path_rel_curve_to (pPath, iNbCurveSteps,
 					dx, 0.,
-					delta - dx, -h_,
-					delta, -h_);
+					delta - dx, -h,
+					delta, -h);
 				cairo_dock_gl_path_rel_line_to (pPath,
 					x2 - x1 - 2*delta, 0.);
 				cairo_dock_gl_path_rel_curve_to (pPath, iNbCurveSteps,
 					dx, 0.,
-					delta - dx, h_,
-					delta, h_);
-				glLineWidth (fLineWidth);
-				glColor4f (myDocksParam.fLineColor[0], myDocksParam.fLineColor[1], myDocksParam.fLineColor[2], myDocksParam.fLineColor[3]);
-				cairo_dock_stroke_gl_path (pPath, FALSE);
+					delta - dx, h,
+					delta, h);
+				
+				glBlendFunc (GL_ONE, GL_ZERO);
+				glColor4f (0., 0., 0., 0.);
+				cairo_dock_fill_gl_path (pPath, 0);
+				glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+				
+				if (fLineWidth > 0)
+				{
+					cairo_dock_gl_path_move_to (pPath, x1, h - fLineWidth/2);  // on part du haut/gauche, le nombre de points est ok.
+					cairo_dock_gl_path_rel_curve_to (pPath, iNbCurveSteps,
+						dx, 0.,
+						delta - dx, -h_,
+						delta, -h_);
+					cairo_dock_gl_path_rel_line_to (pPath,
+						x2 - x1 - 2*delta, 0.);
+					cairo_dock_gl_path_rel_curve_to (pPath, iNbCurveSteps,
+						dx, 0.,
+						delta - dx, h_,
+						delta, h_);
+					glLineWidth (fLineWidth);
+					glColor4f (myDocksParam.fLineColor[0], myDocksParam.fLineColor[1], myDocksParam.fLineColor[2], myDocksParam.fLineColor[3]);
+					cairo_dock_stroke_gl_path (pPath, FALSE);
+				}
+				
+				cairo_dock_free_gl_path (pPath);
 			}
-			
-			cairo_dock_free_gl_path (pPath);
 		}
+		glPopMatrix ();
 	}
-	glPopMatrix ();
-	
 	
 	//\_____________ On dessine la ficelle.
 	if (myIconsParam.iStringLineWidth > 0)
@@ -600,9 +590,12 @@ static void cd_render_opengl (CairoDock *pDock)
 	{
 		pIcon = ic->data;
 		
-		glPushMatrix ();
-		cairo_dock_render_one_icon_opengl (pIcon, pDock, fDockMagnitude, pIcon->bPointed);
-		glPopMatrix ();
+		if (! CAIRO_DOCK_ICON_TYPE_IS_SEPARATOR (pIcon))
+		{
+			glPushMatrix ();
+			cairo_dock_render_one_icon_opengl (pIcon, pDock, fDockMagnitude, pIcon->bPointed);
+			glPopMatrix ();
+		}
 		
 		ic = cairo_dock_get_next_element (ic, pDock->icons);
 	} while (ic != pFirstDrawnElement);
@@ -613,7 +606,7 @@ static void cd_render_opengl (CairoDock *pDock)
 static Icon *cd_calculate_icons (CairoDock *pDock)
 {
 	//\_____________ On calcule le nombre de groupes et la place qu'ils occupent.
-	int iNbGroups = 1, iCurrentOrder = -1;
+	int iNbGroups = 1;
 	double fCurrentGroupWidth = - myIconsParam.iIconGap, fGroupsWidth = 0.;
 	double fSeparatorsPonderation = 0;
 	GList *ic;
@@ -632,30 +625,18 @@ static Icon *cd_calculate_icons (CairoDock *pDock)
 					pIcon->fScale *= (1 + pIcon->fInsertRemoveFactor);
 			}
 			
-			if (CAIRO_DOCK_IS_USER_SEPARATOR (pIcon))  // si c'est un separateur automatique, le changement de groupe incrementera le compteur a l'icone suivante.
-			{
-				if (fCurrentGroupWidth > 0)  // le groupe courant est non vide, sinon c'est juste 2 separateurs cote a cote.
-				{
-					iNbGroups ++;
-					fSeparatorsPonderation += pIcon->fScale;
-					fGroupsWidth += MAX (0, fCurrentGroupWidth);
-					fCurrentGroupWidth = - myIconsParam.iIconGap;
-				}
-			}
-			continue;
-		}
-		if (iCurrentOrder != (int)cairo_dock_get_icon_order (pIcon))
-		{
 			if (fCurrentGroupWidth > 0)  // le groupe courant est non vide, sinon c'est juste 2 separateurs cote a cote.
 			{
 				iNbGroups ++;
-				fSeparatorsPonderation ++; // seuls les separateurs utilisateurs peuvent zoomer.
+				fSeparatorsPonderation += pIcon->fScale;
 				fGroupsWidth += MAX (0, fCurrentGroupWidth);
 				fCurrentGroupWidth = - myIconsParam.iIconGap;
 			}
+			
+			continue;
 		}
 		
-		pIcon->fScale = my_fPanelRatio;
+		pIcon->fScale = 1./** * my_fPanelRatio*/;
 		if (pIcon->fInsertRemoveFactor != 0)
 		{
 			if (pIcon->fInsertRemoveFactor > 0)
@@ -664,7 +645,6 @@ static Icon *cd_calculate_icons (CairoDock *pDock)
 				pIcon->fScale *= (1 + pIcon->fInsertRemoveFactor);
 		}
 		
-		iCurrentOrder = cairo_dock_get_icon_order (pIcon);
 		fCurrentGroupWidth += pIcon->fWidth * pIcon->fScale + myIconsParam.iIconGap;
 	}
 	if (fCurrentGroupWidth > 0)  // le groupe courant est non vide, sinon c'est juste un separateur a la fin.
@@ -693,7 +673,6 @@ static Icon *cd_calculate_icons (CairoDock *pDock)
 	double xg = fScreenBorderGap;  // abscisse de l'icone courante, et abscisse du debut du groupe courant.
 	double x = xg;
 	fCurrentGroupWidth = - myIconsParam.iIconGap;
-	iCurrentOrder = -1;
 	for (ic = pDock->icons; ic != NULL; ic = ic->next)
 	{
 		pIcon = ic->data;
@@ -701,34 +680,32 @@ static Icon *cd_calculate_icons (CairoDock *pDock)
 		{
 			pIcon->fX = x;
 			pIcon->fDrawX = pIcon->fX;
-			if (CAIRO_DOCK_IS_USER_SEPARATOR (pIcon))  // si c'est un separateur automatique, le changement de groupe incrementera le compteur a l'icone suivante.
-			{
-				if (fCurrentGroupWidth > 0)  // le groupe courant est non vide, sinon c'est juste 2 separateurs cote a cote.
-				{
-					xg += fCurrentGroupWidth + fGroupGap * pIcon->fScale;
-					if (pPointedIcon == NULL && xm > x && xm < xg)
-					{
-						pIcon->bPointed = TRUE;
-						pPointedIcon = pIcon;
-					}
-					else
-						pIcon->bPointed = FALSE;
-					x = xg;
-					fCurrentGroupWidth = - myIconsParam.iIconGap;
-				}
-			}
-			continue;
-		}
-		if (iCurrentOrder != (int)cairo_dock_get_icon_order (pIcon))
-		{
+			
+			if (pDock->container.bDirectionUp)
+				pIcon->fY = pDock->iMaxDockHeight - pDock->iMinDockHeight;
+			else
+				pIcon->fY = pDock->iMinDockHeight;
+			pIcon->fDrawY = pIcon->fY;
+			
+			pIcon->fWidthFactor = 0;
+			
 			if (fCurrentGroupWidth > 0)  // le groupe courant est non vide, sinon c'est juste 2 separateurs cote a cote.
 			{
-				xg += fCurrentGroupWidth + fGroupGap;
+				xg += fCurrentGroupWidth + fGroupGap * pIcon->fScale;
+				if (pPointedIcon == NULL && xm > x && xm < xg)
+				{
+					pIcon->bPointed = TRUE;
+					pPointedIcon = pIcon;
+				}
+				else
+					pIcon->bPointed = FALSE;
 				x = xg;
 				fCurrentGroupWidth = - myIconsParam.iIconGap;
 			}
+			
+			continue;
 		}
-		iCurrentOrder = cairo_dock_get_icon_order (pIcon);
+		
 		fCurrentGroupWidth += pIcon->fWidth * pIcon->fScale + myIconsParam.iIconGap;
 		
 		pIcon->fX = x;
@@ -742,7 +719,7 @@ static Icon *cd_calculate_icons (CairoDock *pDock)
 		pIcon->fDrawX = pIcon->fX;
 		
 		if (pDock->container.bDirectionUp)
-			pIcon->fY = pDock->iMaxDockHeight - (myDocksParam.iDockLineWidth + myDocksParam.iFrameMargin + pIcon->fHeight * my_fPanelRatio);
+			pIcon->fY = pDock->iMaxDockHeight - (myDocksParam.iDockLineWidth + myDocksParam.iFrameMargin + pIcon->fHeight/** * my_fPanelRatio*/);
 		else
 			pIcon->fY = myDocksParam.iDockLineWidth + myDocksParam.iFrameMargin;
 		pIcon->fDrawY = pIcon->fY;
@@ -763,6 +740,165 @@ static Icon *cd_calculate_icons (CairoDock *pDock)
 }
 
 
+void cd_update_input_shape (CairoDock *pDock)
+{
+	if (pDock->pShapeBitmap != NULL)
+	{
+		CDPanelData *pData = pDock->pRendererData;
+		g_return_if_fail (pData != NULL);
+		
+		gboolean bValidGroup = FALSE;  // valid = non-empty
+		#if (GTK_MAJOR_VERSION < 3)
+		cairo_t *pCairoContext = gdk_cairo_create (pDock->pShapeBitmap);
+		if  (pCairoContext != NULL)
+		{
+			cairo_set_source_rgba (pCairoContext, 0.0f, 0.0f, 0.0f, 0.0f);
+			cairo_set_operator (pCairoContext, CAIRO_OPERATOR_SOURCE);
+			
+			GList *ic;
+			Icon *pIcon;
+			for (ic = pDock->icons; ic != NULL; ic = ic->next)
+			{
+				pIcon = ic->data;
+				if (CAIRO_DOCK_ICON_TYPE_IS_SEPARATOR (pIcon))
+				{
+					if (bValidGroup)
+					{
+						if (pDock->container.bIsHorizontal)
+						{
+							cairo_rectangle (pCairoContext,
+								pIcon->fXAtRest + 2 * my_fPanelRadius,  // we let a few pixels to be able to grab the separtator, and to avoid leaving the dock too easily.
+								0,
+								pData->fGroupGap - 4 * my_fPanelRadius,
+								pDock->iMaxDockHeight);  // we use iMaxDockHeight instead of the actual window size, because at this time, the dock's window may not have its definite size.
+						}
+						else
+						{
+							cairo_rectangle (pCairoContext,
+								0,
+								pIcon->fXAtRest + 2 * my_fPanelRadius,  // we let a few pixels to be able to grab the separtator, and to avoid leaving the dock too easily.
+								pDock->iMaxDockHeight,
+								pData->fGroupGap - 4 * my_fPanelRadius);
+						}
+						cairo_fill (pCairoContext);
+						bValidGroup = FALSE;
+					}
+				}
+				else
+				{
+					bValidGroup = TRUE;
+				}
+			}
+			
+			cairo_destroy (pCairoContext);
+		}
+		#else
+		cairo_rectangle_int_t rect;
+		GList *ic;
+			Icon *pIcon;
+			for (ic = pDock->icons; ic != NULL; ic = ic->next)
+			{
+				pIcon = ic->data;
+				if (CAIRO_DOCK_ICON_TYPE_IS_SEPARATOR (pIcon))
+				{
+					if (bValidGroup)
+					{
+						if (pDock->container.bIsHorizontal)
+						{
+							rect.x = pIcon->fXAtRest + 2 * my_fPanelRadius;  // we let a few pixels to be able to grab the separtator, and to avoid leaving the dock too easily.
+							rect.y = 0;
+							rect.width = pData->fGroupGap - 4 * my_fPanelRadius;
+							rect.height = pDock->iMaxDockHeight;  // we use iMaxDockHeight instead of the actual window size, because at this time, the dock's window may not have its definite size.
+							cd_debug ("rect %d; %d", (int)rect.x, (int)rect.width);
+						}
+						else
+						{
+							rect.x = 0;
+							rect.y = pIcon->fXAtRest + 2 * my_fPanelRadius;  // we let a few pixels to be able to grab the separtator, and to avoid leaving the dock too easily.
+							rect.width = pDock->iMaxDockHeight;
+							rect.height = pData->fGroupGap - 4 * my_fPanelRadius;
+						}
+						cairo_region_subtract_rectangle (pDock->pShapeBitmap, &rect);
+						bValidGroup = FALSE;
+					}
+				}
+				else
+				{
+					bValidGroup = TRUE;
+				}
+			}
+		#endif
+	}
+}
+
+static void set_icon_size (Icon *icon, CairoDock *pDock)
+{
+	int wi, hi;
+	if (pDock->iIconSize != 0)
+	{
+		wi = hi = pDock->iIconSize;
+	}
+	else  // same size as main dock.
+	{
+		wi = myIconsParam.iIconWidth;
+		hi = myIconsParam.iIconHeight;
+	}
+	if (my_fPanelRatio != 0)  /// workaround; TODO: we need to load the config before loading any icon...
+	{
+		wi *= my_fPanelRatio;
+		hi *= my_fPanelRatio;
+	}
+	//g_print (" ~~~~~~~~~~~~ size: %d => %dx%d\n", pDock->iIconSize, wi, hi);
+	
+	// set the visible size at rest.
+	if (CAIRO_DOCK_ICON_TYPE_IS_APPLET (icon))  // for applets, consider (fWidth,fHeight) as a requested size, if not 0.
+	{
+		//g_print ("%s (%s, %.1fx%.1f\n", __func__, icon->pModuleInstance->pModule->pVisitCard->cModuleName, icon->fWidth, icon->fHeight);
+		if (icon->iImageWidth != 0)
+		{
+			if (pDock->container.bIsHorizontal)
+				icon->fWidth = icon->iImageWidth;
+			else
+				icon->fHeight = icon->iImageWidth;
+		}
+		if (icon->iImageHeight != 0)
+		{
+			if (pDock->container.bIsHorizontal)
+				icon->fHeight = icon->iImageHeight;
+			else
+				icon->fWidth = icon->iImageHeight;
+		}
+		if (icon->fWidth == 0)
+			icon->fWidth = wi;
+		if (icon->fHeight == 0 || icon->fHeight > hi)
+			icon->fHeight = hi;
+	}
+	else if (CAIRO_DOCK_ICON_TYPE_IS_SEPARATOR (icon))  // separators have their own size.
+	{
+		icon->fWidth = myIconsParam.iSeparatorWidth * my_fPanelRatio;;
+		icon->fHeight = MIN (myIconsParam.iSeparatorHeight * my_fPanelRatio, hi);
+	}
+	else  // any other icon use the global size
+	{
+		icon->fWidth = wi;
+		icon->fHeight = hi;
+	}
+	
+	// texture size can be deduced then.
+	if (pDock->container.bIsHorizontal
+	|| (CAIRO_DOCK_ICON_TYPE_IS_SEPARATOR (icon) && myIconsParam.bRevolveSeparator))
+	{
+		icon->iImageWidth = icon->fWidth;
+		icon->iImageHeight = icon->fHeight;
+	}
+	else
+	{
+		icon->iImageWidth = icon->fHeight;
+		icon->iImageHeight = icon->fWidth;
+	}
+	//g_print (" ~~~~~~~~~~~~ => %dx%d\n", icon->iImageWidth, icon->iImageHeight);
+}
+
 void cd_rendering_register_panel_renderer (const gchar *cRendererName)
 {
 	CairoDockRenderer *pRenderer = g_new0 (CairoDockRenderer, 1);
@@ -773,6 +909,8 @@ void cd_rendering_register_panel_renderer (const gchar *cRendererName)
 	pRenderer->render_optimized = cd_render_optimized;
 	pRenderer->render_opengl = cd_render_opengl;
 	pRenderer->set_subdock_position = cairo_dock_set_subdock_position_linear;
+	pRenderer->update_input_shape = cd_update_input_shape;
+	pRenderer->set_icon_size = set_icon_size;
 	// parametres
 	pRenderer->bUseReflect = FALSE;
 	pRenderer->cDisplayedName = D_ (cRendererName);
