@@ -44,7 +44,6 @@ Voila pour la petite explication :-)
 #include <cairo.h>
 
 #ifdef HAVE_GLITZ
-#include <gdk/gdkx.h>
 #include <glitz-glx.h>
 #include <cairo-glitz.h>
 #endif
@@ -90,7 +89,7 @@ static void cd_rendering_set_subdock_position_parabole (Icon *pPointedIcon, Cair
 		}
 		//cd_debug ("recalage : %.2f (%d)\n", -iMouseX + pPointedIcon->fDrawX + pPointedIcon->fWidth * pPointedIcon->fScale / 2, pSubDock->iMaxLabelWidth);
 		pSubDock->fAlign = 0;
-		pSubDock->iGapY = (pDock->iGapY + pDock->iMaxDockHeight);
+		pSubDock->iGapY = (pDock->iGapY + pDock->iActiveHeight);
 		pSubDock->iGapX = iX + pDock->container.iWindowPositionX - (pDock->container.bIsHorizontal ? pDock->iScreenOffsetX : pDock->iScreenOffsetY) - pSubDock->iMaxLabelWidth;
 	}
 	else
@@ -103,7 +102,7 @@ static void cd_rendering_set_subdock_position_parabole (Icon *pPointedIcon, Cair
 			iX += icon->fWidth + (pPointedIcon->fWidth * pPointedIcon->fScale - icon->fWidth) / 2;
 		}
 		pSubDock->fAlign = 1;
-		pSubDock->iGapY = (pDock->iGapY + pDock->iMaxDockHeight);
+		pSubDock->iGapY = (pDock->iGapY + pDock->iActiveHeight);
 		pSubDock->iGapX =  pDock->container.iWindowPositionX - (pDock->container.bIsHorizontal ? pDock->iScreenOffsetX : pDock->iScreenOffsetY) + iX - g_desktopGeometry.iScreenWidth[pDock->container.bIsHorizontal] + pSubDock->iMaxLabelWidth;
 		
 	}
@@ -270,9 +269,9 @@ static void cd_rendering_calculate_max_dock_size_parabole (CairoDock *pDock)
 	}
 	
 	pDock->fMagnitudeMax = my_fParaboleMagnitude;
-	pDock->pFirstDrawnElement = cairo_dock_calculate_icons_positions_at_rest_linear (pDock->icons, pDock->fFlatDockWidth, pDock->iScrollOffset);
+	cairo_dock_calculate_icons_positions_at_rest_linear (pDock->icons, pDock->fFlatDockWidth);
 	
-	int iMaxDockWidth = ceil (cairo_dock_calculate_max_dock_width (pDock, pDock->pFirstDrawnElement, pDock->fFlatDockWidth, 1., 0));
+	int iMaxDockWidth = ceil (cairo_dock_calculate_max_dock_width (pDock, pDock->fFlatDockWidth, 1., 0));
 	GList* ic;
 	Icon *icon;
 	pDock->iMaxLabelWidth = 0;
@@ -310,8 +309,8 @@ static void cd_rendering_calculate_max_dock_size_parabole (CairoDock *pDock)
 		//cd_debug ("=> %.2fx%.2f , %.2f", w, h, lambda);
 	}
 	
-	pDock->iMaxDockHeight = h + pDock->iMaxIconHeight * sqrt (5./4.) * (1 + my_fParaboleMagnitude * g_fAmplitude);
-	pDock->iMaxDockWidth = w + pDock->iMaxIconHeight * (.5+sqrt(5./4.)) * (1 + my_fParaboleMagnitude * g_fAmplitude);  // ce serait plutot MaxIconWidth mais bon ...
+	pDock->iMaxDockHeight = h + pDock->iMaxIconHeight * sqrt (5./4.) * (1 + my_fParaboleMagnitude * myIconsParam.fAmplitude);
+	pDock->iMaxDockWidth = w + pDock->iMaxIconHeight * (.5+sqrt(5./4.)) * (1 + my_fParaboleMagnitude * myIconsParam.fAmplitude);  // ce serait plutot MaxIconWidth mais bon ...
 	
 	pDock->iMaxDockWidth += pDock->iMaxLabelWidth;  // theta(0) = 0 => texte horizontal.
 	double fOrientationMax = G_PI/2 - atan (my_fParaboleRatio * my_fParaboleCurvature);  // fCurve_ (W) se simplifie ici.
@@ -321,8 +320,11 @@ static void cd_rendering_calculate_max_dock_size_parabole (CairoDock *pDock)
 	pDock->iDecorationsWidth = 0;
 	pDock->iDecorationsHeight = 0;
 	
-	pDock->iMinDockWidth = pDock->fFlatDockWidth;
+	pDock->iMinDockWidth = pDock->iMaxDockWidth;
 	pDock->iMinDockHeight = pDock->iMaxIconHeight;
+	
+	pDock->iActiveWidth = pDock->iMaxDockWidth;
+	pDock->iActiveHeight = pDock->iMaxDockHeight;
 }
 
 
@@ -337,12 +339,12 @@ static void cd_rendering_render_parabole (cairo_t *pCairoContext, CairoDock *pDo
 	//\____________________ On dessine le cadre.
 	
 	//\____________________ On dessine la ficelle qui les joint.
-	if (myIcons.iStringLineWidth > 0)
-		cairo_dock_draw_string (pCairoContext, pDock, myIcons.iStringLineWidth, FALSE, FALSE);
+	if (myIconsParam.iStringLineWidth > 0)
+		cairo_dock_draw_string (pCairoContext, pDock, myIconsParam.iStringLineWidth, FALSE, FALSE);
 	
 	
 	//\____________________ On dessine les icones et leurs etiquettes, mais separement.
-	GList *pFirstDrawnElement = (pDock->pFirstDrawnElement != NULL ? pDock->pFirstDrawnElement : pDock->icons);
+	GList *pFirstDrawnElement = pDock->icons;
 	if (pFirstDrawnElement == NULL)
 		return;
 	
@@ -510,7 +512,7 @@ static double cd_rendering_calculate_wave_position (CairoDock *pDock, double fCu
 {
 	//cd_debug ("%s (%.2f)", __func__, fCurvilignAbscisse);
 	
-	Icon *pFirstIcon = cairo_dock_get_first_drawn_icon (pDock);
+	Icon *pFirstIcon = cairo_dock_get_first_icon (pDock->icons);
 	if (pFirstIcon == NULL || fCurvilignAbscisse <= 0)
 		return 0;
 	double fWaveOffset, fWaveExtrema;
@@ -520,7 +522,7 @@ static double cd_rendering_calculate_wave_position (CairoDock *pDock, double fCu
 	do
 	{
 		//cd_debug ("  x_abs : %.2f / %.2f", x_abs, pDock->fFlatDockWidth);
-		cairo_dock_calculate_wave_with_position_linear (pDock->icons, pDock->pFirstDrawnElement, x_abs, fMagnitude, pDock->fFlatDockWidth, pDock->fFlatDockWidth, pDock->container.iHeight, 0*pDock->fAlign, pDock->fFoldingFactor, pDock->container.bDirectionUp);
+		cairo_dock_calculate_wave_with_position_linear (pDock->icons, x_abs, fMagnitude, pDock->fFlatDockWidth, pDock->fFlatDockWidth, pDock->container.iHeight, 0*pDock->fAlign, pDock->fFoldingFactor, pDock->container.bDirectionUp);
 		fWaveOffset = - pFirstIcon->fX;
 		
 		fWaveExtrema = fWaveOffset + x_abs;
@@ -545,7 +547,7 @@ Icon *cd_rendering_calculate_icons_parabole (CairoDock *pDock)
 		return NULL;
 	
 	//\____________________ On calcule la projection du curseur sur la courbe.
-	double fMaxScale =  1. + my_fParaboleMagnitude * g_fAmplitude;
+	double fMaxScale =  1. + my_fParaboleMagnitude * myIconsParam.fAmplitude;
 	double w = MAX (1, pDock->container.iWidth - pDock->iMaxLabelWidth - pDock->iMaxIconHeight * (.5+sqrt(5./4.)) * fMaxScale);
 	double h = my_fParaboleRatio * w;
 	double alpha = my_fParaboleCurvature, lambda = h / pow (w, alpha);
@@ -579,7 +581,7 @@ Icon *cd_rendering_calculate_icons_parabole (CairoDock *pDock)
 	//cd_debug (" => x_abs : %d (fMagnitude:%.2f ; fFoldingFactor:%.2f)", x_abs, fMagnitude, pDock->fFoldingFactor);
 	
 	//\_______________ On en deduit l'ensemble des parametres des icones.
-	pPointedIcon = cairo_dock_calculate_wave_with_position_linear (pDock->icons, pDock->pFirstDrawnElement, x_abs, fMagnitude, (int) pDock->fFlatDockWidth, (int) pDock->fFlatDockWidth, pDock->container.iHeight, 0*pDock->fAlign, pDock->fFoldingFactor, pDock->container.bDirectionUp);
+	pPointedIcon = cairo_dock_calculate_wave_with_position_linear (pDock->icons, x_abs, fMagnitude, (int) pDock->fFlatDockWidth, (int) pDock->fFlatDockWidth, pDock->container.iHeight, 0*pDock->fAlign, pDock->fFoldingFactor, pDock->container.bDirectionUp);
 	//cd_debug (" => pPointedIcon : %s; %.2f", pPointedIcon->cName, pPointedIcon->fX);
 	
 	
@@ -593,7 +595,7 @@ Icon *cd_rendering_calculate_icons_parabole (CairoDock *pDock)
 		theta_ = -theta_;
 	//g_print ("theta_ : %.2fdeg\n", theta_);
 	
-	GList *pFirstDrawnElement = (pDock->pFirstDrawnElement != NULL ? pDock->pFirstDrawnElement : pDock->icons);
+	GList *pFirstDrawnElement = pDock->icons;
 	ic = pFirstDrawnElement;
 	Icon *pFirstIcon = pFirstDrawnElement->data;
 	double fOrientationMax = G_PI/2 - atan (my_fParaboleRatio * my_fParaboleCurvature);
@@ -682,11 +684,11 @@ static void cd_rendering_render_parabole_opengl (CairoDock *pDock)
 	//\____________________ On dessine le cadre.
 	
 	//\____________________ On dessine la ficelle qui les joint.
-	if (myIcons.iStringLineWidth > 0)
-		cairo_dock_draw_string_opengl (pDock, myIcons.iStringLineWidth, FALSE, FALSE);
+	if (myIconsParam.iStringLineWidth > 0)
+		cairo_dock_draw_string_opengl (pDock, myIconsParam.iStringLineWidth, FALSE, FALSE);
 	
 	//\____________________ On dessine les icones et leurs etiquettes, mais separement.
-	GList *pFirstDrawnElement = (pDock->pFirstDrawnElement != NULL ? pDock->pFirstDrawnElement : pDock->icons);
+	GList *pFirstDrawnElement = pDock->icons;
 	if (pFirstDrawnElement == NULL)
 		return;
 	
@@ -701,7 +703,7 @@ static void cd_rendering_render_parabole_opengl (CairoDock *pDock)
 		
 		cairo_dock_render_one_icon_opengl (icon, pDock, fDockMagnitude, FALSE);
 		
-		if (icon->pTextBuffer != NULL)  // en opengl on dessine les etiquettes meme pendant le depliage.
+		if (icon->iLabelTexture != 0)  // en opengl on dessine les etiquettes meme pendant le depliage.
 		{
 			glPushMatrix ();
 			cairo_dock_translate_on_icon_opengl (icon, CAIRO_CONTAINER (pDock), 1.);
@@ -711,7 +713,8 @@ static void cd_rendering_render_parabole_opengl (CairoDock *pDock)
 			glTranslatef (icon->fWidth * icon->fScale/2, -icon->fHeight * icon->fScale/2, 0.);
 			
 			_cairo_dock_enable_texture ();
-			_cairo_dock_set_blend_over ();
+			///_cairo_dock_set_blend_over ();
+			_cairo_dock_set_blend_alpha ();
 			_cairo_dock_set_alpha ((1 - pDock->fFoldingFactor) * (1 - pDock->fFoldingFactor));
 			
 			if (pDock->fAlign == 1)
