@@ -1,4 +1,4 @@
-	/**
+/**
 * This file is a part of the Cairo-Dock project
 *
 * Copyright : (C) see the 'copyright' file.
@@ -29,13 +29,9 @@
 #include <gio/gio.h>
 #define G_VFS_DBUS_DAEMON_NAME "org.gtk.vfs.Daemon"
 
-#include <cairo-dock-launcher-manager.h>
-#include <cairo-dock-launcher-factory.h>
-#include <cairo-dock-log.h>
-#include <cairo-dock-desktop-file-factory.h>
-#include <cairo-dock-dbus.h>
+#include <cairo-dock.h>
 
-static void cairo_dock_gio_vfs_empty_dir (const gchar *cBaseURI);
+static void _cairo_dock_gio_vfs_empty_dir (const gchar *cBaseURI);
 
 static GHashTable *s_hMonitorHandleTable = NULL;
 
@@ -95,7 +91,14 @@ static gchar *_cd_get_icon_path (GIcon *pIcon, const gchar *cTargetURI)  // cTar
 		for (i = 0; cFileNames[i] != NULL && cIconPath == NULL; i ++)
 		{
 			//g_print (" une icone possible est : %s\n", cFileNames[i]);
-			cIconPath = cairo_dock_search_icon_s_path (cFileNames[i]);
+			
+			gchar *path = cairo_dock_search_icon_s_path (cFileNames[i], CAIRO_DOCK_DEFAULT_ICON_SIZE);
+			if (path)
+			{
+				g_free (path);
+				cIconPath = g_strdup (cFileNames[i]);
+			}
+			
 			//g_print ("  chemin trouve : %s\n", cIconPath);
 		}
 	}
@@ -140,7 +143,7 @@ static void _cd_find_mount_from_volume_name (const gchar *cVolumeName, GMount **
 		&erreur);
 	if (erreur != NULL)
 	{
-		cd_warning ("gnome_integration : %s", erreur->message);
+		cd_warning ("gvfs-integration : %s", erreur->message);
 		g_error_free (erreur);
 		g_object_unref (pFile);
 		return ;
@@ -154,7 +157,7 @@ static void _cd_find_mount_from_volume_name (const gchar *cVolumeName, GMount **
 		pFileInfo = g_file_enumerator_next_file (pFileEnum, NULL, &erreur);
 		if (erreur != NULL)
 		{
-			cd_warning ("gnome_integration : %s", erreur->message);
+			cd_warning ("gvfs-integration : %s", erreur->message);
 			g_error_free (erreur);
 			erreur = NULL;
 		}
@@ -166,7 +169,7 @@ static void _cd_find_mount_from_volume_name (const gchar *cVolumeName, GMount **
 			if (iFileType == G_FILE_TYPE_MOUNTABLE)
 			{
 				const gchar *cFileName = g_file_info_get_name (pFileInfo);
-				cd_message ("  test de  %s...", cFileName);
+				cd_debug ("  test of  %s...", cFileName);
 				const gchar *cTargetURI = g_file_info_get_attribute_string (pFileInfo, G_FILE_ATTRIBUTE_STANDARD_TARGET_URI);
 				
 				GMount *pMount = NULL;
@@ -182,7 +185,7 @@ static void _cd_find_mount_from_volume_name (const gchar *cVolumeName, GMount **
 					cd_message ("    mount : %s", cName);
 					if (cName != NULL && strcmp (cName, cVolumeName) == 0)
 					{
-						cd_message ("TROUVE");
+						cd_debug ("Found");
 						*pFoundMount = pMount;
 						*cURI = g_strconcat ("computer:///", cFileName, NULL);
 						GIcon *pSystemIcon = g_mount_get_icon (pMount);
@@ -291,12 +294,16 @@ static gboolean _cd_find_can_eject_from_drive_name (const gchar *cName)
 
 static void cairo_dock_gio_vfs_get_file_info (const gchar *cBaseURI, gchar **cName, gchar **cURI, gchar **cIconName, gboolean *bIsDirectory, int *iVolumeID, double *fOrder, CairoDockFMSortType iSortType)
 {
+	*cName = NULL;
+	*cURI = NULL;
+	*cIconName = NULL;
 	g_return_if_fail (cBaseURI != NULL);
 	GError *erreur = NULL;
 	cd_message ("%s (%s)", __func__, cBaseURI);
 	
-	gchar *cFullURI;
-	if (strncmp (cBaseURI, "x-nautilus-desktop://", 21) == 0)
+	// make it a valid URI.
+	gchar *cValidUri;
+	if (strncmp (cBaseURI, "x-nautilus-desktop://", 21) == 0)  // shortcut on the desktop (nautilus)
 	{
 		gchar *cNautilusFile = g_strdup (cBaseURI+14);
 		memcpy (cNautilusFile, "file", 4);
@@ -308,56 +315,71 @@ static void cairo_dock_gio_vfs_get_file_info (const gchar *cBaseURI, gchar **cNa
 		{
 			cNautilusFile[strlen(cNautilusFile)-6] = '\0';
 		}
-		cFullURI = g_filename_from_uri (cNautilusFile, NULL, &erreur);
+		cValidUri = g_filename_from_uri (cNautilusFile, NULL, &erreur);
 		if (erreur != NULL)
 		{
-			cd_warning ("gnome_integration : %s", erreur->message);
+			cd_warning ("gvfs-integration : %s", erreur->message);
 			g_error_free (erreur);
 			return ;
 		}
-		gchar *cVolumeName = cFullURI + 1;  // on saute le '/'.
+		gchar *cVolumeName = cValidUri + 1;  // on saute le '/'.
 		cd_message ("cVolumeName : %s", cVolumeName);
 		
 		GMount *pMount = NULL;
-		_cd_find_mount_from_volume_name (cVolumeName, &pMount, cURI, cIconName);
+		gchar *uri=NULL, *iconname=NULL;
+		_cd_find_mount_from_volume_name (cVolumeName, &pMount, &uri, &iconname);
 		g_return_if_fail (pMount != NULL);
 		
-		*cName = g_strdup (cVolumeName);
-		*bIsDirectory = TRUE;
-		*iVolumeID = 1;
-		*fOrder = 0;
+		if (cURI)
+			*cURI = uri;
+		else
+			g_free (uri);
+		if (cIconName)
+			*cIconName = iconname;
+		else
+			g_free (iconname);
+		if (cName)
+			*cName = g_strdup (cVolumeName);
+		if (bIsDirectory)
+			*bIsDirectory = TRUE;
+		if (iVolumeID)
+			*iVolumeID = 1;
+		if (fOrder)
+			*fOrder = 0;
 		//g_object_unref (pMount);
 		
-		g_free (cFullURI);
+		g_free (cValidUri);
 		//g_free (cNautilusFile);
 		return;
 	}
-	else
+	else  // normal file
 	{
 		if (*cBaseURI == '/')
-			cFullURI = g_filename_to_uri (cBaseURI, NULL, NULL);
+			cValidUri = g_filename_to_uri (cBaseURI, NULL, NULL);
 		else
-			cFullURI = g_strdup (cBaseURI);
-		if (*cBaseURI == ':' || *cFullURI == ':')  // cas bizarre au demontage d'un signet ftp quand celui-ci n'est pas accessible plantage dans dbus).
+			cValidUri = g_strdup (cBaseURI);
+		if (*cBaseURI == ':' || *cValidUri == ':')  // cas bizarre au demontage d'un signet ftp quand celui-ci n'est pas accessible plantage dans dbus).
 		{
-			cd_warning ("invalid URI (%s ; %s), skip it", cBaseURI, cFullURI);
-			g_free (cFullURI);
-			*cName = NULL;
-			*cURI = NULL;
-			*cIconName = NULL;
+			cd_warning ("invalid URI (%s ; %s), skip it", cBaseURI, cValidUri);
+			g_free (cValidUri);
 			return;
 		}
 	}
-	cd_message (" -> cFullURI : %s", cFullURI);
-	
-	GFile *pFile = g_file_new_for_uri (cFullURI);
 	
+	// get its attributes.
+	GFile *pFile = g_file_new_for_uri (cValidUri);
+	g_return_if_fail (pFile);
 	const gchar *cQuery = G_FILE_ATTRIBUTE_STANDARD_TYPE","
 		G_FILE_ATTRIBUTE_STANDARD_SIZE","
 		G_FILE_ATTRIBUTE_TIME_MODIFIED","
+		G_FILE_ATTRIBUTE_TIME_ACCESS","
 		G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE","
 		G_FILE_ATTRIBUTE_STANDARD_NAME","
 		G_FILE_ATTRIBUTE_STANDARD_ICON","
+		G_FILE_ATTRIBUTE_THUMBNAIL_PATH","
+		#if (GLIB_MAJOR_VERSION > 2) || (GLIB_MAJOR_VERSION == 2 && GLIB_MINOR_VERSION >= 20)
+		///G_FILE_ATTRIBUTE_PREVIEW_ICON","
+		#endif
 		G_FILE_ATTRIBUTE_STANDARD_TARGET_URI","
 		G_FILE_ATTRIBUTE_MOUNTABLE_UNIX_DEVICE;
 	GFileInfo *pFileInfo = g_file_query_info (pFile,
@@ -365,121 +387,167 @@ static void cairo_dock_gio_vfs_get_file_info (const gchar *cBaseURI, gchar **cNa
 		G_FILE_QUERY_INFO_NONE,  /// G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS
 		NULL,
 		&erreur);
-	//g_object_unref (pFile);
 	if (erreur != NULL)  // peut arriver si l'emplacement n'est pas monte.
 	{
-		cd_debug ("gnome_integration : %s", erreur->message);  // inutile d'en faire un warning.
+		cd_debug ("gvfs-integration : %s", erreur->message);  // inutile d'en faire un warning.
 		g_error_free (erreur);
+		g_free (cValidUri);
+		g_object_unref (pFile);
 		return ;
 	}
 	
-	*cURI = cFullURI;
 	const gchar *cFileName = g_file_info_get_name (pFileInfo);
 	const gchar *cMimeType = g_file_info_get_content_type (pFileInfo);
 	GFileType iFileType = g_file_info_get_file_type (pFileInfo);
-	
-	if (iSortType == CAIRO_DOCK_FM_SORT_BY_DATE)
-	{
-		GTimeVal t;
-		g_file_info_get_modification_time (pFileInfo, &t);
-		*fOrder = t.tv_sec;
-	}
-	else if (iSortType == CAIRO_DOCK_FM_SORT_BY_SIZE)
-		*fOrder = g_file_info_get_size (pFileInfo);
-	else if (iSortType == CAIRO_DOCK_FM_SORT_BY_TYPE)
-		*fOrder = (cMimeType != NULL ? *((int *) cMimeType) : 0);
-	else
-		*fOrder = 0;
-	
-	*bIsDirectory = (iFileType == G_FILE_TYPE_DIRECTORY);
-	cd_message (" => '%s' (mime:%s ; bIsDirectory:%d)", cFileName, cMimeType, *bIsDirectory);
-	
 	const gchar *cTargetURI = g_file_info_get_attribute_string (pFileInfo, G_FILE_ATTRIBUTE_STANDARD_TARGET_URI);
+	
+	// find the order of the file
+	if (fOrder)
+	{
+		if (iSortType == CAIRO_DOCK_FM_SORT_BY_DATE)
+		{
+			GTimeVal t;
+			g_file_info_get_modification_time (pFileInfo, &t);
+			*fOrder = t.tv_sec;
+		}
+		else if (iSortType == CAIRO_DOCK_FM_SORT_BY_ACCESS)
+			*fOrder =  g_file_info_get_attribute_uint64 (pFileInfo, G_FILE_ATTRIBUTE_TIME_ACCESS);
+		else if (iSortType == CAIRO_DOCK_FM_SORT_BY_SIZE)
+			*fOrder = g_file_info_get_size (pFileInfo);
+		else if (iSortType == CAIRO_DOCK_FM_SORT_BY_TYPE)
+			*fOrder = (cMimeType != NULL ? *((int *) cMimeType) : 0);
+		else
+			*fOrder = 0;
+	}
+	
+	if (bIsDirectory)
+		*bIsDirectory = (iFileType == G_FILE_TYPE_DIRECTORY);
+	
+	
+	// find a readable name if it's a mount point.
 	if (iFileType == G_FILE_TYPE_MOUNTABLE)
 	{
-		*cName = NULL;
-		*iVolumeID = 1;
+		if (iVolumeID)
+			*iVolumeID = 1;
 		
-		cd_message ("  cTargetURI:%s", cTargetURI);
-		GMount *pMount = NULL;
-		if (cTargetURI != NULL)
+		if (cName)
 		{
-			GFile *file = g_file_new_for_uri (cTargetURI);
-			pMount = g_file_find_enclosing_mount (file, NULL, NULL);
-			//g_object_unref (file);
-		}
-		if (pMount != NULL)
-		{
-			*cName = g_mount_get_name (pMount);
-			cd_message ("un GMount existe (%s)",* cName);
-		}
-		else
-		{
-			gchar *cMountName = g_strdup (cFileName);
-			gchar *str = strrchr (cMountName, '.');  // on vire l'extension ".volume" ou ".drive".
-			if (str != NULL)
+			*cName = NULL;
+			
+			cd_message ("  cTargetURI:%s", cTargetURI);
+			GMount *pMount = NULL;
+			if (cTargetURI != NULL)
 			{
-				*str = '\0';
-				if (strcmp (str+1, "link") == 0)  // pour les liens, on prend le nom du lien.
-				{
-					if (strcmp (cMountName, "root") == 0)  // on remplace 'root' par un nom plus parlant, sinon on prendra le nom du lien.
-					{
-						*cName = g_strdup ("/");
-					}
-				}
-				else if (strcmp (str+1, "drive") == 0)  // on cherche un nom plus parlant si possible.
-				{
-					gchar *cVolumeName = _cd_find_volume_name_from_drive_name (cMountName);
-					if (cVolumeName != NULL)
-					{
-						*cName = cVolumeName;
-					}
-				}
+				GFile *file = g_file_new_for_uri (cTargetURI);
+				pMount = g_file_find_enclosing_mount (file, NULL, NULL);
+				//g_object_unref (file);
 			}
-			if (*cName == NULL)
-				*cName = cMountName;
-			//else
-				//g_free (cMountName);
+			if (pMount != NULL)
+			{
+				*cName = g_mount_get_name (pMount);
+				cd_message ("un GMount existe (%s)",* cName);
+			}
+			else
+			{
+				gchar *cMountName = g_strdup (cFileName);
+				gchar *str = strrchr (cMountName, '.');  // on vire l'extension ".volume" ou ".drive".
+				if (str != NULL)
+				{
+					*str = '\0';
+					if (strcmp (str+1, "link") == 0)  // pour les liens, on prend le nom du lien.
+					{
+						if (strcmp (cMountName, "root") == 0)  // on remplace 'root' par un nom plus parlant, sinon on prendra le nom du lien.
+						{
+							*cName = g_strdup ("/");
+						}
+					}
+					else if (strcmp (str+1, "drive") == 0)  // on cherche un nom plus parlant si possible.
+					{
+						gchar *cVolumeName = _cd_find_volume_name_from_drive_name (cMountName);
+						if (cVolumeName != NULL)
+						{
+							*cName = cVolumeName;
+						}
+					}
+				}
+				if (*cName == NULL)
+					*cName = cMountName;
+				//else
+					//g_free (cMountName);
+			}
+			if (*cName ==  NULL)
+				*cName = g_strdup (cFileName);
 		}
-		if (*cName ==  NULL)
+	}
+	else  // for an normal file, just re-use the filename
+	{
+		if (iVolumeID)
+			*iVolumeID = 0;
+		if (cName)
 			*cName = g_strdup (cFileName);
 	}
-	else
+	
+	// find the target URI if it's a mount point
+	if (cURI)
 	{
-		*iVolumeID = 0;
-		*cName = g_strdup (cFileName);
+		if (cTargetURI)
+		{
+			*cURI = g_strdup (cTargetURI);
+			g_free (cValidUri);
+			cValidUri = NULL;
+		}
+		else
+			*cURI = cValidUri;
 	}
 	
-	*cIconName = NULL;
-	if (cMimeType != NULL && strncmp (cMimeType, "image", 5) == 0)
+	// find an icon.
+	if (cIconName)
 	{
-		gchar *cHostname = NULL;
-		GError *erreur = NULL;
-		gchar *cFilePath = g_filename_from_uri (cBaseURI, &cHostname, &erreur);
-		if (erreur != NULL)
+		*cIconName = NULL;
+		// first use an available thumbnail.
+		*cIconName = g_strdup (g_file_info_get_attribute_byte_string (pFileInfo, G_FILE_ATTRIBUTE_THUMBNAIL_PATH));
+		#if (GLIB_MAJOR_VERSION > 2) || (GLIB_MAJOR_VERSION == 2 && GLIB_MINOR_VERSION >= 20)
+		/**if (*cIconName == NULL)
 		{
-			g_error_free (erreur);
-		}
-		else if (cHostname == NULL || strcmp (cHostname, "localhost") == 0)  // on ne recupere la vignette que sur les fichiers locaux.
+			GIcon *pPreviewIcon = (GIcon *)g_file_info_get_attribute_object (pFileInfo, G_FILE_ATTRIBUTE_PREVIEW_ICON);
+			if (pPreviewIcon != NULL)
+			{
+				*cIconName = _cd_get_icon_path (pPreviewIcon, NULL);
+				//g_print ("got preview icon '%s'\n", *cIconName);
+			}
+		}*/
+		#endif
+		// if no thumbnail available and the file is an image, use it directly.
+		if (*cIconName == NULL && cMimeType != NULL && strncmp (cMimeType, "image", 5) == 0)
 		{
-			*cIconName = g_strdup (cFilePath);
-			cairo_dock_remove_html_spaces (*cIconName);
+			gchar *cHostname = NULL;
+			GError *erreur = NULL;
+			gchar *cFilePath = g_filename_from_uri (cBaseURI, &cHostname, &erreur);
+			if (erreur != NULL)
+			{
+				g_error_free (erreur);
+			}
+			else if (cHostname == NULL || strcmp (cHostname, "localhost") == 0)  // on ne recupere la vignette que sur les fichiers locaux.
+			{
+				*cIconName = g_strdup (cFilePath);
+				cairo_dock_remove_html_spaces (*cIconName);
+			}
+			g_free (cHostname);
 		}
-		//g_free (cHostname);
+		// else, get the icon for the mime-types.
+		if (*cIconName == NULL)
+		{
+			GIcon *pSystemIcon = g_file_info_get_icon (pFileInfo);
+			if (pSystemIcon != NULL)
+			{
+				*cIconName = _cd_get_icon_path (pSystemIcon, cTargetURI ? cTargetURI : *cURI);
+			}
+		}
+		cd_message ("cIconName : %s", *cIconName);
 	}
-	if (*cIconName == NULL)
-	{
-		GIcon *pSystemIcon = g_file_info_get_icon (pFileInfo);
-		if (pSystemIcon != NULL)
-		{
-			*cIconName = _cd_get_icon_path (pSystemIcon, cTargetURI ? cTargetURI : *cURI);
-		}
-	}
-	cd_message ("cIconName : %s", *cIconName);
 	
-	//*iVolumeID = g_file_info_get_attribute_uint32 (pFileInfo, G_FILE_ATTRIBUTE_MOUNTABLE_UNIX_DEVICE);
-	//cd_message ("ID : %d\n", *iVolumeID);
 	g_object_unref (pFileInfo);
+	g_object_unref (pFile);
 }
 
 static Icon *_cd_get_icon_for_volume (GVolume *pVolume, GMount *pMount)
@@ -626,7 +694,7 @@ static GList *cairo_dock_gio_vfs_list_volumes (void)
 	return pIconsList;
 }
 
-static GList *cairo_dock_gio_vfs_list_directory (const gchar *cBaseURI, CairoDockFMSortType iSortType, int iNewIconsType, gboolean bListHiddenFiles, int iNbMaxFiles, gchar **cFullURI)
+static GList *cairo_dock_gio_vfs_list_directory (const gchar *cBaseURI, CairoDockFMSortType iSortType, int iNewIconsGroup, gboolean bListHiddenFiles, int iNbMaxFiles, gchar **cValidUri)
 {
 	g_return_val_if_fail (cBaseURI != NULL, NULL);
 	cd_message ("%s (%s)", __func__, cBaseURI);
@@ -637,7 +705,7 @@ static GList *cairo_dock_gio_vfs_list_directory (const gchar *cBaseURI, CairoDoc
 	{
 		cURI = g_strdup ("computer://");
 		bAddHome = TRUE;
-		///*cFullURI = cURI;
+		///*cValidUri = cURI;
 		///return cairo_dock_gio_vfs_list_volumes ();
 		//cairo_dock_gio_vfs_list_volumes ();
 	}
@@ -645,7 +713,7 @@ static GList *cairo_dock_gio_vfs_list_directory (const gchar *cBaseURI, CairoDoc
 		cURI = g_strdup ("network://");
 	else
 		cURI = (*cBaseURI == '/' ? g_strconcat ("file://", cBaseURI, NULL) : g_strdup (cBaseURI));
-	*cFullURI = cURI;
+	*cValidUri = cURI;
 	
 	GFile *pFile = g_file_new_for_uri (cURI);
 	GError *erreur = NULL;
@@ -656,6 +724,10 @@ static GList *cairo_dock_gio_vfs_list_directory (const gchar *cBaseURI, CairoDoc
 		G_FILE_ATTRIBUTE_STANDARD_NAME","
 		G_FILE_ATTRIBUTE_STANDARD_IS_HIDDEN","
 		G_FILE_ATTRIBUTE_STANDARD_ICON","
+		G_FILE_ATTRIBUTE_THUMBNAIL_PATH","
+		#if (GLIB_MAJOR_VERSION > 2) || (GLIB_MAJOR_VERSION == 2 && GLIB_MINOR_VERSION >= 20)
+		///G_FILE_ATTRIBUTE_PREVIEW_ICON","
+		#endif
 		G_FILE_ATTRIBUTE_STANDARD_TARGET_URI","
 		G_FILE_ATTRIBUTE_MOUNTABLE_UNIX_DEVICE;
 	GFileEnumerator *pFileEnum = g_file_enumerate_children (pFile,
@@ -665,7 +737,7 @@ static GList *cairo_dock_gio_vfs_list_directory (const gchar *cBaseURI, CairoDoc
 		&erreur);
 	if (erreur != NULL)
 	{
-		cd_warning ("gnome_integration : %s", erreur->message);
+		cd_warning ("gvfs-integration : %s", erreur->message);
 		g_error_free (erreur);
 		g_object_unref (pFile);
 		return NULL;
@@ -681,7 +753,7 @@ static GList *cairo_dock_gio_vfs_list_directory (const gchar *cBaseURI, CairoDoc
 		pFileInfo = g_file_enumerator_next_file (pFileEnum, NULL, &erreur);
 		if (erreur != NULL)
 		{
-			cd_warning ("gnome_integration : %s", erreur->message);
+			cd_warning ("gvfs-integration : %s", erreur->message);
 			g_error_free (erreur);
 			erreur = NULL;
 			continue;
@@ -705,9 +777,9 @@ static GList *cairo_dock_gio_vfs_list_directory (const gchar *cBaseURI, CairoDoc
 			
 			icon = cairo_dock_create_dummy_launcher (NULL, NULL, NULL, NULL, 0);
 			icon->iTrueType = CAIRO_DOCK_ICON_TYPE_FILE;
-			icon->iType = iNewIconsType;
-			icon->cBaseURI = g_strconcat (*cFullURI, "/", cFileName, NULL);
-			cd_message ("+ %s (mime:%s)", icon->cBaseURI, cMimeType);
+			icon->iGroup = iNewIconsGroup;
+			icon->cBaseURI = g_strconcat (*cValidUri, "/", cFileName, NULL);
+			//g_print	 ("+ %s (mime:%s)n", icon->cBaseURI, cMimeType);
 			
 			if (iFileType == G_FILE_TYPE_MOUNTABLE)
 			{
@@ -782,6 +854,18 @@ static GList *cairo_dock_gio_vfs_list_directory (const gchar *cBaseURI, CairoDoc
 				icon->cCommand = g_strdup (icon->cBaseURI);
 			icon->cName = cName;
 			icon->cFileName = NULL;
+			icon->cFileName = g_strdup (g_file_info_get_attribute_byte_string (pFileInfo, G_FILE_ATTRIBUTE_THUMBNAIL_PATH));
+			#if (GLIB_MAJOR_VERSION > 2) || (GLIB_MAJOR_VERSION == 2 && GLIB_MINOR_VERSION >= 20)
+			/**if (icon->cFileName == NULL)
+			{
+				GIcon *pPreviewIcon = (GIcon *)g_file_info_get_attribute_object (pFileInfo, G_FILE_ATTRIBUTE_PREVIEW_ICON);
+				if (pPreviewIcon != NULL)
+				{
+					icon->cFileName = _cd_get_icon_path (pPreviewIcon, NULL);
+					// g_print ("got preview icon '%s'\n", icon->cFileName);
+				}
+			}*/
+			#endif
 			if (cMimeType != NULL && strncmp (cMimeType, "image", 5) == 0)
 			{
 				gchar *cHostname = NULL;
@@ -843,12 +927,12 @@ static GList *cairo_dock_gio_vfs_list_directory (const gchar *cBaseURI, CairoDoc
 		
 		icon = cairo_dock_create_dummy_launcher (g_strdup ("home"),
 			g_strdup (pRootIcon->cFileName),
-			g_strdup ("/home"),
+			g_strdup (g_getenv ("HOME")),
 			NULL,
 			iOrder++);
 		icon->iTrueType = CAIRO_DOCK_ICON_TYPE_FILE;
-		icon->iType = iNewIconsType;
-		icon->cBaseURI = g_strdup_printf ("file://%s", "/home");
+		icon->iGroup = iNewIconsGroup;
+		icon->cBaseURI = g_strdup_printf ("file://%s", g_getenv ("HOME"));
 		icon->iVolumeID = 0;
 		
 		pIconList = g_list_insert_sorted (pIconList,
@@ -884,7 +968,7 @@ static gsize cairo_dock_gio_vfs_measure_directory (const gchar *cBaseURI, gint i
 		&erreur);
 	if (erreur != NULL)
 	{
-		cd_warning ("gnome_integration : %s", erreur->message);
+		cd_warning ("gvfs-integration : %s", erreur->message);
 		g_error_free (erreur);
 		g_object_unref (pFile);
 		return 0;
@@ -898,7 +982,7 @@ static gsize cairo_dock_gio_vfs_measure_directory (const gchar *cBaseURI, gint i
 		pFileInfo = g_file_enumerator_next_file (pFileEnum, NULL, &erreur);
 		if (erreur != NULL)
 		{
-			cd_warning ("gnome_integration : %s", erreur->message);
+			cd_warning ("gvfs-integration : %s", erreur->message);
 			g_error_free (erreur);
 			erreur = NULL;
 			continue;
@@ -969,21 +1053,65 @@ static gchar *_cd_find_target_uri (const gchar *cBaseURI)
 static void cairo_dock_gio_vfs_launch_uri (const gchar *cURI)
 {
 	g_return_if_fail (cURI != NULL);
-	GError *erreur = NULL;
-	gchar *cFullURI = (*cURI == '/' ? g_strconcat ("file://", cURI, NULL) : g_strdup (cURI));
-	cd_message ("%s (%s)", __func__, cFullURI);
 	
-	gchar *cTargetURI = _cd_find_target_uri (cFullURI);
-	gboolean bSuccess = g_app_info_launch_default_for_uri (cTargetURI != NULL ? cTargetURI : cFullURI,
+	// in case it's a mount point, take the URI that can actually be launched. 
+	GError *erreur = NULL;
+	gchar *cValidUri = (*cURI == '/' ? g_strconcat ("file://", cURI, NULL) : g_strdup (cURI));
+	cd_message ("%s (%s)", __func__, cValidUri);
+	
+	gchar *cTargetURI = _cd_find_target_uri (cValidUri);
+	cURI= (cTargetURI ? cTargetURI : cValidUri);
+	
+	// now, try to launch it with the default program know by gvfs.
+	gboolean bSuccess = g_app_info_launch_default_for_uri (cURI,
 		NULL,
 		&erreur);
-	g_free (cFullURI);
-	g_free (cTargetURI);
-	if (erreur != NULL)
+	if (erreur != NULL)  // error can happen (for instance, opening 'trash:/' on XFCE with a previous installation of nautilus) => try with another method.
 	{
-		cd_warning ("gnome_integration : couldn't launch '%s' [%s]", cURI, erreur->message);
+		cd_debug ("gvfs-integration : couldn't launch '%s' [%s]", cURI, erreur->message);
 		g_error_free (erreur);
+		erreur = NULL;
+		
+		// get the mime-type.
+		GFile *pFile = (*cURI == '/' ? g_file_new_for_path (cURI) : g_file_new_for_uri (cURI));
+		const gchar *cQuery = G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE;
+		GFileInfo *pFileInfo = g_file_query_info (pFile,
+			cQuery,
+			G_FILE_QUERY_INFO_NONE,
+			NULL,
+			&erreur);
+		if (erreur != NULL)  // if no mime-type (can happen with not mounted volumes), abort.
+		{
+			cd_warning ("gvfs-integration : %s", erreur->message);
+			g_error_free (erreur);
+		}
+		else
+		{
+			// get all the apps that can launch it.
+			const gchar *cMimeType = g_file_info_get_content_type (pFileInfo);
+			GList *pAppsList = g_app_info_get_all_for_type (cMimeType);
+			gchar *cCommand;
+			GAppInfo *pAppInfo;
+			const char *cExec;
+			GList *a;
+			for (a = pAppsList; a != NULL; a = a->next)
+			{
+				pAppInfo = a->data;
+				cExec = g_app_info_get_executable (pAppInfo);
+				if (cExec)  // use the first available. it is supposed to be the most suitable, but the default method doesn't seem to use it.
+				{
+					gchar *cPath = g_filename_from_uri (cURI, NULL, NULL);
+					cairo_dock_launch_command_printf ("%s \"%s\"", NULL, cExec, cPath?cPath:cURI);  // in case the program doesn't handle URI (geeqie, etc).
+					g_free (cPath);
+					break;
+				}
+			}
+			g_list_free (pAppsList);
+		}
+		g_object_unref (pFile);
 	}
+	g_free (cValidUri);
+	g_free (cTargetURI);
 }
 
 static GMount *_cd_find_mount_from_uri (const gchar *cURI, gchar **cTargetURI)
@@ -1098,7 +1226,7 @@ static void _gio_vfs_mount_callback (gpointer pObject, GAsyncResult *res, gpoint
 		bSuccess = g_mount_eject_finish (G_MOUNT (pObject), res, &erreur);
 	if (erreur != NULL)
 	{
-		cd_warning ("gnome-integration : %s", erreur->message);
+		cd_warning ("gvfs-integration : %s", erreur->message);
 		g_error_free (erreur);
 	}
 	
@@ -1253,7 +1381,7 @@ static void cairo_dock_gio_vfs_add_monitor (const gchar *cURI, gboolean bDirecto
 	//g_object_unref (pFile);
 	if (erreur != NULL)
 	{
-		cd_warning ("gnome-integration : couldn't add monitor on '%s' (%d) [%s]", cURI, bDirectory, erreur->message);
+		cd_warning ("gvfs-integration : couldn't add monitor on '%s' (%d) [%s]", cURI, bDirectory, erreur->message);
 		g_error_free (erreur);
 		return ;
 	}
@@ -1296,7 +1424,7 @@ static gboolean cairo_dock_gio_vfs_delete_file (const gchar *cURI, gboolean bNoT
 			&erreur);
 		if (erreur != NULL)
 		{
-			cd_warning ("gnome_integration : %s", erreur->message);
+			cd_warning ("gvfs-integration : %s", erreur->message);
 			g_error_free (erreur);
 			g_object_unref (pFile);
 			return FALSE;
@@ -1305,13 +1433,13 @@ static gboolean cairo_dock_gio_vfs_delete_file (const gchar *cURI, gboolean bNoT
 		GFileType iFileType = g_file_info_get_file_type (pFileInfo);
 		if (iFileType == G_FILE_TYPE_DIRECTORY)
 		{
-			cairo_dock_gio_vfs_empty_dir (cURI);
+			_cairo_dock_gio_vfs_empty_dir (cURI);
 		}
 		
 		bSuccess = g_file_delete (pFile, NULL, &erreur);
 		if (erreur != NULL)
 		{
-			cd_warning ("gnome-integration : %s", erreur->message);
+			cd_warning ("gvfs-integration : %s", erreur->message);
 			g_error_free (erreur);
 		}
 	}
@@ -1320,7 +1448,7 @@ static gboolean cairo_dock_gio_vfs_delete_file (const gchar *cURI, gboolean bNoT
 		bSuccess = g_file_trash (pFile, NULL, &erreur);
 		if (erreur != NULL)
 		{
-			cd_warning ("gnome-integration : %s", erreur->message);
+			cd_warning ("gvfs-integration : %s", erreur->message);
 			g_error_free (erreur);
 		}
 	}
@@ -1336,7 +1464,7 @@ static gboolean cairo_dock_gio_vfs_rename_file (const gchar *cOldURI, const gcha
 	GFile *pNewFile = g_file_set_display_name (pOldFile, cNewName, NULL, &erreur);
 	if (erreur != NULL)
 	{
-		cd_warning ("gnome-integration : %s", erreur->message);
+		cd_warning ("gvfs-integration : %s", erreur->message);
 		g_error_free (erreur);
 	}
 	gboolean bSuccess = (pNewFile != NULL);
@@ -1368,7 +1496,7 @@ static gboolean cairo_dock_gio_vfs_move_file (const gchar *cURI, const gchar *cD
 		&erreur);
 	if (erreur != NULL)
 	{
-		cd_warning ("gnome-integration : %s", erreur->message);
+		cd_warning ("gvfs-integration : %s", erreur->message);
 		g_error_free (erreur);
 	}
 	g_object_unref (pFile);
@@ -1391,7 +1519,7 @@ static gboolean cairo_dock_gio_vfs_create_file (const gchar *cURI, gboolean bDir
 		g_file_create (pFile, G_FILE_CREATE_PRIVATE, NULL, &erreur);
 	if (erreur != NULL)
 	{
-		cd_warning ("gnome-integration : %s", erreur->message);
+		cd_warning ("gvfs-integration : %s", erreur->message);
 		g_error_free (erreur);
 		bSuccess = FALSE;
 	}
@@ -1420,7 +1548,7 @@ static void cairo_dock_gio_vfs_get_file_properties (const gchar *cURI, guint64 *
 		&erreur);
 	if (erreur != NULL)
 	{
-		cd_warning ("gnome-integration : couldn't get file properties for '%s' [%s]", cURI, erreur->message);
+		cd_warning ("gvfs-integration : couldn't get file properties for '%s' [%s]", cURI, erreur->message);
 		g_error_free (erreur);
 	}
 	
@@ -1471,7 +1599,7 @@ static gchar *cairo_dock_gio_vfs_get_desktop_path (void)
 	return cPath;
 }
 
-static void cairo_dock_gio_vfs_empty_dir (const gchar *cBaseURI)
+static void _cairo_dock_gio_vfs_empty_dir (const gchar *cBaseURI)
 {
 	if (cBaseURI == NULL)
 		return ;
@@ -1487,7 +1615,7 @@ static void cairo_dock_gio_vfs_empty_dir (const gchar *cBaseURI)
 		&erreur);
 	if (erreur != NULL)
 	{
-		cd_warning ("gnome_integration : %s", erreur->message);
+		cd_warning ("gvfs-integration : %s", erreur->message);
 		g_object_unref (pFile);
 		g_error_free (erreur);
 		return ;
@@ -1501,7 +1629,7 @@ static void cairo_dock_gio_vfs_empty_dir (const gchar *cBaseURI)
 		pFileInfo = g_file_enumerator_next_file (pFileEnum, NULL, &erreur);
 		if (erreur != NULL)
 		{
-			cd_warning ("gnome_integration : %s", erreur->message);
+			cd_warning ("gvfs-integration : %s", erreur->message);
 			g_error_free (erreur);
 			erreur = NULL;
 			continue;
@@ -1515,14 +1643,14 @@ static void cairo_dock_gio_vfs_empty_dir (const gchar *cBaseURI)
 		g_string_printf (sFileUri, "%s/%s", cBaseURI, cFileName);
 		if (iFileType == G_FILE_TYPE_DIRECTORY)
 		{
-			cairo_dock_gio_vfs_empty_dir (sFileUri->str);
+			_cairo_dock_gio_vfs_empty_dir (sFileUri->str);
 		}
 		
 		file = (*cBaseURI == '/' ? g_file_new_for_path (sFileUri->str) : g_file_new_for_uri (sFileUri->str));
 		g_file_delete (file, NULL, &erreur);
 		if (erreur != NULL)
 		{
-			cd_warning ("gnome_integration : %s", erreur->message);
+			cd_warning ("gvfs-integration : %s", erreur->message);
 			g_error_free (erreur);
 			erreur = NULL;
 		}
@@ -1550,7 +1678,8 @@ static void cairo_dock_gio_vfs_empty_trash (void)
 	GFile *pFile = g_file_new_for_uri ("trash://");
 	GError *erreur = NULL;
 	const gchar *cAttributes = G_FILE_ATTRIBUTE_STANDARD_TARGET_URI","
-		G_FILE_ATTRIBUTE_STANDARD_NAME;
+		G_FILE_ATTRIBUTE_STANDARD_NAME","
+		G_FILE_ATTRIBUTE_STANDARD_TYPE;
 	GFileEnumerator *pFileEnum = g_file_enumerate_children (pFile,
 		cAttributes,
 		G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS,
@@ -1558,7 +1687,7 @@ static void cairo_dock_gio_vfs_empty_trash (void)
 		&erreur);
 	if (erreur != NULL)
 	{
-		cd_warning ("gnome_integration : %s", erreur->message);
+		cd_warning ("gvfs-integration : %s", erreur->message);
 		g_object_unref (pFile);
 		g_error_free (erreur);
 		return ;
@@ -1572,7 +1701,7 @@ static void cairo_dock_gio_vfs_empty_trash (void)
 		pFileInfo = g_file_enumerator_next_file (pFileEnum, NULL, &erreur);
 		if (erreur != NULL)
 		{
-			cd_warning ("gnome_integration : %s", erreur->message);
+			cd_warning ("gvfs-integration : %s", erreur->message);
 			g_error_free (erreur);
 			erreur = NULL;
 			continue;
@@ -1581,15 +1710,20 @@ static void cairo_dock_gio_vfs_empty_trash (void)
 			break ;
 		
 		const gchar *cFileName = g_file_info_get_name (pFileInfo);
-		g_print (" - %s\n", cFileName);
+		//g_print (" - %s\n", cFileName);
 		
 		// il y'a 2 cas : un fichier dans la poubelle du home, et un fichier dans une poubelle d'un autre volume.
 		if (cFileName && *cFileName == '\\')  // nom de la forme "\media\Fabounet2\.Trash-1000\files\t%C3%A8st%201" et URI "trash:///%5Cmedia%5CFabounet2%5C.Trash-1000%5Cfiles%5Ct%25C3%25A8st%25201", mais cette URI ne marche pas des qu'il y'a des caracteres non ASCII-7 dans le nom (bug dans gio/gvfs ?). Donc on feinte, en construisant le chemin du fichier (et de son double dans 'info').
 		{
 			g_string_printf (sFileUri, "file://%s", cFileName);
 			g_strdelimit (sFileUri->str, "\\", '/');
-			g_print ("   - %s\n", sFileUri->str);
+			//g_print ("   - %s\n", sFileUri->str);
 			
+			GFileType iFileType = g_file_info_get_file_type (pFileInfo);
+			if (iFileType == G_FILE_TYPE_DIRECTORY)  // can't delete a non-empty folder located on a different volume than home.
+			{
+				_cairo_dock_gio_vfs_empty_dir (sFileUri->str);
+			}
 			GFile *file = g_file_new_for_uri (sFileUri->str);
 			g_file_delete (file, NULL, &erreur);
 			g_object_unref (file);
@@ -1599,19 +1733,28 @@ static void cairo_dock_gio_vfs_empty_trash (void)
 			{
 				*str = '\0';
 				gchar *cInfo = g_strdup_printf ("%s/info/%s.trashinfo", sFileUri->str, str+7);
-				g_print ("   - %s\n", cInfo);
+				//g_print ("   - %s\n", cInfo);
 				file = g_file_new_for_uri (cInfo);
 				g_free (cInfo);
-				g_file_delete (file, NULL, &erreur);
+				g_file_delete (file, NULL, NULL);
 				g_object_unref (file);
 			}
 		}
-		else  // poubelle principale : nom de la forme "tèst 1" et URI "trash:///t%C3%A8st%201"
+		else  // poubelle principale : nom de la forme "tȿst 1" et URI "trash:///t%C3%A8st%201"
 		{
-			g_string_printf (sFileUri, "trash:///%s", cFileName);
+			if (strchr (cFileName, '%'))  // if there is a % inside the name, it disturb gio, so let's remove it.
+			{
+				gchar *cTmpPath = g_strdup_printf ("/%s", cFileName);
+				gchar *cEscapedFileName = g_filename_to_uri (cTmpPath, NULL, NULL);
+				g_free (cTmpPath);
+				g_string_printf (sFileUri, "trash://%s", cEscapedFileName+7);  // replace file:// with trash://
+				g_free (cEscapedFileName);
+			}
+			else  // else it can handle the URI as usual.
+				g_string_printf (sFileUri, "trash:///%s", cFileName);
 			GFile *file = g_file_new_for_uri (sFileUri->str);
 			/*gchar *cValidURI = g_file_get_uri (file);
-			g_print ("   - %s\n", cValidURI);
+			//g_print ("   - %s\n", cValidURI);
 			g_object_unref (file);
 			
 			file = g_file_new_for_uri (cValidURI);
@@ -1621,7 +1764,7 @@ static void cairo_dock_gio_vfs_empty_trash (void)
 		}
 		if (erreur != NULL)
 		{
-			cd_warning ("gnome_integration : %s", erreur->message);
+			cd_warning ("gvfs-integration : %s", erreur->message);
 			g_error_free (erreur);
 			erreur = NULL;
 		}
@@ -1636,12 +1779,12 @@ static void cairo_dock_gio_vfs_empty_trash (void)
 
 static GList *cairo_dock_gio_vfs_list_apps_for_file (const gchar *cBaseURI)
 {
-	gchar *cFullURI;
+	gchar *cValidUri;
 	if (*cBaseURI == '/')
-		cFullURI = g_filename_to_uri (cBaseURI, NULL, NULL);
+		cValidUri = g_filename_to_uri (cBaseURI, NULL, NULL);
 	else
-		cFullURI = g_strdup (cBaseURI);
-	GFile *pFile = g_file_new_for_uri (cFullURI);
+		cValidUri = g_strdup (cBaseURI);
+	GFile *pFile = g_file_new_for_uri (cValidUri);
 	
 	GError *erreur = NULL;
 	const gchar *cQuery = G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE;
@@ -1653,9 +1796,9 @@ static GList *cairo_dock_gio_vfs_list_apps_for_file (const gchar *cBaseURI)
 	
 	if (erreur != NULL)  // peut arriver si l'emplacement n'est pas monte, mais on signale tout de meme la raison avec un warning.
 	{
-		cd_warning ("gnome_integration : %s", erreur->message);
+		cd_warning ("gvfs-integration : %s", erreur->message);
 		g_error_free (erreur);
-		g_free (cFullURI);
+		g_free (cValidUri);
 		g_object_unref (pFile);
 		return NULL;
 	}
@@ -1689,8 +1832,9 @@ static GList *cairo_dock_gio_vfs_list_apps_for_file (const gchar *cBaseURI)
 		#endif
 		pList = g_list_prepend (pList, pData);
 	}
+	pList = g_list_reverse (pList);
 	
-	g_free (cFullURI);
+	g_free (cValidUri);
 	g_object_unref (pFile);
 	g_list_free (pAppsList);
 	g_object_unref (pFileInfo);
