@@ -79,7 +79,7 @@ static gchar *_cd_get_icon_path (GIcon *pIcon)
 		for (i = 0; cFileNames[i] != NULL && cIconPath == NULL; i ++)
 		{
 			//cd_message (" une icone possible est : %s\n", cFileNames[i]);
-			cIconPath = cairo_dock_search_icon_s_path (cFileNames[i]);
+			cIconPath = cairo_dock_search_icon_s_path (cFileNames[i], CAIRO_DOCK_DEFAULT_ICON_SIZE);
 			//cd_message ("  chemin trouve : %s\n", cIconPath);
 		}
 	}
@@ -203,7 +203,7 @@ static GDrive *_cd_find_drive_from_name (const gchar *cName)
 static gchar *_cd_find_volume_name_from_drive_name (const gchar *cName)
 {
 	g_return_val_if_fail (cName != NULL, NULL);
-	cd_message ("%s (%s)", __func__, cName);
+	cd_debug ("%s (%s)", __func__, cName);
 	GDrive *pDrive = _cd_find_drive_from_name (cName);
 	g_return_val_if_fail (pDrive != NULL, NULL);
 	
@@ -215,12 +215,12 @@ static gchar *_cd_find_volume_name_from_drive_name (const gchar *cName)
 		GList *av;
 		if (pAssociatedVolumes->next != NULL)
 		{
-			cd_message ("ce disque contient plus d'un volume, on garde le nom du disque plutot que de selectionner le nom d'un volume");
-			cd_message ("Pour info, la liste des volumes disponibles sur ce disque est :");
+			cd_debug ("ce disque contient plus d'un volume, on garde le nom du disque plutot que de selectionner le nom d'un volume");
+			cd_debug ("Pour info, la liste des volumes disponibles sur ce disque est :");
 			for (av = pAssociatedVolumes; av != NULL; av = av->next)
 			{
 				pVolume = av->data;
-				cd_message ("  - %s", g_volume_get_name  (pVolume));
+				cd_debug ("  - %s", g_volume_get_name  (pVolume));
 				/*if (cVolumeName == NULL)
 					cVolumeName = g_volume_get_name  (pVolume);
 				else
@@ -234,7 +234,7 @@ static gchar *_cd_find_volume_name_from_drive_name (const gchar *cName)
 			pVolume = pAssociatedVolumes->data;
 			cVolumeName = g_volume_get_name  (pVolume);
 			g_object_unref (pVolume);
-			cd_message ("ce disque contient 1 seul volume (%s), on prend son nom", cVolumeName);
+			cd_debug ("ce disque contient 1 seul volume (%s), on prend son nom", cVolumeName);
 		}
 		g_list_free (pAssociatedVolumes);
 	}
@@ -633,7 +633,7 @@ GList *vfs_backend_list_directory (const gchar *cBaseURI, CairoDockFMSortType iS
 			
 			icon = cairo_dock_create_dummy_launcher (NULL, NULL, NULL, NULL, 0);
 			icon->iTrueType = CAIRO_DOCK_ICON_TYPE_FILE;
-			icon->iType = iNewIconsType;
+			icon->iGroup = iNewIconsType;
 			icon->cBaseURI = g_strconcat (*cFullURI, "/", cFileName, NULL);
 			cd_message ("+ %s (mime:%s)", icon->cBaseURI, cMimeType);
 			
@@ -769,7 +769,7 @@ GList *vfs_backend_list_directory (const gchar *cBaseURI, CairoDockFMSortType iS
 			NULL,
 			iOrder++);
 		icon->iTrueType = CAIRO_DOCK_ICON_TYPE_FILE;
-		icon->iType = iNewIconsType;
+		icon->iGroup = iNewIconsType;
 		icon->cBaseURI = g_strdup_printf ("file://%s", "/home");
 		icon->iVolumeID = 0;
 		pIconList = g_list_insert_sorted (pIconList,
