@@ -139,9 +139,6 @@ static void _cd_mail_mark_all_as_read (GtkMenuItem *menu_item, CairoDockModuleIn
 	_cd_mail_force_update(myApplet);
 }
 CD_APPLET_ON_BUILD_MENU_BEGIN
-	GtkWidget *pSubMenu = CD_APPLET_CREATE_MY_SUB_MENU ();
-	
-	// Main Menu
 	if(myData.pMailAccounts && myData.pMailAccounts->len > 0)
 	{
 		if (myData.pMailAccounts->len > 1)  // many accounts -> list them in a sub-menu
@@ -174,11 +171,6 @@ CD_APPLET_ON_BUILD_MENU_BEGIN
 		CD_APPLET_ADD_IN_MENU_WITH_STOCK (cLabel, GTK_STOCK_EXECUTE, _cd_mail_launch_mail_appli, CD_APPLET_MY_MENU);
 		g_free (cLabel);
 	}
-	
-	// Sub-Menu
-	if (pSubMenu == CD_APPLET_MY_MENU)
-		CD_APPLET_ADD_SEPARATOR_IN_MENU (CD_APPLET_MY_MENU);
-	CD_APPLET_ADD_ABOUT_IN_MENU (pSubMenu);
 CD_APPLET_ON_BUILD_MENU_END
 
 void _cd_mail_show_current_mail(CDMailAccount *pMailAccount)
@@ -264,7 +256,7 @@ GtkWidget *cd_mail_messages_container_new(CDMailAccount *pMailAccount)
 	 * | <--       CLOSE       --> |    <---- 3 buttons, attached Left,Center,Right
 	 * -----------------------------
 	 */
-	GtkWidget *vbox = gtk_vbox_new(FALSE, 0);
+	GtkWidget *vbox = _gtk_vbox_new (0);
 
 	GtkWidget *pTextView = gtk_text_view_new();
 	gtk_text_view_set_editable(GTK_TEXT_VIEW(pTextView), FALSE);
@@ -278,8 +270,8 @@ GtkWidget *cd_mail_messages_container_new(CDMailAccount *pMailAccount)
 	gtk_scrolled_window_set_policy( GTK_SCROLLED_WINDOW(pScrolledWindow), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC );
 	
 	gtk_box_pack_start(GTK_BOX(vbox), pScrolledWindow, TRUE, TRUE, 0);
-	
-	GtkWidget *hbox = gtk_hbox_new(TRUE, 0);
+
+	GtkWidget *hbox = _gtk_hbox_new(0);
 	myData.pPrevButton = gtk_button_new_from_stock( GTK_STOCK_GO_BACK );
 	GtkWidget *pCloseButton = gtk_button_new_from_stock( GTK_STOCK_CLOSE );
 	myData.pNextButton = gtk_button_new_from_stock( GTK_STOCK_GO_FORWARD );
@@ -291,9 +283,9 @@ GtkWidget *cd_mail_messages_container_new(CDMailAccount *pMailAccount)
 	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, TRUE, 0);
 	
 	// then we need to put the callbacks
-	gtk_signal_connect( GTK_OBJECT(myData.pPrevButton), "clicked", G_CALLBACK(_cd_mail_show_prev_mail_cb), (gpointer)pMailAccount );
-	gtk_signal_connect( GTK_OBJECT(myData.pNextButton), "clicked", G_CALLBACK(_cd_mail_show_next_mail_cb), (gpointer)pMailAccount );
-	gtk_signal_connect( GTK_OBJECT(pCloseButton),       "clicked", G_CALLBACK(_cd_mail_close_preview_cb), (gpointer)pMailAccount );
+	g_signal_connect( G_OBJECT(myData.pPrevButton), "clicked", G_CALLBACK(_cd_mail_show_prev_mail_cb), (gpointer)pMailAccount );
+	g_signal_connect( G_OBJECT(myData.pNextButton), "clicked", G_CALLBACK(_cd_mail_show_next_mail_cb), (gpointer)pMailAccount );
+	g_signal_connect( G_OBJECT(pCloseButton),       "clicked", G_CALLBACK(_cd_mail_close_preview_cb), (gpointer)pMailAccount );
 
 	GList *l = pMailAccount->pUnseenMessageList;
 	const gchar *cMessage = "";
