@@ -191,7 +191,7 @@ static inline void _extract_metadata (GHashTable *data_list)
 	else myData.cPlayingUri = NULL;
 	cd_message ("  cUri <- %s", myData.cPlayingUri);
 	
-	cd_musicplayer_get_cover_path (NULL, TRUE);
+	cd_musicplayer_set_cover_path (NULL);
 }
 
 /* Recupere les infos de la chanson courante, y compris le chemin de la couverture (la telecharge au besoin).
@@ -263,7 +263,7 @@ static void onChangeSong(DBusGProxy *player_proxy, GHashTable *metadata, gpointe
 		myData.iTrackNumber = 0;
 		myData.cover_exist = FALSE;
 	}
-	cd_musicplayer_update_icon (TRUE);
+	cd_musicplayer_update_icon ();
 	CD_APPLET_LEAVE ();
 }
 
@@ -302,7 +302,6 @@ static void cd_audacious_control (MyPlayerControl pControl, const char* song)
 		case PLAYER_SHUFFLE :
 		case PLAYER_REPEAT :
 		{
-			
 			DBusGProxy *dbus_proxy_atheme = cairo_dock_create_new_session_proxy (
 				"org.atheme.audacious",
 				"/org/atheme/audacious",
@@ -344,6 +343,22 @@ static void cd_audacious_control (MyPlayerControl pControl, const char* song)
 				G_TYPE_INVALID);
 		break;
 		
+		case PLAYER_VOLUME :
+		{
+			int iVolume = cd_mpris_get_volume ();  // [0, 100]
+			if (song && strcmp (song, "up") == 0)
+				iVolume += 5;
+			else
+				iVolume -= 5;
+			
+			if (iVolume > 100)
+				iVolume = 100;
+			else if (iVolume < 0)
+			iVolume = 0;
+			
+			cd_mpris_set_volume (iVolume);
+		}
+		
 		default :
 			return;
 		break;
@@ -391,7 +406,7 @@ static void cd_audacious_start (void)
 	// get the current state.
 	cd_mpris_getPlaying ();  ///_audacious_getPlaying_mpris ();
 	cd_audacious_getSongInfos ();
-	cd_musicplayer_update_icon (TRUE);
+	cd_musicplayer_update_icon ();
 }
 
 /* On enregistre notre lecteur.
@@ -425,8 +440,9 @@ void cd_musicplayer_register_audacious_handler (void)
 	pHandler->appclass = "audacious";  // desktop file is audasious2.desktop, and command is audacious2, but audacious also exists, and the class is Audacious ... this is really not serious.
 	pHandler->launch = "audacious";
 	pHandler->cMprisService = "org.mpris.audacious";
+	pHandler->cMpris2Service = "org.mpris.MediaPlayer2.audacious";
 	pHandler->bSeparateAcquisition = FALSE;
-	pHandler->iPlayerControls = PLAYER_PREVIOUS | PLAYER_PLAY_PAUSE | PLAYER_NEXT | PLAYER_STOP | PLAYER_JUMPBOX | PLAYER_SHUFFLE | PLAYER_REPEAT | PLAYER_ENQUEUE;
+	pHandler->iPlayerControls = PLAYER_PREVIOUS | PLAYER_PLAY_PAUSE | PLAYER_NEXT | PLAYER_STOP | PLAYER_JUMPBOX | PLAYER_SHUFFLE | PLAYER_REPEAT | PLAYER_ENQUEUE | PLAYER_VOLUME;
 	pHandler->iLevel = PLAYER_GOOD;
 	
 	pHandler->cMprisService = "org.mpris.audacious";
