@@ -29,7 +29,6 @@
 #include "applet-dbus.h" 
 #include "3dcover-draw.h"
 
-#include "applet-xmms.h" //Support XMMS
 #include "applet-xmms2.h" //Support XMMS2
 #include "applet-exaile.h" //Support Exaile
 #include "applet-exaile3.h" //Support Exaile 0.3
@@ -44,26 +43,31 @@
 #include "applet-amarok2.h" //Support Amarok 2
 #include "applet-audacious.h" //Support Audacious
 #include "applet-clementine.h" // Support Clementine
+#include "applet-mpris2.h" // Support Clementine
 
-CD_APPLET_DEFINITION (N_("musicPlayer"),
+CD_APPLET_DEFINE_BEGIN (N_("musicPlayer"),
 	2,0,0,
 	CAIRO_DOCK_CATEGORY_APPLET_ACCESSORY,
 	N_("This applet lets you control any music player.\n"
-	"Left click to Play/Pause, middle-click to play Next song.\n"
-	"Scroll up/down to play previous/next song.\n"
+	"First choose the player you want to control.\n"
+	"<b>click</b> to show/hide the player or pause,\n"
+	"<b>middle-click</b> to pause or go to next song,\n"
+	"<b>Scroll up/down</b> to change the song or control the volume.\n"
 	"You can drag and drop songs on the icon to put them in the queue (depends on Player),\n"
 	" and jpeg image to use as cover.\n"
-	"Note : For XMMS, you have to install the 'xmms-infopipe' plug-in.\n"
-	"       For SongBird, you have to install its dbus add-on.\n"
-	"       For Exaile 0.3, you have to activate its MPRIS plug-in"),
+	"Note: you may have to install or activate the MPRIS plug-in of the player."),
 	"ChanGFu (Rémy Robertson), Mav (Yann SLADEK), Tofe, Jackass, Nochka85, Fabounet")
+	CD_APPLET_DEFINE_COMMON_APPLET_INTERFACE
+	CD_APPLET_ALLOW_EMPTY_TITLE
+	CD_APPLET_ACT_AS_LAUNCHER
+CD_APPLET_DEFINE_END
 
 
 //\___________ Here is where you initiate your applet. myConfig is already set at this point, and also myIcon, myContainer, myDock, myDesklet (and myDrawContext if you're in dock mode). The macro CD_APPLET_MY_CONF_FILE and CD_APPLET_MY_KEY_FILE can give you access to the applet's conf-file and its corresponding key-file (also available during reload). If you're in desklet mode, myDrawContext is still NULL, and myIcon's buffers has not been filled, because you may not need them then (idem when reloading).
 CD_APPLET_INIT_BEGIN
-	// Add here all player's registering functions
-	// Don't forget to add the registered Name in ../data/musicPlayer.conf.in
-	cd_musicplayer_register_xmms_handler ();
+	// Register the players
+	// Note: with MPRIS2, we don't need to register a custom handler for each player. We only register handlers of old interfaces (MPRIS1 and before).
+	// Once all players use MPRIS2, we can drop all handlers but the MPRIS2 one.
 	cd_musicplayer_register_exaile_handler();
 	cd_musicplayer_register_exaile3_handler();
 	cd_musicplayer_register_songbird_handler();
@@ -78,12 +82,13 @@ CD_APPLET_INIT_BEGIN
 	cd_musicplayer_register_guayadeque_handler();
 	cd_musicplayer_register_clementine_handler();
 	cd_musicplayer_register_gmusicbrowser_handler();
+	cd_musicplayer_register_mpris2_handler();
 	
 	gchar *cCoverPath = g_strdup_printf ("%s/musicplayer", g_cCairoDockDataDir);
 	if (! g_file_test (cCoverPath, G_FILE_TEST_EXISTS))
 	{
 		if (g_mkdir (cCoverPath, 7*8*8+7*8+5) != 0)
-			cd_warning ("couldn't create directory %s", cCoverPath);
+			cd_warning ("couldn't create directory %s to download covers", cCoverPath);
 	}
 	g_free (cCoverPath);
 	
@@ -93,7 +98,9 @@ CD_APPLET_INIT_BEGIN
 		CD_APPLET_SET_DESKLET_RENDERER ("Simple");
 	}
 	else if (myIcon->cName == NULL || *myIcon->cName == '\0')
-		CD_APPLET_SET_NAME_FOR_MY_ICON (myConfig.cMusicPlayer);
+	{
+		CD_APPLET_SET_NAME_FOR_MY_ICON (cd_musicplayer_get_string_with_first_char_to_upper (myConfig.cMusicPlayer));
+	}
 	
 	
 	//\_______________ on charge le theme 3D si necessaire.
@@ -108,17 +115,7 @@ CD_APPLET_INIT_BEGIN
 	myData.iPreviousTrackNumber = -1;
 	myData.iPreviousCurrentTime = -1;
 	
-	myData.pCurrentHandeler = cd_musicplayer_get_handler_by_name (myConfig.cMusicPlayer);
-	if (myData.pCurrentHandeler == NULL) {
- 		cd_warning ("MP : this player (%s) is not supported.", myConfig.cMusicPlayer); 
- 		return; 
-	}
-	
-	cd_musicplayer_launch_handler ();  // connexion au bus, detection de l'appli, recuperation de l'etat du lecteur si il est en marche, sinon dessin de l'icone "eteint".
-	
-	//\_______________ On prend en charge l'icone de l'appli player.
-	if (myConfig.bStealTaskBarIcon)
-		CD_APPLET_MANAGE_APPLICATION (myData.pCurrentHandeler->appclass);
+	cd_musicplayer_set_current_handler (myConfig.cMusicPlayer);
 	
 	//\_______________ On s'abonne aux notifications.
 	CD_APPLET_REGISTER_FOR_CLICK_EVENT;
@@ -129,7 +126,7 @@ CD_APPLET_INIT_BEGIN
 	if (CD_APPLET_MY_CONTAINER_IS_OPENGL && myConfig.bOpenglThemes)
 	{
 		CD_APPLET_REGISTER_FOR_UPDATE_ICON_SLOW_EVENT;  // pour les animation de transitions.
-		if (myDesklet)  // On ne teste le survol des boutons que si l'applet est détachée
+		if (myDesklet)  // On ne teste le survol des boutons que si l'applet est detachee
 		{
 			cairo_dock_register_notification_on_object (myContainer,
 				NOTIFICATION_MOUSE_MOVED,
@@ -153,13 +150,13 @@ CD_APPLET_STOP_BEGIN
 		(CairoDockNotificationFunc) cd_opengl_test_mouse_over_buttons,
 		myApplet);
 	
+	// stop the current handler.
+	cd_musicplayer_stop_current_handler (TRUE);
+	
 	// On stoppe les boucles de recup de la pochette.
-	if (myData.iSidCheckXmlFile != 0)
-		g_source_remove (myData.iSidCheckXmlFile);
 	if (myData.iSidCheckCover != 0)
 		g_source_remove (myData.iSidCheckCover);
-	if (myData.iSidGetCoverInfoTwice != 0)
-		g_source_remove (myData.iSidGetCoverInfoTwice);
+	cairo_dock_free_task (myData.pCoverTask);
 	
 	// on libere la classe.
 	CD_APPLET_MANAGE_APPLICATION (NULL);
@@ -168,9 +165,6 @@ CD_APPLET_STOP_END
 
 //\___________ The reload occurs in 2 occasions : when the user changes the applet's config, and when the user reload the cairo-dock's config or modify the desklet's size. The macro CD_APPLET_MY_CONFIG_CHANGED can tell you this. myConfig has already been reloaded at this point if you're in the first case, myData is untouched. You also have the macro CD_APPLET_MY_CONTAINER_TYPE_CHANGED that can tell you if you switched from dock/desklet to desklet/dock mode.
 CD_APPLET_RELOAD_BEGIN
-	if (myDock && (myIcon->cName == NULL || *myIcon->cName == '\0'))
-		CD_APPLET_SET_NAME_FOR_MY_ICON (myConfig.cMusicPlayer);
-	
 	//\_______________ On reset surfaces et textures.
 	int i;
 	for (i = 0; i < PLAYER_NB_STATUS; i ++) { // reset surfaces.
@@ -231,42 +225,28 @@ CD_APPLET_RELOAD_BEGIN
 	myData.iPreviousCurrentTime = -1;
 	
 	myData.cover_exist = FALSE;
-	myData.bCoverNeedsTest = FALSE;
-	myData.iNbCheckFile = 0;
 	myData.iCurrentFileSize = 0;
 	myData.iGetTimeFailed = 0;
 	
 	//\_______________ On gere le changement de player ou on redessine juste l'icone.
-	if (CD_APPLET_MY_CONFIG_CHANGED) {
+	if (CD_APPLET_MY_CONFIG_CHANGED)
+	{
 		// on stoppe l'ancien backend et on relance le nouveau.
-		if (myData.pCurrentHandeler)
-		{
-			cd_musicplayer_stop_handler ();  // libere tout ce qu'occupe notre ancien handler.
-			CD_APPLET_MANAGE_APPLICATION (NULL);
-		}
-		myData.pCurrentHandeler = cd_musicplayer_get_handler_by_name (myConfig.cMusicPlayer);
-		if (myData.pCurrentHandeler == NULL)
-		{
- 			cd_warning ("MP : this player (%s) is not supported.", myConfig.cMusicPlayer); 
- 			return FALSE; 
-		}
-		cd_musicplayer_launch_handler ();
+		cd_musicplayer_stop_current_handler (TRUE);  // libere tout ce qu'occupe notre ancien handler.
+
+		CD_APPLET_MANAGE_APPLICATION (NULL);
 		
-		if (myConfig.bStealTaskBarIcon)
-			CD_APPLET_MANAGE_APPLICATION (myData.pCurrentHandeler->appclass);
+		cd_musicplayer_set_current_handler (myConfig.cMusicPlayer);
 	}
-	else { // on redessine juste l'icone.
-		cd_musicplayer_update_icon (FALSE);  // FALSE pour ne pas avoir 2 fois le dialogue.
-		if (! myData.cover_exist)
+	else  // on redessine juste l'icone.
+	{
+		if (myConfig.bEnableCover && myData.cover_exist && myData.cCoverPath != NULL)  // cover is available
 		{
-			if(myData.iPlayingStatus == PLAYER_PLAYING)
-			{
-				cd_musicplayer_set_surface (PLAYER_PLAYING);
-			}
-			else
-			{
-				cd_musicplayer_set_surface (PLAYER_PAUSED);
-			}
+			cd_musiplayer_apply_cover ();
+		}
+		else  // no cover -> set the status surface.
+		{
+			cd_musicplayer_apply_status_surface (myData.iPlayingStatus);
 		}
 	}
 CD_APPLET_RELOAD_END
