@@ -262,6 +262,25 @@ void cd_mpris_getPlaying_async (void)
 		G_TYPE_INVALID);
 }
 
+int cd_mpris_get_volume (void)
+{
+	GError *erreur = NULL;
+	int iVolume;
+	dbus_g_proxy_call (myData.dbus_proxy_player, "VolumeGet", &erreur,
+		G_TYPE_INVALID,
+		G_TYPE_INT, &iVolume,
+		G_TYPE_INVALID);
+	return iVolume;
+}
+
+void cd_mpris_set_volume (int iVolume)
+{
+	dbus_g_proxy_call_no_reply (myData.dbus_proxy_player, "VolumeSet",
+		G_TYPE_INT, iVolume,
+		G_TYPE_INVALID,
+		G_TYPE_INVALID);
+}
+
 /* Teste si MP joue de la musique ou non
  */
 void cd_mpris_getPlaying (void)  // sync version, used by Audacious.
@@ -448,7 +467,7 @@ static inline void _extract_metadata (GHashTable *data_list)
 	{
 		cCoverPath = g_value_get_string(value);
 	}
-	cd_musicplayer_get_cover_path (cCoverPath, TRUE);
+	cd_musicplayer_set_cover_path (cCoverPath);
 }
 
 static void _on_got_song_infos (DBusGProxy *proxy, DBusGProxyCall *call_id, CairoDockModuleInstance *myApplet)
@@ -495,7 +514,7 @@ static void _on_got_song_infos (DBusGProxy *proxy, DBusGProxyCall *call_id, Cair
 		myData.cover_exist = FALSE;
 	}
 	
-	cd_musicplayer_update_icon (TRUE);
+	cd_musicplayer_update_icon ();
 	cd_musicplayer_relaunch_handler ();
 	
 	CD_APPLET_LEAVE ();
@@ -545,7 +564,7 @@ static void onChangeSong_mpris(DBusGProxy *player_proxy, GHashTable *metadata, g
 		myData.iTrackNumber = 0;
 		myData.cover_exist = FALSE;
 	}
-	cd_musicplayer_update_icon (TRUE);
+	cd_musicplayer_update_icon ();
 	CD_APPLET_LEAVE ();
 }
 
@@ -568,7 +587,7 @@ void onChangePlaying_mpris (DBusGProxy *player_proxy, GValueArray *status, gpoin
 	
 	if(! myData.cover_exist/** && (myData.cPlayingUri != NULL || myData.cTitle != NULL)*/)
 	{
-		cd_musicplayer_set_surface (myData.iPlayingStatus);
+		cd_musicplayer_apply_status_surface (myData.iPlayingStatus);
 	}
 	else
 	{
@@ -687,6 +706,22 @@ static void cd_mpris_control (MyPlayerControl pControl, const char* song)
 				G_TYPE_INVALID);
 		break;
 		
+		case PLAYER_VOLUME :
+		{
+			int iVolume = cd_mpris_get_volume ();  // [0, 100]
+			if (song && strcmp (song, "up") == 0)
+				iVolume += 5;
+			else
+				iVolume -= 5;
+			
+			if (iVolume > 100)
+				iVolume = 100;
+			else if (iVolume < 0)
+			iVolume = 0;
+			
+			cd_mpris_set_volume (iVolume);
+		}
+		
 		default :
 		break;
 	}
@@ -746,7 +781,7 @@ static void cd_mpris_start (void)
 	cd_mpris_getPlaying_async ();  // will get song infos after playing status.
 	/**cd_mpris_getPlaying ();
 	cd_mpris_getSongInfos ();
-	cd_musicplayer_update_icon (TRUE);
+	cd_musicplayer_update_icon ();
 	cd_musicplayer_relaunch_handler ();*/
 }
 
@@ -757,7 +792,7 @@ MusicPlayerHandler *cd_mpris_new_handler (void)
 	pHandler->stop = cd_mpris_stop;
 	pHandler->start = cd_mpris_start;
 	pHandler->control = cd_mpris_control;
-	pHandler->iPlayerControls = PLAYER_PREVIOUS | PLAYER_PLAY_PAUSE | PLAYER_NEXT | PLAYER_STOP | PLAYER_SHUFFLE | PLAYER_REPEAT | PLAYER_ENQUEUE;
+	pHandler->iPlayerControls = PLAYER_PREVIOUS | PLAYER_PLAY_PAUSE | PLAYER_NEXT | PLAYER_STOP | PLAYER_SHUFFLE | PLAYER_REPEAT | PLAYER_ENQUEUE | PLAYER_VOLUME;
 	pHandler->bSeparateAcquisition = FALSE;
 	pHandler->iLevel = PLAYER_GOOD;
 	
