@@ -144,7 +144,7 @@ static void _on_got_playing_status (DBusGProxy *proxy, DBusGProxyCall *call_id,
 	
 	CD_APPLET_LEAVE ();
 }
-static void cd_mpris2_getPlaying_async (void)  // used by Audacious.
+static void cd_mpris2_getPlaying_async (void)
 {
 	if (s_pGetStatusCall != NULL)
 		return;
@@ -190,6 +190,16 @@ static void cd_mpris2_get_time_elapsed (void)
 	}
 }
 
+/*static gboolean _can_control (void)
+{
+	return cairo_dock_dbus_get_property_as_int (myData.dbus_proxy_player, "org.mpris.MediaPlayer2.Player", "CanControl");
+}*/
+
+static double cd_mpris2_get_volume (void)
+{
+	return cairo_dock_dbus_get_property_as_double (myData.dbus_proxy_player, "org.mpris.MediaPlayer2.Player", "Volume");
+}
+
 static gboolean _extract_metadata (GHashTable *pMetadata)
 {
 	gboolean bTrackHasChanged = FALSE;
@@ -275,7 +285,7 @@ static gboolean _extract_metadata (GHashTable *pMetadata)
 	{
 		cCoverPath = g_value_get_string(v);
 	}
-	cd_musicplayer_get_cover_path (cCoverPath, TRUE); // did it at the end (we have to know the artist and the album if (cCoverPath == NULL))
+	cd_musicplayer_set_cover_path (cCoverPath);  // do it at the end (we have to know the artist and the album if (cCoverPath == NULL))
 
 	/// we miss iTrackListIndex and tracklist-length ...
 	
@@ -334,7 +344,7 @@ static void _on_got_song_infos (DBusGProxy *proxy, DBusGProxyCall *call_id, Cair
 		myData.cover_exist = FALSE;
 	}
 	
-	cd_musicplayer_update_icon (TRUE);
+	cd_musicplayer_update_icon ();
 	cd_musicplayer_relaunch_handler ();
 	
 	CD_APPLET_LEAVE ();
@@ -366,7 +376,7 @@ static void on_properties_changed (DBusGProxy *player_proxy, const gchar *cInter
 	if (strcmp (cInterface, "org.mpris.MediaPlayer2.Player") == 0)
 	{
 		v = g_hash_table_lookup (pChangedProps, "PlaybackStatus");
-		if (v != NULL && G_VALUE_HOLDS_STRING (v))
+		if (v != NULL && G_VALUE_HOLDS_STRING (v))  // status has changed
 		{
 			const gchar *cStatus = g_value_get_string (v);  // "Playing", "Paused" or "Stopped"
 			myData.iPlayingStatus = _extract_status (cStatus);
@@ -375,15 +385,7 @@ static void on_properties_changed (DBusGProxy *player_proxy, const gchar *cInter
 			if (myData.iPlayingStatus == PLAYER_PLAYING)  // le handler est stoppe lorsque le lecteur ne joue rien.
 				cd_musicplayer_relaunch_handler ();
 			
-			if (! myData.cover_exist && (myData.cPlayingUri != NULL || myData.cTitle != NULL))
-			{
-				cd_musicplayer_set_surface (myData.iPlayingStatus);
-				cd_debug ("cover (%d), cPlayingUri (%s), cTitle (%s)", myData.cover_exist, myData.cPlayingUri, myData.cTitle);
-			}
-			else
-			{
-				CD_APPLET_REDRAW_MY_ICON;
-			}
+			cd_musicplayer_update_icon ();
 		}
 		
 		v = g_hash_table_lookup (pChangedProps, "Metadata");
@@ -392,19 +394,18 @@ static void on_properties_changed (DBusGProxy *player_proxy, const gchar *cInter
 			GHashTable *pMetadata = g_value_get_boxed (v);
 			gboolean bTrackHasChanged = _extract_metadata (pMetadata);
 			
-			if (bTrackHasChanged)
+			if (bTrackHasChanged)  // new song (song changed or started playing)
 			{
 				myData.iPlayingStatus = PLAYER_PLAYING;  // pour les lecteurs bugues comme Exaile qui envoit un statut "stop" au changement de musique sans envoyer de status "play" par la suite. On considere donc que si le lecteur joue une nouvelle musique, c'est qu'il est en "play".
-				cd_musicplayer_update_icon (TRUE);
+				cd_musicplayer_update_icon ();
 			}
-			else if (myData.iPlayingStatus == PLAYER_STOPPED)
-				cd_musicplayer_update_icon (TRUE); // Force the update of the icon if the player is now stopped.
 		}
 	}
-	else /*if (strcmp (cInterface, "org.mpris.MediaPlayer2.TrackList") == 0)
+	/*else if (strcmp (cInterface, "org.mpris.MediaPlayer2.TrackList") == 0)
 	{
 		
 	}*/
+	else
 		cd_debug ("Another interface: %s", cInterface);
 }
 
@@ -437,6 +438,7 @@ static void cd_mpris2_stop (void)
  */
 static void cd_mpris2_control (MyPlayerControl pControl, const char* song)
 {
+	static GValue s_pValue = G_VALUE_INIT;
 	gboolean bToggleValue;
 	switch (pControl)
 	{
@@ -466,23 +468,19 @@ static void cd_mpris2_control (MyPlayerControl pControl, const char* song)
 		case PLAYER_SHUFFLE :
 			bToggleValue = cd_mpris2_is_shuffle ();
 			cd_debug ("SetRandom <- %d\n", !bToggleValue);
-			dbus_g_proxy_call_no_reply (myData.dbus_proxy_player, "Set",
-				G_TYPE_INVALID,
-				G_TYPE_STRING, "org.mpris.MediaPlayer2.Player",
-				G_TYPE_STRING, "Shuffle",
-				G_TYPE_BOOLEAN, !bToggleValue,
-				G_TYPE_INVALID);
+			g_value_init (&s_pValue, G_TYPE_BOOLEAN);
+			g_value_set_boolean (&s_pValue, !bToggleValue);
+			cairo_dock_dbus_set_property (myData.dbus_proxy_player, "org.mpris.MediaPlayer2.Player", "Shuffle", &s_pValue);
+			g_value_unset (&s_pValue);
 		break;
 		
 		case PLAYER_REPEAT :
 			bToggleValue = cd_mpris2_is_loop ();
 			cd_debug ("SetLoop <- %d\n", !bToggleValue);
-			dbus_g_proxy_call_no_reply (myData.dbus_proxy_player, "Set",
-				G_TYPE_INVALID,
-				G_TYPE_STRING, "org.mpris.MediaPlayer2.Player",
-				G_TYPE_STRING, "LoopStatus",
-				G_TYPE_STRING, (bToggleValue ? "None" : "Playlist"),
-				G_TYPE_INVALID);
+			g_value_init (&s_pValue, G_TYPE_STRING);
+			g_value_set_static_string (&s_pValue, bToggleValue ? "None" : "Playlist");
+			cairo_dock_dbus_set_property (myData.dbus_proxy_player, "org.mpris.MediaPlayer2.Player", "LoopStatus", &s_pValue);
+			g_value_unset (&s_pValue);
 		break;
 		
 		case PLAYER_ENQUEUE :
@@ -512,6 +510,20 @@ static void cd_mpris2_control (MyPlayerControl pControl, const char* song)
 		}
 		break;
 		
+		case PLAYER_VOLUME :
+		{
+			double fVolume = cd_mpris2_get_volume ();  // [0, 1]
+			if (song && strcmp (song, "up") == 0)
+				fVolume += .05;
+			else
+				fVolume -= .05;
+			cd_debug ("volume <- %f\n", fVolume);
+			g_value_init (&s_pValue, G_TYPE_DOUBLE);
+			g_value_set_double (&s_pValue, fVolume);
+			cairo_dock_dbus_set_property (myData.dbus_proxy_player, "org.mpris.MediaPlayer2.Player", "Volume", &s_pValue);
+			g_value_unset (&s_pValue);
+		}
+		
 		default :
 		break;
 	}
@@ -632,6 +644,6 @@ void cd_musicplayer_register_mpris2_handler (void)
 	
 	pHandler->appclass = NULL;  // will be filled later.
 	pHandler->launch = NULL;  // will be filled later.
-	pHandler->iPlayerControls = PLAYER_PREVIOUS | PLAYER_PLAY_PAUSE | PLAYER_NEXT | PLAYER_STOP | PLAYER_SHUFFLE | PLAYER_REPEAT | PLAYER_ENQUEUE;
+	pHandler->iPlayerControls = PLAYER_PREVIOUS | PLAYER_PLAY_PAUSE | PLAYER_NEXT | PLAYER_STOP | PLAYER_SHUFFLE | PLAYER_REPEAT | PLAYER_ENQUEUE | PLAYER_VOLUME;
 	cd_musicplayer_register_my_handler (pHandler);
 }
