@@ -49,7 +49,6 @@ owned =>
  if mpris2: set mpris2 handler as current
  launch it
 not owned => stop handler
-
 */
 
 MusicPlayerHandler *cd_musicplayer_get_handler_by_name (const gchar *cName)
@@ -66,31 +65,77 @@ MusicPlayerHandler *cd_musicplayer_get_handler_by_name (const gchar *cName)
 	return NULL;
 }
 
-MusicPlayerHandler *cd_musicplayer_get_handler_by_service (const gchar *cService)
-{
-	g_return_val_if_fail (cService != NULL, NULL);
-	GList *h;
-	MusicPlayerHandler *handler;
-	for (h = myData.pHandlers; h != NULL; h = h->next)
-	{
-		handler = h->data;
-		if (handler->cMprisService && strcmp (handler->cMprisService, cService) == 0)
-			return handler;
-	}
-	return NULL;
-}
-
 
 static void _cd_musicplayer_get_data_async (gpointer data) {
 	if (myData.pCurrentHandler->get_data)
 		myData.pCurrentHandler->get_data();
 }
 
+static gboolean _cd_musicplayer_update_from_data (gpointer data)
+{
+	g_return_val_if_fail (myData.pCurrentHandler->iLevel != PLAYER_EXCELLENT, FALSE);
+	//cd_debug ("MP - %s (%d : %d -> %d)\n", __func__, myData.iPlayingStatus, myData.iPreviousCurrentTime, myData.iCurrentTime);
+	CD_APPLET_ENTER;
+	gboolean bNeedRedraw = FALSE;
+	
+	// All players: update the time.
+	if (myData.iCurrentTime != myData.iPreviousCurrentTime)
+	{
+		myData.iPreviousCurrentTime = myData.iCurrentTime;
+		if (myData.iPlayingStatus == PLAYER_PLAYING || myData.iPlayingStatus == PLAYER_PAUSED)
+		{
+			if (myData.iCurrentTime >= 0)  // peut etre -1 si le lecteur a demarre mais ne fournit pas encore de temps.
+			{
+				if (myConfig.iQuickInfoType == MY_APPLET_TIME_ELAPSED)
+				{
+					CD_APPLET_SET_MINUTES_SECONDES_AS_QUICK_INFO (myData.iCurrentTime);
+				}
+				else if (myConfig.iQuickInfoType == MY_APPLET_TIME_LEFT)
+				{
+					CD_APPLET_SET_MINUTES_SECONDES_AS_QUICK_INFO (myData.iCurrentTime - myData.iSongLength);
+				}
+			}
+			else
+				CD_APPLET_SET_QUICK_INFO_ON_MY_ICON (NULL);
+		}
+		else
+		{
+			CD_APPLET_SET_QUICK_INFO_ON_MY_ICON (NULL);
+		}
+		bNeedRedraw = TRUE;
+	}
+	
+	// Bad players: update the icon if the state or song has changed (no signals for this).
+	if (myData.pCurrentHandler->iLevel == PLAYER_BAD)
+	{
+		if (myData.iPlayingStatus != myData.pPreviousPlayingStatus)  // state has changed.
+		{
+			cd_debug ("MP : PlayingStatus : %d -> %d\n", myData.pPreviousPlayingStatus, myData.iPlayingStatus);
+			myData.pPreviousPlayingStatus = myData.iPlayingStatus;
+			
+			cd_musicplayer_update_icon ();
+			bNeedRedraw = FALSE;
+		}
+		else if (cairo_dock_strings_differ (myData.cPreviousRawTitle, myData.cRawTitle))  // song has changed.
+		{
+			g_free (myData.cPreviousRawTitle);
+			myData.cPreviousRawTitle = g_strdup (myData.cRawTitle);
+			cd_musicplayer_update_icon ();
+			bNeedRedraw = FALSE;
+		}
+	}
+	
+	if (bNeedRedraw)
+		CD_APPLET_REDRAW_MY_ICON;
+	
+	CD_APPLET_LEAVE (myData.pCurrentHandler->iLevel == PLAYER_BAD || (myData.pCurrentHandler->iLevel == PLAYER_GOOD && myData.iPlayingStatus == PLAYER_PLAYING));
+}
+
 static gboolean _cd_musicplayer_get_data_and_update (gpointer data) {
 	CD_APPLET_ENTER;
 	if (myData.pCurrentHandler->get_data)
 		myData.pCurrentHandler->get_data();
-	return cd_musicplayer_draw_icon (data);  // cette fonction sort.
+	return _cd_musicplayer_update_from_data (data);  // cette fonction sort.
 }
 
 /* Initialise le backend et lance la tache periodique si necessaire.
@@ -117,7 +162,7 @@ void cd_musicplayer_launch_handler (void)
 		{
   			myData.pTask = cairo_dock_new_task (1,
   				(CairoDockGetDataAsyncFunc) _cd_musicplayer_get_data_async,
-  				(CairoDockUpdateSyncFunc) cd_musicplayer_draw_icon,
+  				(CairoDockUpdateSyncFunc) _cd_musicplayer_update_from_data,
   				NULL);
 		}
 		else
@@ -158,6 +203,11 @@ void cd_musicplayer_stop_current_handler (gboolean bStopWatching)
 		dbus_g_proxy_cancel_call (cairo_dock_get_main_proxy (), myData.pDetectPlayerCall);
 		myData.pDetectPlayerCall = NULL;
 	}
+	if (myData.pGetPropsCall)
+	{
+		dbus_g_proxy_cancel_call (cairo_dock_get_main_proxy (), myData.pGetPropsCall);
+		myData.pGetPropsCall = NULL;
+	}
 	
 	if (bStopWatching)
 	{
@@ -212,6 +262,65 @@ void cd_musicplayer_free_handler (MusicPlayerHandler *pHandler)
 	g_free (pHandler);
 }
 
+static void _on_got_desktop_entry (DBusGProxy *proxy, DBusGProxyCall *call_id, gpointer data)
+{
+	CD_APPLET_ENTER;
+	myData.pGetPropsCall = NULL;
+	
+	GKeyFile* pKeyFile = NULL;
+	GValue v = G_VALUE_INIT;
+	GError *error = NULL;
+	gboolean bSuccess = dbus_g_proxy_end_call (proxy,
+		call_id,
+		&error,
+		G_TYPE_VALUE,
+		&v,
+		G_TYPE_INVALID);
+	if (error)
+	{
+		cd_warning ("%s", error->message);
+		g_error_free (error);
+	}
+	if (bSuccess && G_VALUE_HOLDS_STRING (&v))
+	{
+		const gchar *cDesktopFileName = g_value_get_string (&v);
+		cd_debug (" got desktop-entry '%s' from the service '%s'", cDesktopFileName, myData.pCurrentHandler->cMprisService);
+		
+		if (cDesktopFileName != NULL)
+		{
+			if (myConfig.cLastKnownDesktopFile == NULL || strcmp (cDesktopFileName, myConfig.cLastKnownDesktopFile) != 0)  // the property has changed from the previous time.
+			{
+				cd_debug ("  desktop-entry has changed, update");
+				// store the desktop filename, since we can't have it until the service is up, the next time the applet is started, which means we wouldn't be able to launch the player.
+				cairo_dock_update_conf_file (CD_APPLET_MY_CONF_FILE,
+					G_TYPE_STRING, "Configuration", "desktop-entry", cDesktopFileName,
+					G_TYPE_INVALID);
+				g_free (myConfig.cLastKnownDesktopFile);
+				myConfig.cLastKnownDesktopFile = g_strdup (cDesktopFileName);
+				
+				// register the desktop file, and get the common properties of this class.
+				g_free ((gchar*)myData.pCurrentHandler->launch);
+				g_free ((gchar*)myData.pCurrentHandler->appclass);
+				g_free ((gchar*)myData.pCurrentHandler->cDisplayedName);
+				
+				myData.pCurrentHandler->appclass = cairo_dock_register_class (cDesktopFileName);
+				myData.pCurrentHandler->launch = g_strdup (cairo_dock_get_class_command (myData.pCurrentHandler->appclass));
+				myData.pCurrentHandler->cDisplayedName = g_strdup (cairo_dock_get_class_name (myData.pCurrentHandler->appclass));
+				cd_debug ("  class is now '%s'", myData.pCurrentHandler->appclass);
+				
+				if (myData.pCurrentHandler->appclass != NULL)
+				{
+					cairo_dock_set_data_from_class (myData.pCurrentHandler->appclass, myIcon);
+				}
+				
+				if (myConfig.bStealTaskBarIcon)
+					CD_APPLET_MANAGE_APPLICATION (myData.pCurrentHandler->appclass);
+			}
+		}
+		g_value_unset (&v);
+	}
+	CD_APPLET_LEAVE ();
+}
 static void _on_name_owner_changed (const gchar *cName, gboolean bOwned, gpointer data)
 {
 	CD_APPLET_ENTER;
@@ -223,30 +332,38 @@ static void _on_name_owner_changed (const gchar *cName, gboolean bOwned, gpointe
 		if (strncmp (cName, CD_MPRIS2_SERVICE_BASE, strlen (CD_MPRIS2_SERVICE_BASE)) == 0)  // the MPRIS2 service is now on the bus, it has priority.
 		{
 			cd_debug ("the MPRIS2 service is now on the bus, it has priority");
-			if (strcmp (myData.pCurrentHandler->name, "Mpris2") != 0)  // our current handler is not the MPRIS2 one, stop it and use the latter instead.
+			// set the MPRIS2 handler as the current one if not already the case.
+			if (strcmp (myData.pCurrentHandler->name, "Mpris2") != 0)  // the current handler is not the MPRIS2 one, stop it and use the latter instead.
 			{
 				cd_debug ("our current handler is not the MPRIS2 one, stop it and use the latter instead");
-				gboolean bAlreadyWatching = FALSE;
-				if (myData.cMpris2Service && strcmp (myData.cMpris2Service, cName) == 0)  // we're already watching it, don't unwatch (we can't unwatch ourselves in the callback, plus it's a waste of CPU).
-				{
-					bAlreadyWatching = TRUE;
+				// stop the old handler
+				if (myData.cMpris2Service != cName)
 					g_free (myData.cMpris2Service);
-					myData.cMpris2Service = NULL;
-				}
+				myData.cMpris2Service = NULL;  // we're already watching it, don't re-watch (we can't unwatch ourselves in the callback, plus it's a waste of CPU).
+				
 				cd_musicplayer_stop_current_handler (TRUE);  // so once we detect the MPRIS2 service on the bus, the other one is dropped forever.
 				
-				myData.pCurrentHandler = cd_musicplayer_get_handler_by_name ("Mpris2");
-				g_free ((gchar*)myData.pCurrentHandler->cMprisService);  // well, in _this_ case the string is not constant.
+				// set the MPRIS2 handler as the current one
+				myData.pCurrentHandler = cd_musicplayer_get_handler_by_name ("Mpris2");  // no need to watch it, it was already done (that's why we are here !)
+				
+				g_free ((gchar*)myData.pCurrentHandler->cMprisService);
 				myData.pCurrentHandler->cMprisService = g_strdup (cName);
-				
-				myData.pCurrentHandler->launch = g_strdup (cName+strlen (CD_MPRIS2_SERVICE_BASE"."));
-				myData.pCurrentHandler->appclass = g_strdup (cName+strlen (CD_MPRIS2_SERVICE_BASE"."));
-				if (myConfig.bStealTaskBarIcon)
-					CD_APPLET_MANAGE_APPLICATION (myData.pCurrentHandler->appclass);
-				
-				if (! bAlreadyWatching)
-					cairo_dock_watch_dbus_name_owner (myData.pCurrentHandler->cMprisService, (CairoDockDbusNameOwnerChangedFunc) _on_name_owner_changed, NULL);
 			}
+			// get the desktop properties of the player.
+			// we do it now that the service is on the bus, because we can't guess them for sure from the MPRIS service name only (ex.: Rhythmbox has "org.mpris.MediaPlayer2.rhythmbox3" but its class is "rhythmbox").
+			DBusGProxy *pProxyProp = cairo_dock_create_new_session_proxy (
+				myData.pCurrentHandler->cMprisService,
+				CD_MPRIS2_OBJ,
+				DBUS_INTERFACE_PROPERTIES);
+			if (myData.pGetPropsCall)
+				dbus_g_proxy_cancel_call (cairo_dock_get_main_proxy (), myData.pGetPropsCall);
+			myData.pGetPropsCall = dbus_g_proxy_begin_call (pProxyProp, "Get",
+				(DBusGProxyCallNotify)_on_got_desktop_entry,
+				myApplet,
+				(GDestroyNotify) NULL,
+				G_TYPE_STRING, CD_MPRIS2_MAIN_IFACE,
+				G_TYPE_STRING, "DesktopEntry",
+				G_TYPE_INVALID);
 		}
 		else  // it's not the MPRIS2 service, ignore it if we already have the MPRIS2 service (shouldn't happen though).
 		{
@@ -263,9 +380,11 @@ static void _on_name_owner_changed (const gchar *cName, gboolean bOwned, gpointe
 	{
 		cd_debug ("stop the handler");
 		cd_musicplayer_stop_current_handler (FALSE);  // FALSE = keep watching it.
-		cd_musicplayer_set_surface (PLAYER_NONE);
+		cd_musicplayer_apply_status_surface (PLAYER_NONE);
 		if (myConfig.cDefaultTitle != NULL)
+		{
 			CD_APPLET_SET_NAME_FOR_MY_ICON (myConfig.cDefaultTitle);
+		}
 		else
 		{
 			if (strcmp (myData.pCurrentHandler->name, "Mpris2") == 0)
@@ -285,7 +404,7 @@ static void _on_detect_handler (gboolean bPresent, gpointer data)
 {
 	CD_APPLET_ENTER;
 	myData.pDetectPlayerCall = NULL;
-	cd_debug ("%s is present on the bus: %d\n", myData.pCurrentHandler->cMprisService, bPresent);
+	cd_debug ("%s presence on the bus: %d", myData.pCurrentHandler->cMprisService, bPresent);
 	if (bPresent)
 	{
 		_on_name_owner_changed (myData.pCurrentHandler->cMprisService, bPresent, data);
@@ -297,7 +416,7 @@ static void _on_detect_mpris2 (gboolean bPresent, gpointer data)
 {
 	CD_APPLET_ENTER;
 	myData.pDetectPlayerCall = NULL;
-	cd_debug ("MPRIS2 is present on the bus: %d\n", bPresent);
+	cd_debug ("MPRIS2 presence on the bus: %d", bPresent);
 	if (bPresent)
 		_on_name_owner_changed (myData.cMpris2Service, bPresent, data);
 	else if (myData.pCurrentHandler->cMprisService != NULL)  // couldn't detect it, but it has another service, so try this one.
@@ -305,85 +424,98 @@ static void _on_detect_mpris2 (gboolean bPresent, gpointer data)
 	CD_APPLET_LEAVE ();
 }
 
-/*if prefered player is defined:
-get handler from name
-if none: make a generic handler from the service
-set as current
-watch it; watch mpris2 too
-detect async -> present => owned*/
+/**
+Set the current handler from its name.
+It can be either the name of an old handler, or the name of an MPRIS2 service.
+ - if an old handler matches the name, use it, and detect it on the bus; also detect an MPRIS2 with the same name
+ - if no handler matches, then it's an MPRIS2 service; set the MPRIS2 handler as the current one, and detect it on the bus.
+*/
 void cd_musicplayer_set_current_handler (const gchar *cName)
 {
 	cd_debug ("%s (%s)", __func__, cName);
+	// stop completely any previous handler
 	cd_musicplayer_stop_current_handler (TRUE);
 	
+	// if no handler is defined, go to a neutral state.
 	if (cName == NULL)
 	{
 		myData.pCurrentHandler = NULL;
-		cd_musicplayer_set_surface (PLAYER_NONE);
+		cd_musicplayer_apply_status_surface (PLAYER_NONE);
 		if (myConfig.cDefaultTitle == NULL)
 			CD_APPLET_SET_NAME_FOR_MY_ICON (myApplet->pModule->pVisitCard->cTitle);
 		return;
 	}
 	
-	// find a handler which has this name
+	// find a handler from the given name
 	myData.pCurrentHandler = cd_musicplayer_get_handler_by_name (cName);
 	
-	// make the corresponding MPRIS2 name in any case, as it is the standard way.
-	gchar *cMpris2Service;
-	if (strncmp (cName, CD_MPRIS2_SERVICE_BASE, strlen (CD_MPRIS2_SERVICE_BASE)) == 0)  // it's already an MPRIS2 service name
+	if (myData.pCurrentHandler != NULL)  // an old handler exist with this name, use it but also look for the associated MPRIS2 service.
 	{
-		cMpris2Service = g_strdup (cName);
+		myData.cMpris2Service = myData.pCurrentHandler->cMpris2Service ? g_strdup (myData.pCurrentHandler->cMpris2Service) : g_strdup_printf (CD_MPRIS2_SERVICE_BASE".%s", cName);
+		cd_debug ("We check this MPRIS2 service: %s", myData.cMpris2Service);
+		
+		cairo_dock_watch_dbus_name_owner (myData.cMpris2Service, (CairoDockDbusNameOwnerChangedFunc) _on_name_owner_changed, NULL);
+		
+		myData.pDetectPlayerCall = cairo_dock_dbus_detect_application_async (myData.cMpris2Service, (CairoDockOnAppliPresentOnDbus) _on_detect_mpris2, NULL);  // mpris2 first, and then the other one.	
 	}
-	else  // make the default MPRIS2 service name.
-	{
-		cMpris2Service = g_strdup_printf (CD_MPRIS2_SERVICE_BASE".%s", cName);
-	}
-	
-	// if no such handler, make a generic MPRIS2 handler.
-	if (myData.pCurrentHandler == NULL)  // no such handler
+	else  // no such handler, make an MPRIS2 service with this name.
 	{
+		// get the MPRIS2 handler
 		myData.pCurrentHandler = cd_musicplayer_get_handler_by_name ("Mpris2");
 		
-		if (strncmp (cName, CD_MPRIS2_SERVICE_BASE".", strlen (CD_MPRIS2_SERVICE_BASE".")) == 0)
+		// fill its properties
+		if (myConfig.cLastKnownDesktopFile != NULL)
 		{
-			myData.pCurrentHandler->launch = g_strdup (cName+strlen (CD_MPRIS2_SERVICE_BASE"."));
-			myData.pCurrentHandler->appclass = g_strdup (cName+strlen (CD_MPRIS2_SERVICE_BASE"."));
+			myData.pCurrentHandler->appclass = cairo_dock_register_class (myConfig.cLastKnownDesktopFile);
+			myData.pCurrentHandler->launch = g_strdup (cairo_dock_get_class_command (myData.pCurrentHandler->appclass));
+			myData.pCurrentHandler->cDisplayedName = g_strdup (cairo_dock_get_class_name (myData.pCurrentHandler->appclass));
 		}
-		else
+		else  // can happen when transitionning from an old version of the applet where we didn't use the "Desktop Entry" property yet -> use some heuristic as a fallback.
 		{
-			myData.pCurrentHandler->launch = g_strdup (cName);
-			myData.pCurrentHandler->appclass = g_strdup (cName);
+			myData.pCurrentHandler->appclass = cairo_dock_register_class (cName);
+			myData.pCurrentHandler->launch = g_strdup (cairo_dock_get_class_command (myData.pCurrentHandler->appclass));
+			myData.pCurrentHandler->cDisplayedName = g_strdup (cairo_dock_get_class_name (myData.pCurrentHandler->appclass));
+			if (myData.pCurrentHandler->launch == NULL)  // we really need a command to launch it on click, so insist a little
+			{
+				myData.pCurrentHandler->launch = g_strdup (cName);
+			}
 		}
-		myData.pCurrentHandler->cMprisService = cMpris2Service;
-	}
-	else
-	{
-		myData.cMpris2Service = cMpris2Service;
+		myData.pCurrentHandler->cMprisService = g_strdup_printf (CD_MPRIS2_SERVICE_BASE".%s", cName);
+		myData.cMpris2Service = NULL;
 	}
 	
 	// watch it on the bus.
-	if (myData.cMpris2Service != NULL)
-		cairo_dock_watch_dbus_name_owner (myData.cMpris2Service, (CairoDockDbusNameOwnerChangedFunc) _on_name_owner_changed, NULL);
-	if (myData.pCurrentHandler->cMprisService != NULL)  // watch both, but MPRIS2 will have the priority.
+	if (myData.pCurrentHandler->cMprisService != NULL)  // paranoia
+	{
 		cairo_dock_watch_dbus_name_owner (myData.pCurrentHandler->cMprisService, (CairoDockDbusNameOwnerChangedFunc) _on_name_owner_changed, NULL);
 	
-	// detect its current presence on the bus.
-	if (myData.cMpris2Service != NULL)
-		myData.pDetectPlayerCall = cairo_dock_dbus_detect_application_async (myData.cMpris2Service, (CairoDockOnAppliPresentOnDbus) _on_detect_mpris2, NULL);  // mpris2 first, and then the other one.	
-	else if (myData.pCurrentHandler->cMprisService != NULL)
-		myData.pDetectPlayerCall = cairo_dock_dbus_detect_application_async (myData.pCurrentHandler->cMprisService, (CairoDockOnAppliPresentOnDbus) _on_detect_handler, NULL);
-	cd_musicplayer_set_surface (PLAYER_NONE);  // meanwhile, consider it's not running.
+		// detect its presence on the bus.
+		if (myData.pDetectPlayerCall == NULL)  // if we're already detecting MPRIS2, we'll detect this handler after we got the answer.
+			myData.pDetectPlayerCall = cairo_dock_dbus_detect_application_async (myData.pCurrentHandler->cMprisService, (CairoDockOnAppliPresentOnDbus) _on_detect_handler, NULL);
+	}
+	
+	// set the current icon and label.
+	if (myData.pCurrentHandler->appclass != NULL)
+	{
+		cairo_dock_set_data_from_class (myData.pCurrentHandler->appclass, myIcon);
+	}
+	
+	cd_musicplayer_apply_status_surface (PLAYER_NONE);  // until we detect any service, consider it's not running.
 	if (myConfig.cDefaultTitle == NULL)
 	{
-		if (strcmp (myData.pCurrentHandler->name, "Mpris2") == 0)
+		/**if (myIcon->cName != NULL)
 		{
-			CD_APPLET_SET_NAME_FOR_MY_ICON (cd_musicplayer_get_string_with_first_char_to_upper (myData.pCurrentHandler->launch));
-			cd_debug ("musicPlayer label: set name (with upper): %s", cd_musicplayer_get_string_with_first_char_to_upper (myData.pCurrentHandler->launch));
+			CD_APPLET_SET_NAME_FOR_MY_ICON (myIcon->cName);
+		}
+		else */if (strcmp (myData.pCurrentHandler->name, "Mpris2") != 0)
+		{
+			CD_APPLET_SET_NAME_FOR_MY_ICON (myData.pCurrentHandler->name);
 		}
 		else
 		{
-			CD_APPLET_SET_NAME_FOR_MY_ICON (myData.pCurrentHandler->name);
-			cd_debug ("musicPlayer label: set name: %s", myData.pCurrentHandler->name);
+			gchar *cDefaultName = cd_musicplayer_get_string_with_first_char_to_upper (myData.pCurrentHandler->launch);
+			CD_APPLET_SET_NAME_FOR_MY_ICON (cDefaultName);
+			g_free (cDefaultName);
 		}
 	}
 	
