@@ -106,6 +106,9 @@ static void _init_fill_menu_from_dir (CDQuickBrowserItem *pItem)
 	{
 		pMenuItem = gtk_image_menu_item_new_with_label (D_("Open this folder"));
 		GtkWidget *image = gtk_image_new_from_stock (GTK_STOCK_OPEN, GTK_ICON_SIZE_MENU);
+#if (GTK_MAJOR_VERSION > 2 || GTK_MINOR_VERSION >= 16)
+		gtk_image_menu_item_set_always_show_image (GTK_IMAGE_MENU_ITEM (pMenuItem), TRUE);
+#endif
 		gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (pMenuItem), image);
 	}
 	else
@@ -151,9 +154,14 @@ static void _fill_submenu_with_items (CDQuickBrowserItem *pRootItem, int iNbSubI
 		if (cIconName != NULL)
 		{
 			pMenuItem = gtk_image_menu_item_new_with_label (cFileName);
-			GdkPixbuf *pixbuf = gdk_pixbuf_new_from_file_at_size (cIconName, myConfig.iIconSize, myConfig.iIconSize, NULL);
+			gchar *cPath = cairo_dock_search_icon_s_path (cIconName, myConfig.iIconSize);
+			GdkPixbuf *pixbuf = gdk_pixbuf_new_from_file_at_size (cPath, myConfig.iIconSize, myConfig.iIconSize, NULL);
+			g_free (cPath);
 			GtkWidget *image = gtk_image_new_from_pixbuf (pixbuf);
 			g_object_unref (pixbuf);
+#if (GTK_MAJOR_VERSION > 2 || GTK_MINOR_VERSION >= 16)
+			gtk_image_menu_item_set_always_show_image (GTK_IMAGE_MENU_ITEM (pMenuItem), TRUE);
+#endif
 			gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (pMenuItem), image);
 			g_free (cIconName);
 			cIconName = NULL;
