@@ -36,6 +36,8 @@ CD_APPLET_DEFINITION (N_("showDesktop"),
 	"Rom1 (Romain PEROL) &amp; Fabounet (Fabrice Rey)")
 
 
+static const gchar *s_cShortkeyDescription[CD_NB_ACTIONS] = {"Show desktop", "Show the desklets", "Show desktop and desklets", "Show the Widget Layer", "Expose all the desktops"};  // same names as in the config file, so no need to add N_()
+
 static void _show_desktop_for_drop (Icon *pIcon)
 {
 	cairo_dock_show_hide_desktop (! myData.bDesktopVisible);
@@ -62,7 +64,11 @@ CD_APPLET_INIT_BEGIN
 		CD_APPLET_SET_IMAGE_ON_MY_ICON (myConfig.cVisibleImage);
 	else
 		CD_APPLET_SET_DEFAULT_IMAGE_ON_MY_ICON_IF_NONE;
-	cd_keybinder_bind (myConfig.cShortcut, (CDBindkeyHandler) on_keybinding_pull, (gpointer)NULL);
+	
+	myData.cKeyBinding = CD_APPLET_BIND_KEY (myConfig.cShortcut,
+		D_(s_cShortkeyDescription[myConfig.iActionOnMiddleClick]),
+		"Configuration", "shortkey",
+		(CDBindkeyHandler) on_keybinding_pull);
 CD_APPLET_INIT_END
 
 
@@ -74,6 +80,8 @@ CD_APPLET_STOP_BEGIN
 	cairo_dock_remove_notification_func_on_object (&myDesktopMgr,
 		NOTIFICATION_DESKTOP_VISIBILITY_CHANGED,
 		(CairoDockNotificationFunc) on_show_desktop, myApplet);
+	
+	cd_keybinder_unbind (myData.cKeyBinding);
 CD_APPLET_STOP_END
 
 
@@ -91,6 +99,6 @@ CD_APPLET_RELOAD_BEGIN
 		else
 			CD_APPLET_SET_DEFAULT_IMAGE_ON_MY_ICON_IF_NONE;
 		
-		cd_keybinder_bind (myConfig.cShortcut, (CDBindkeyHandler) on_keybinding_pull, (gpointer)NULL);
+		cd_keybinder_rebind (myData.cKeyBinding, myConfig.cShortcut, D_(s_cShortkeyDescription[myConfig.iActionOnMiddleClick]));
 	}
 CD_APPLET_RELOAD_END
