@@ -24,6 +24,7 @@
 #ifdef HAVE_XRANDR
 #include<X11/extensions/Xrandr.h>
 #endif
+#include <gdk/gdkx.h>
 
 #include "applet-struct.h"
 #include "applet-notifications.h"
@@ -93,10 +94,22 @@ static void _cd_action (CDActionOnClick iAction)
 			_cd_show_hide_desktop (TRUE);  // TRUE <=> show the desklets
 		break ;
 		case CD_SHOW_WIDGET_LAYER :
-			_cd_show_widget_layer ();
+			if (cairo_dock_wm_can_show_widget_layer ())
+				_cd_show_widget_layer ();
+			else
+			{
+				cd_warning ("It seems there is no widget layer, we show/hide the desktop");
+				_cd_show_hide_desktop (FALSE);
+			}
 		break ;
 		case CD_EXPOSE :
-			_cd_expose ();
+			if (cairo_dock_wm_can_present_desktops ())
+				_cd_expose ();
+			else
+			{
+				cd_warning ("It seems we can't present desktops, we show/hide the desktop");
+				_cd_show_hide_desktop (FALSE);
+			}
 		break ;
 		default:
 		break;
@@ -144,8 +157,6 @@ static void _show_desktop (GtkMenuItem *menu_item, gpointer data)
 	_cd_show_hide_desktop (FALSE);
 }
 CD_APPLET_ON_BUILD_MENU_BEGIN
-	GtkWidget *pSubMenu = CD_APPLET_CREATE_MY_SUB_MENU ();
-	
 	gchar *cLabel;
 	
 	if (myConfig.iActionOnLeftClick != CD_SHOW_DESKTOP)  // action is not bound to left-click => put it in the menu
@@ -155,7 +166,7 @@ CD_APPLET_ON_BUILD_MENU_BEGIN
 		else
 			cLabel = g_strdup (D_("Show desktop"));
 		CD_APPLET_ADD_IN_MENU_WITH_STOCK (cLabel,
-			MY_APPLET_SHARE_DATA_DIR"/../shared-images/show-desktop.svg",
+			MY_APPLET_SHARE_DATA_DIR"/../shared-files/images/show-desktop.svg",
 			_show_desktop,
 			CD_APPLET_MY_MENU);
 		g_free (cLabel);
@@ -167,7 +178,7 @@ CD_APPLET_ON_BUILD_MENU_BEGIN
 		else
 			cLabel = g_strdup (D_("Expose all the desktops"));
 		CD_APPLET_ADD_IN_MENU_WITH_STOCK (cLabel,
-			MY_APPLET_SHARE_DATA_DIR"/../shared-images/expose-desktops.svg",
+			MY_APPLET_SHARE_DATA_DIR"/../shared-files/images/expose-desktops.svg",
 			_cd_expose,
 			CD_APPLET_MY_MENU);
 		g_free (cLabel);
@@ -179,7 +190,7 @@ CD_APPLET_ON_BUILD_MENU_BEGIN
 		else
 			cLabel = g_strdup (D_("Show the Widget Layer"));
 		CD_APPLET_ADD_IN_MENU_WITH_STOCK (cLabel,
-			MY_APPLET_SHARE_DATA_DIR"/../shared-images/widget-layer.svg",
+			MY_APPLET_SHARE_DATA_DIR"/../shared-files/images/widget-layer.svg",
 		_cd_show_widget_layer,
 		CD_APPLET_MY_MENU);
 		g_free (cLabel);
@@ -231,9 +242,6 @@ CD_APPLET_ON_BUILD_MENU_BEGIN
 		XRRFreeScreenConfigInfo (conf);
 	}
 	#endif
-	
-	//Sub-Menu
-	CD_APPLET_ADD_ABOUT_IN_MENU (pSubMenu);
 CD_APPLET_ON_BUILD_MENU_END
 
 
@@ -312,7 +320,7 @@ static void _download_to_desktop (GtkMenuItem *menu_item, CairoDockModuleInstanc
 	if (cDesktopDir != NULL)
 	{
 		
-		CairoDockTask *pTask = cairo_dock_download_file_async (NULL, NULL, myData.cPendingFile, NULL, (GFunc)_dl_finished, myApplet);
+		CairoDockTask *pTask = cairo_dock_download_file_async (myData.cPendingFile, NULL, (GFunc)_dl_finished, myApplet);
 		
 		
 		
@@ -327,14 +335,14 @@ CD_APPLET_ON_DROP_DATA_BEGIN
 	
 	if (*CD_APPLET_RECEIVED_DATA == '/' || strncmp (CD_APPLET_RECEIVED_DATA, "file://", 7))  // fichier local
 	{
-		cairo_dock_add_in_menu_with_stock_and_data (("Move to the Desktop"), GTK_STOCK_CUT, (GFunc) _move_to_desktop, pMenu, myApplet);
-		cairo_dock_add_in_menu_with_stock_and_data (("Copy to the Desktop"), GTK_STOCK_COPY, (GFunc) _copy_to_desktop, pMenu, myApplet);
-		cairo_dock_add_in_menu_with_stock_and_data (("Link to the Desktop"), GTK_STOCK_JUMP_TO, (GFunc) _link_to_desktop, pMenu, myApplet);
+		cairo_dock_add_in_menu_with_stock_and_data (("Move to the Desktop"), GTK_STOCK_CUT, G_CALLBACK (_move_to_desktop), pMenu, myApplet);
+		cairo_dock_add_in_menu_with_stock_and_data (("Copy to the Desktop"), GTK_STOCK_COPY, G_CALLBACK (_copy_to_desktop), pMenu, myApplet);
+		cairo_dock_add_in_menu_with_stock_and_data (("Link to the Desktop"), GTK_STOCK_JUMP_TO, G_CALLBACK (_link_to_desktop), pMenu, myApplet);
 	}
 	else  // fichier a telecharger.
 	{
-		cairo_dock_add_in_menu_with_stock_and_data (("Link to the Desktop"), GTK_STOCK_JUMP_TO, (GFunc) _make_link_to_desktop, pMenu, myApplet);
-		cairo_dock_add_in_menu_with_stock_and_data (("Download onto the Desktop"), GTK_STOCK_COPY, (GFunc) _download_to_desktop, pMenu, myApplet);
+		cairo_dock_add_in_menu_with_stock_and_data (("Link to the Desktop"), GTK_STOCK_JUMP_TO, G_CALLBACK (_make_link_to_desktop), pMenu, myApplet);
+		cairo_dock_add_in_menu_with_stock_and_data (("Download onto the Desktop"), GTK_STOCK_COPY, G_CALLBACK (_download_to_desktop), pMenu, myApplet);
 	}
 	CD_APPLET_POPUP_MENU_ON_MY_ICON (pMenu);
 	
