@@ -31,11 +31,11 @@
 static inline void _toggle_pause (CairoDockModuleInstance *myApplet)
 {
 	if (!myData.bPause) {
-		myData.bPause = TRUE;  // coupera le timer.
+		myData.bPause = TRUE;  // will stop the slider.
 	}
 	else {
 		myData.bPause = FALSE;
-		cd_slider_next_slide (myApplet); //on relance le diapo
+		cd_slider_jump_to_next_slide (myApplet);  // restart the slider
 	}
 }
 
@@ -91,25 +91,15 @@ static gboolean _cd_slider_scroll_delayed (CairoDockModuleInstance *myApplet)
 {
 	CD_APPLET_ENTER;
 	if (myData.iNbScroll == 0)
+	{
+		myData.iScrollID = 0;
 		CD_APPLET_LEAVE (FALSE);
-	
-	if (myConfig.bUseThread)
-		cairo_dock_stop_task (myData.pMeasureImage);
+	}
 	
 	int i;
 	if (myData.iNbScroll > 0)
 	{
-		if (myData.iTimerID == 0)  // en cours d'animation, on la finit en affichant l'image courante.
-		{
-			cd_slider_draw_default (myApplet);
-			CD_APPLET_REDRAW_MY_ICON;
-		}
-		else
-		{
-			g_source_remove (myData.iTimerID);  //on coupe le timer en cours
-			myData.iTimerID = 0;
-		}
-		for (i = 0; i < myData.iNbScroll-1; i ++)  // le 1er scroll n'implique rien.
+		for (i = 0; i < myData.iNbScroll-1; i ++)  // N-1 because we'll jump to next slide.
 		{
 			if (myData.pElement == NULL)  // debut
 				myData.pElement = myData.pList;
@@ -119,20 +109,18 @@ static gboolean _cd_slider_scroll_delayed (CairoDockModuleInstance *myApplet)
 	}
 	else if (myData.iNbScroll < 0)
 	{
-		if (myData.iTimerID != 0)
+		for (i = 0; i <= -myData.iNbScroll; i ++)  // N+1 because we'll jump to next slide.
 		{
-			g_source_remove(myData.iTimerID); //on coupe le timer en cours
-			myData.iTimerID = 0;
-		}
-		for (i = 0; i <= -myData.iNbScroll; i ++)  // fait une fois de plus.
-		{
-			myData.pElement = cairo_dock_get_previous_element (myData.pElement, myData.pList);
+			if (myData.pElement == NULL)  // debut
+				myData.pElement = myData.pList;
+			else
+				myData.pElement = cairo_dock_get_previous_element (myData.pElement, myData.pList);
 		}
 	}
 	
 	myData.iNbScroll = 0;
 	myData.iScrollID = 0;
-	cd_slider_next_slide (myApplet);
+	cd_slider_jump_to_next_slide (myApplet);
 	CD_APPLET_LEAVE (FALSE);
 }
 CD_APPLET_ON_SCROLL_BEGIN
@@ -202,9 +190,15 @@ static void _cd_slider_run_dir (GtkMenuItem *menu_item, CairoDockModuleInstance
 	}
 	CD_APPLET_LEAVE();
 }
-CD_APPLET_ON_BUILD_MENU_BEGIN
-	GtkWidget *pSubMenu = CD_APPLET_CREATE_MY_SUB_MENU ();
+static void _cd_slider_refresh_images_list (GtkMenuItem *menu_item, CairoDockModuleInstance *myApplet)
+{
+	CD_APPLET_ENTER;
+	cd_slider_stop (myApplet);
 	
+	cd_slider_start (myApplet, FALSE);  // FALSE <=> immediately
+	CD_APPLET_LEAVE();
+}
+CD_APPLET_ON_BUILD_MENU_BEGIN
 	// get the current image selected at the moment of the click.
 	g_free (myData.cSelectedImagePath);
 	if (myData.pElement != NULL && myData.pElement->data != NULL)
@@ -215,7 +209,6 @@ CD_APPLET_ON_BUILD_MENU_BEGIN
 	else
 		myData.cSelectedImagePath = NULL;
 	
-	// Main Menu
 	gchar *cLabel;
 	if (myConfig.iClickOption != SLIDER_PAUSE)
 	{
@@ -240,7 +233,8 @@ CD_APPLET_ON_BUILD_MENU_BEGIN
 			GtkWidget *pSubMenu = CD_APPLET_ADD_SUB_MENU_WITH_IMAGE (D_("Open with"), CD_APPLET_MY_MENU, GTK_STOCK_OPEN);
 			
 			cd_slider_free_apps_list (myApplet);
-			
+
+			gint iDesiredIconSize = cairo_dock_search_icon_size (GTK_ICON_SIZE_LARGE_TOOLBAR); // 24px
 			GList *a;
 			gchar **pAppInfo;
 			gchar *cIconPath;
@@ -256,7 +250,7 @@ CD_APPLET_ON_BUILD_MENU_BEGIN
 				myData.pAppList = g_list_prepend (myData.pAppList, app);
 				
 				if (pAppInfo[2] != NULL)
-					cIconPath = cairo_dock_search_icon_s_path (pAppInfo[2]);
+					cIconPath = cairo_dock_search_icon_s_path (pAppInfo[2], iDesiredIconSize);
 				else
 					cIconPath = NULL;
 				CD_APPLET_ADD_IN_MENU_WITH_STOCK_AND_DATA (pAppInfo[0], cIconPath, _cd_slider_launch_with, pSubMenu, app);
@@ -272,12 +266,10 @@ CD_APPLET_ON_BUILD_MENU_BEGIN
 	else
 		cLabel = g_strdup (D_("Browse images folder"));
 	CD_APPLET_ADD_IN_MENU_WITH_STOCK (cLabel, GTK_STOCK_DIRECTORY, _cd_slider_run_dir, CD_APPLET_MY_MENU);
-	g_free (cLabel);
 	
-	// Sub-Menu
-	if (pSubMenu == CD_APPLET_MY_MENU)
-		CD_APPLET_ADD_SEPARATOR_IN_MENU (CD_APPLET_MY_MENU);
-	CD_APPLET_ADD_ABOUT_IN_MENU (pSubMenu);
+	CD_APPLET_ADD_IN_MENU_WITH_STOCK (D_("Refresh images list"), GTK_STOCK_REFRESH, _cd_slider_refresh_images_list, CD_APPLET_MY_MENU);
+	
+	g_free (cLabel);
 CD_APPLET_ON_BUILD_MENU_END
 
 
