@@ -220,16 +220,8 @@ static void _get_html_page (CDHtmlLink *pHtmlLink)
 				}
 				cd_debug ("cFaviconURL: %s", cFaviconURL);
 				
-				gchar *cTmpFavIconPath = cairo_dock_download_file (NULL, NULL, cFaviconURL, NULL, NULL);
-				if (cTmpFavIconPath != NULL)
-				{
-					gchar *cCommand = g_strdup_printf ("mv \"%s\" \"%s\"", cTmpFavIconPath, cLocalFaviconPath);
-					cd_debug ("%s", cCommand);
-					int r = system (cCommand);
-					g_free (cCommand);
-					g_free (cTmpFavIconPath);
-					bGotFavIcon = TRUE;
-				}
+				bGotFavIcon = cairo_dock_download_file (cFaviconURL, cLocalFaviconPath);
+				
 				g_free (cFaviconURL);
 				g_free (cRelPath);
 			}
@@ -272,10 +264,7 @@ static gboolean _update_html_link (CDHtmlLink *pHtmlLink)
 				cairo_dock_set_icon_name (pHtmlLink->cTitle, pIcon, pContainer);
 				
 				cd_debug ("draw emblem on %s", pIcon->cName);
-				CairoEmblem *pEmblem = cairo_dock_make_emblem (pHtmlLink->cFaviconPath, pIcon);
-				cairo_dock_set_emblem_position (pEmblem, CAIRO_DOCK_EMBLEM_LOWER_RIGHT);
-				cairo_dock_draw_emblem_on_icon (pEmblem, pIcon, pContainer);
-				cairo_dock_free_emblem (pEmblem);
+				cairo_dock_print_overlay_on_icon_from_image (pIcon, pContainer, pHtmlLink->cFaviconPath, CAIRO_OVERLAY_LOWER_RIGHT);
 				cairo_dock_redraw_icon (pIcon, pContainer);
 				break;
 			}
@@ -373,7 +362,7 @@ static Icon *_cd_stack_create_new_item (CairoDockModuleInstance *myApplet, const
 	}
 	else
 	{
-		cName = cairo_dock_cut_string (cContent, 15);  // 15 caracteres par defaut.
+		cName = cairo_dock_cut_string (cContent, 20);  // we only display the first 20 chars.
 	}
 	g_return_val_if_fail (cName != NULL, NULL);
 	
