@@ -88,22 +88,34 @@ static void _cd_switcher_draw_windows_on_viewport (Icon *pIcon, CDSwitcherDeskto
 		cairo_stroke (pCairoContext);
 	}
 	
-	if (pIcon->pIconBuffer != NULL)
+	/**Icon *pDisplayedIcon = pIcon;
+	if (pDisplayedIcon->pIconBuffer == NULL)  // icon not loaded (because not in a dock, because inhibited)
 	{
-		int iWidth, iHeight;
-		cairo_dock_get_icon_extent (pIcon, &iWidth, &iHeight);
-		double fZoomX = (double) w/g_desktopGeometry.iXScreenWidth[CAIRO_DOCK_HORIZONTAL]*iOneViewportWidth / iWidth;
-		double fZoomY = (double) h/g_desktopGeometry.iXScreenHeight[CAIRO_DOCK_HORIZONTAL]*iOneViewportHeight / iHeight;
+		
+		/// TODO: we'll have to load the icons that are not loaded (not inside a dock)...
+		Icon *pInhibitor = cairo_dock_get_inhibitor (pIcon, TRUE);  // TRUE = only in dock.
+		if (pInhibitor != NULL)
+			pDisplayedIcon = pInhibitor;
+	}
+	if (pDisplayedIcon->pIconBuffer != NULL)*/
+	const CairoDockImageBuffer *pImage = cairo_dock_appli_get_image_buffer (pIcon);
+	if (pImage && pImage->pSurface)
+	{
+		///int iWidth, iHeight;
+		///cairo_dock_get_icon_extent (pDisplayedIcon, &iWidth, &iHeight);
+		double fZoomX = (double) w/g_desktopGeometry.iXScreenWidth[CAIRO_DOCK_HORIZONTAL]*iOneViewportWidth / pImage->iWidth;
+		double fZoomY = (double) h/g_desktopGeometry.iXScreenHeight[CAIRO_DOCK_HORIZONTAL]*iOneViewportHeight / pImage->iHeight;
 		double fZoom = MIN (fZoomX, fZoomY);  // on garde le ratio.
 		
 		cairo_translate (pCairoContext,
-			(1.*x/g_desktopGeometry.iXScreenWidth[CAIRO_DOCK_HORIZONTAL] - iNumViewportX)*iOneViewportWidth + (fZoomX - fZoom) * iWidth/2,
-			(1.*y/g_desktopGeometry.iXScreenHeight[CAIRO_DOCK_HORIZONTAL] - iNumViewportY)*iOneViewportHeight + (fZoomY - fZoom) * iHeight/2);
+			(1.*x/g_desktopGeometry.iXScreenWidth[CAIRO_DOCK_HORIZONTAL] - iNumViewportX)*iOneViewportWidth + (fZoomX - fZoom) * pImage->iWidth/2,
+			(1.*y/g_desktopGeometry.iXScreenHeight[CAIRO_DOCK_HORIZONTAL] - iNumViewportY)*iOneViewportHeight + (fZoomY - fZoom) * pImage->iHeight/2);
 		cairo_scale (pCairoContext,
 			fZoom,
 			fZoom);
 		cairo_set_source_surface (pCairoContext,
-			pIcon->pIconBuffer,
+			///pDisplayedIcon->pIconBuffer,
+			pImage->pSurface,
 			0.,
 			0.);
 		cairo_paint (pCairoContext);
@@ -134,8 +146,6 @@ void cd_switcher_draw_main_icon_compact_mode (void)
 	cairo_dock_erase_cairo_context (myDrawContext);
 	
 	// definition des parametres de dessin.
-	//double fRatio = (myDock ? myDock->container.fRatio : 1.);
-	//double fMaxScale = cairo_dock_get_max_scale (myContainer); //coefficient Max icone Width
 	int iWidth, iHeight;
 	CD_APPLET_GET_MY_ICON_EXTENT (&iWidth, &iHeight);
 	
@@ -150,14 +160,14 @@ void cd_switcher_draw_main_icon_compact_mode (void)
 		if (r_ > r)  // on etire trop en largeur.
 		{
 			myData.switcher.fOneViewportWidth /= r_ / r;
-			dx = (double) iWidth/2 * (1 - r / r_);
-			w /= r_ / r;
+			w = myData.switcher.fOneViewportWidth * myData.switcher.iNbColumns + 2 * myConfig.iLineSize + (myData.switcher.iNbColumns - 1) * myConfig.iInLineSize;
+			dx = (double) (iWidth - w) / 2;
 		}
 		else
 		{
 			myData.switcher.fOneViewportHeight /= r / r_;
-			dy = (double) iHeight/2 * (1 - r_ / r);
-			h /= r / r_;
+			h = myData.switcher.fOneViewportHeight * myData.switcher.iNbLines + 2 * myConfig.iLineSize + (myData.switcher.iNbLines - 1) * myConfig.iInLineSize;
+			dy = (iHeight - h) / 2;
 		}
 	}
 	myData.switcher.fOffsetX = dx;
@@ -171,15 +181,13 @@ void cd_switcher_draw_main_icon_compact_mode (void)
 	if (myConfig.bMapWallpaper)
 	{
 		pSurface = myData.pDesktopBgMapSurface;
-		fZoomX = (double) myData.switcher.fOneViewportWidth / iWidth;
-		fZoomY= (double) myData.switcher.fOneViewportHeight / iHeight;
 	}
 	if (pSurface == NULL)
 	{
 		pSurface = myData.pDefaultMapSurface;
-		fZoomX = (double) myData.switcher.fOneViewportWidth / iWidth;
-		fZoomY = (double) myData.switcher.fOneViewportHeight / iHeight;
 	}
+	fZoomX = (double) myData.switcher.fOneViewportWidth / myData.iSurfaceWidth;  // both surfaces are loaded at the same size.
+	fZoomY= (double) myData.switcher.fOneViewportHeight / myData.iSurfaceHeight;
 	
 	// cadre exterieur.
 	cairo_set_line_width (myDrawContext,myConfig.iLineSize);
@@ -325,62 +333,62 @@ void cd_switcher_draw_main_icon_compact_mode (void)
 	
 	if (CD_APPLET_MY_CONTAINER_IS_OPENGL)
 		cairo_dock_update_icon_texture (myIcon);
-	else
-		CD_APPLET_UPDATE_REFLECT_ON_MY_ICON;
+	/**else
+		CD_APPLET_UPDATE_REFLECT_ON_MY_ICON;*/
 }
 
 
 void cd_switcher_draw_main_icon_expanded_mode (void)
 {
-	// definition des parametres de dessin.
+	// apply the desktop bg or the user image on the main icon, in dock mode
 	int iWidth, iHeight;
-	CD_APPLET_GET_MY_ICON_EXTENT (&iWidth, &iHeight);
-	//double fRatio = (myDock ? myDock->container.fRatio : 1.);
-	//double fMaxScale = cairo_dock_get_max_scale (myContainer); //coefficient Max icone Width
-	myData.switcher.fOneViewportHeight = (iHeight - 2 * myConfig.iLineSize - (myData.switcher.iNbLines - 1) * myConfig.iInLineSize) / myData.switcher.iNbLines; //hauteur d'un bureau/viewport sans compter les lignes exterieures et interieures.
-	myData.switcher.fOneViewportWidth = (iWidth - 2 * myConfig.iLineSize - (myData.switcher.iNbColumns - 1) * myConfig.iInLineSize) / myData.switcher.iNbColumns; //largeur d'un bureau/viewport sans compter les lignes exterieures et interieures.
-
-	cairo_surface_t *pSurface = NULL;
-	double fZoomX, fZoomY;
-	if (myConfig.bMapWallpaper)
+	
+	if (myDock)
 	{
-		cairo_dock_erase_cairo_context (myDrawContext);
-		
-		pSurface = myData.pDesktopBgMapSurface;
-		fZoomX = 1. * iWidth / iWidth;
-		fZoomY= 1. * iHeight / iHeight;
-		cairo_translate (myDrawContext,
-			0.,
-			0.);
+		CD_APPLET_GET_MY_ICON_EXTENT (&iWidth, &iHeight);
+		myData.switcher.fOneViewportHeight = (iHeight - 2 * myConfig.iLineSize - (myData.switcher.iNbLines - 1) * myConfig.iInLineSize) / myData.switcher.iNbLines; //hauteur d'un bureau/viewport sans compter les lignes exterieures et interieures.
+		myData.switcher.fOneViewportWidth = (iWidth - 2 * myConfig.iLineSize - (myData.switcher.iNbColumns - 1) * myConfig.iInLineSize) / myData.switcher.iNbColumns; //largeur d'un bureau/viewport sans compter les lignes exterieures et interieures.
 
-		cairo_save (myDrawContext);
-		cairo_scale (myDrawContext,
-			fZoomX ,
-			fZoomY );
-		cairo_set_source_surface (myDrawContext,
-			pSurface,
-			0.,
-			0.);
-		cairo_paint(myDrawContext);
-		cairo_restore (myDrawContext);
-		
-		if (CD_APPLET_MY_CONTAINER_IS_OPENGL)
-			cairo_dock_update_icon_texture (myIcon);
+		cairo_surface_t *pSurface = NULL;
+		double fZoomX, fZoomY;
+		if (myConfig.bMapWallpaper)
+		{
+			cairo_dock_erase_cairo_context (myDrawContext);
+
+			pSurface = myData.pDesktopBgMapSurface;
+			fZoomX = 1. * iWidth / myData.iSurfaceWidth;
+			fZoomY= 1. * iHeight / myData.iSurfaceHeight;
+			cairo_translate (myDrawContext,
+				0.,
+				0.);
+
+			cairo_save (myDrawContext);
+			cairo_scale (myDrawContext,
+				fZoomX ,
+				fZoomY );
+			cairo_set_source_surface (myDrawContext,
+				pSurface,
+				0.,
+				0.);
+			cairo_paint(myDrawContext);
+			cairo_restore (myDrawContext);
+
+			if (CD_APPLET_MY_CONTAINER_IS_OPENGL)
+				cairo_dock_update_icon_texture (myIcon);
+			/**else
+				CD_APPLET_UPDATE_REFLECT_ON_MY_ICON;*/
+		}
 		else
-			CD_APPLET_UPDATE_REFLECT_ON_MY_ICON;
+		{
+			CD_APPLET_SET_IMAGE_ON_MY_ICON (MY_APPLET_SHARE_DATA_DIR"/"MY_APPLET_ICON_FILE);
+		}
 	}
-	else
-	{
-		CD_APPLET_SET_IMAGE_ON_MY_ICON (MY_APPLET_SHARE_DATA_DIR"/"MY_APPLET_ICON_FILE);
-	}
-
+	
 	if (myConfig.bDrawWindows)
 	{
 		GList *pWindowList = cairo_dock_get_current_applis_list ();
 		pWindowList = g_list_sort (pWindowList, (GCompareFunc) _compare_icons_stack_order);
 		
-		//fMaxScale = (myIcon->pSubDock != NULL ? cairo_dock_get_max_scale (myIcon->pSubDock) : 1);
-		//fRatio = (myIcon->pSubDock != NULL ? myIcon->pSubDock->container.fRatio : 1);
 		CDSwitcherDesktop data;
 		int iNumDesktop=0, iNumViewportX=0, iNumViewportY=0;
 		cairo_t *pCairoContext;
@@ -392,6 +400,7 @@ void cd_switcher_draw_main_icon_expanded_mode (void)
 		{
 			pIcon = ic->data;
 			cairo_dock_get_icon_extent (pIcon, &iWidth, &iHeight);
+			
 			pCairoContext = cairo_create (pIcon->pIconBuffer);
 			cairo_set_line_width (pCairoContext, 1.);
 			cairo_set_source_rgba (pCairoContext, myConfig.RGBWLineColors[0], myConfig.RGBWLineColors[1], myConfig.RGBWLineColors[2], myConfig.RGBWLineColors[3]);
@@ -520,68 +529,35 @@ static void _cd_switcher_list_window_on_viewport (Icon *pIcon, int iNumDesktop,
 {
 	//g_print (" + %s\n", pIcon->cName);
 	// on recupere la taille de l'icone.
-	int iWidth, iHeight;
-	cairo_dock_get_icon_extent (pIcon, &iWidth, &iHeight);
+	/**Icon *pDisplayedIcon = pIcon;
+	if (pDisplayedIcon->pIconBuffer == NULL)  // icon not loaded (because not in a dock, because inhibited)
+	{
+		Icon *pInhibitor = cairo_dock_get_inhibitor (pIcon, TRUE);  // TRUE = only in dock.
+		if (pInhibitor != NULL)
+			pDisplayedIcon = pInhibitor;
+	}*/
+	const CairoDockImageBuffer *pImage = cairo_dock_appli_get_image_buffer (pIcon);
 	
 	// on cree une copie de la surface de l'icone a la taille du menu.
-	GdkPixbuf *pixbuf = NULL;
-	if (iWidth > 0 && iHeight > 0 && pIcon->pIconBuffer != NULL)
+	GdkPixbuf *pixbuf = cairo_dock_icon_buffer_to_pixbuf (pIcon/**pDisplayedIcon*/);
+	if (pixbuf == NULL)
 	{
-		int w = 24, h = w;
-		cairo_surface_t *surface = cairo_image_surface_create (CAIRO_FORMAT_RGB24,
-			w,
-			h);
-		cairo_t *pCairoContext = cairo_create (surface);
-		cairo_scale (pCairoContext, (double)w/iWidth, (double)h/iHeight);
-		cairo_set_source_surface (pCairoContext, pIcon->pIconBuffer, 0., 0.);
-		cairo_paint (pCairoContext);
-		cairo_destroy (pCairoContext);
-		guchar *d, *data = cairo_image_surface_get_data (surface);
-		int r = cairo_image_surface_get_stride (surface);
-		
-		// on la convertit en un pixbuf.
-		pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB,
-			TRUE,
-			8,
-			w,
-			h);
-		guchar *p, *pixels = gdk_pixbuf_get_pixels (pixbuf);
-		int iNbChannels = gdk_pixbuf_get_n_channels (pixbuf);
-		int iRowstride = gdk_pixbuf_get_rowstride (pixbuf);
-		
-		int x, y;
-		int red, green, blue;
-		float fAlphaFactor;
-		for (y = 0; y < h; y ++)
+		const gchar *cIcon = cairo_dock_get_class_icon (pIcon->cClass);
+		gint iDesiredIconSize = cairo_dock_search_icon_size (GTK_ICON_SIZE_LARGE_TOOLBAR); // 24px
+		gchar *cIconPath = cairo_dock_search_icon_s_path (cIcon, iDesiredIconSize);
+		if (cIconPath)
 		{
-			for (x = 0; x < w; x ++)
-			{
-				p = pixels + y * iRowstride + x * iNbChannels;
-				d = data + y * r + x * 4;
-				
-				fAlphaFactor = (float) d[3] / 255;
-				if (fAlphaFactor != 0)
-				{
-					red = d[0] / fAlphaFactor;
-					green = d[1] / fAlphaFactor;
-					blue = d[2] / fAlphaFactor;
-				}
-				else
-				{
-					red = blue = green = 0;
-				}
-				p[0] = blue;
-				p[1] = green;
-				p[2] = red;
-				p[3] = d[3];
-			}
+			pixbuf = gdk_pixbuf_new_from_file_at_size (cIconPath,
+				iDesiredIconSize,
+				iDesiredIconSize,
+				NULL);
 		}
-		
-		cairo_surface_destroy (surface);
 	}
 	
 	// on ajoute une entree au menu avec le pixbuf.
-	GtkWidget *pMenuItem = gtk_image_menu_item_new_with_label (pIcon->cName);
+	gchar *cLabel = cairo_dock_cut_string (pIcon->cName, 50);
+	GtkWidget *pMenuItem = gtk_image_menu_item_new_with_label (cLabel);
+	g_free (cLabel);
 	if (pixbuf)
 	{
 		GtkWidget *image = gtk_image_new_from_pixbuf (pixbuf);
@@ -642,7 +618,7 @@ void cd_switcher_build_windows_list (GtkWidget *pMenu)
 			g_object_set (pMenuItem, "height-request", 3, NULL);
 			
 			// on ajoute les fenetres du viewport au menu.
-			//g_print (" listing des fenetres du bureau (%d;%d;%d) ...\n", iNumDesktop, iNumViewportX, iNumViewportY);
+			cd_debug ("Windows' listing (%d;%d;%d) ...", iNumDesktop, iNumViewportX, iNumViewportY);
 			cd_switcher_foreach_window_on_viewport (iNumDesktop,
 				iNumViewportX,
 				iNumViewportY,
