@@ -114,7 +114,7 @@ static gboolean _cd_switcher_get_viewport_from_clic (Icon *pClickedIcon, int *iN
 			iMouseX = iMouseY;
 			iMouseY = tmp;
 		}
-		double fMaxScale = cairo_dock_get_max_scale (myContainer);
+		double fMaxScale = cairo_dock_get_icon_max_scale (myIcon);
 		double dx = myData.switcher.fOffsetX / fMaxScale * myIcon->fScale;
 		double dy = myData.switcher.fOffsetY / fMaxScale * myIcon->fScale;
 		w -= 2 * dx;
@@ -122,16 +122,16 @@ static gboolean _cd_switcher_get_viewport_from_clic (Icon *pClickedIcon, int *iN
 		iMouseX -= dx;
 		iMouseY -= dy;
 		
-		if (iMouseX < 0 || iMouseX > w || iMouseY < 0 || iMouseY > h)
-			return FALSE;
-		/**if (iMouseX < 0)
+		/**if (iMouseX < 0 || iMouseX > w || iMouseY < 0 || iMouseY > h)
+			return FALSE;*/
+		if (iMouseX < 0)
 			iMouseX = 0;
 		if (iMouseY < 0)
 			iMouseY = 0;
 		if (iMouseX > w)
 			iMouseX = w;
 		if (iMouseY > h)
-			iMouseY = h;*/
+			iMouseY = h;
 		
 		int iNumLine = (int) (iMouseY / (h) * myData.switcher.iNbLines);
 		int iNumColumn = (int) (iMouseX / (w) * myData.switcher.iNbColumns);
@@ -262,16 +262,15 @@ static void _cd_switcher_expose_desktops (GtkMenuItem *menu_item, CairoDockModul
 	_cd_expose_desktops ();
 }
 CD_APPLET_ON_BUILD_MENU_BEGIN
-	// Sub-Menu
-	GtkWidget *pSubMenu = CD_APPLET_CREATE_MY_SUB_MENU ();
+	// Workspaces
 	CD_APPLET_ADD_IN_MENU_WITH_STOCK (D_("Add a workspace"),
 		GTK_STOCK_ADD,
 		_cd_switcher_add_desktop,
-		pSubMenu);
+		CD_APPLET_MY_MENU);
 	CD_APPLET_ADD_IN_MENU_WITH_STOCK (D_("Remove last workspace"),
 		GTK_STOCK_REMOVE,
 		_cd_switcher_remove_last_desktop,
-		pSubMenu);
+		CD_APPLET_MY_MENU);
 	
 	int iNumDesktop, iNumViewportX, iNumViewportY;
 	if (_cd_switcher_get_viewport_from_clic (pClickedIcon, &iNumDesktop, &iNumViewportX, &iNumViewportY))
@@ -280,24 +279,24 @@ CD_APPLET_ON_BUILD_MENU_BEGIN
 		CD_APPLET_ADD_IN_MENU_WITH_STOCK_AND_DATA (D_("Rename this workspace"),
 			GTK_STOCK_EDIT,
 			_cd_switcher_rename_desktop,
-			pSubMenu,
+			CD_APPLET_MY_MENU,
 			GINT_TO_POINTER (iIndex));
 		if (iNumDesktop != myData.switcher.iCurrentDesktop || iNumViewportX != myData.switcher.iCurrentViewportX || iNumViewportY != myData.switcher.iCurrentViewportY)
 		{
 			GtkWidget *pMenuItem = CD_APPLET_ADD_IN_MENU_WITH_STOCK_AND_DATA (D_("Move current workspace to this workspace"),
 				GTK_STOCK_JUMP_TO,
 				_cd_switcher_move_to_desktop,
-				pSubMenu,
+				CD_APPLET_MY_MENU,
 				GINT_TO_POINTER (iIndex));
 			gtk_widget_set_tooltip_text (pMenuItem, D_("This will move all windows from the current desktop to the one you clicked on."));
 		}
 	}
 	
-	// Main Menu
-	if (pSubMenu == CD_APPLET_MY_MENU)
-		CD_APPLET_ADD_SEPARATOR_IN_MENU (pSubMenu);
+	// separator
+	CD_APPLET_ADD_SEPARATOR_IN_MENU (CD_APPLET_MY_MENU);
 	gchar *cLabel;
 	
+	// desktop actions
 	cLabel = (myConfig.iActionOnMiddleClick == SWICTHER_WINDOWS_LIST ? g_strdup_printf ("%s (%s)", D_("Windows List"), D_("middle-click")) : g_strdup (D_("Windows List")));
 	GtkWidget *pWindowsListMenu = CD_APPLET_ADD_SUB_MENU_WITH_IMAGE (cLabel, CD_APPLET_MY_MENU, GTK_STOCK_DND_MULTIPLE);
 	g_free (cLabel);
@@ -305,7 +304,7 @@ CD_APPLET_ON_BUILD_MENU_BEGIN
 
 	cLabel = (myConfig.iActionOnMiddleClick == SWICTHER_SHOW_DESKTOP ? g_strdup_printf ("%s (%s)", D_("Show the desktop"), D_("middle-click")) : g_strdup (D_("Show the desktop")));
 	CD_APPLET_ADD_IN_MENU_WITH_STOCK (cLabel,
-		GTK_STOCK_FULLSCREEN,
+		MY_APPLET_SHARE_DATA_DIR"/../shared-files/images/show-desktop.svg",
 		_cd_switcher_show_desktop,
 		CD_APPLET_MY_MENU);
 	g_free (cLabel);
@@ -314,7 +313,7 @@ CD_APPLET_ON_BUILD_MENU_BEGIN
 	{
 		cLabel = (myConfig.iActionOnMiddleClick == SWICTHER_EXPOSE_DESKTOPS ? g_strdup_printf ("%s (%s)", D_("Expose all the desktops"), D_("middle-click")) : g_strdup (D_("Expose all the desktops")));
 		CD_APPLET_ADD_IN_MENU_WITH_STOCK (cLabel,
-			GTK_STOCK_LEAVE_FULLSCREEN,
+			MY_APPLET_SHARE_DATA_DIR"/../shared-files/images/expose-desktops.svg",
 			_cd_switcher_expose_desktops,
 			CD_APPLET_MY_MENU);
 		g_free (cLabel);
@@ -323,21 +322,17 @@ CD_APPLET_ON_BUILD_MENU_BEGIN
 	{
 		cLabel = (myConfig.iActionOnMiddleClick == SWICTHER_EXPOSE_WINDOWS ? g_strdup_printf ("%s (%s)", D_("Expose all the windows"), D_("middle-click")) : g_strdup (D_("Expose all the windows")));
 		CD_APPLET_ADD_IN_MENU_WITH_STOCK (cLabel,
-			GTK_STOCK_LEAVE_FULLSCREEN,
+			MY_APPLET_SHARE_DATA_DIR"/../shared-files/images/expose-windows.svg",
 			_cd_switcher_expose_windows,
 			CD_APPLET_MY_MENU);
 		g_free (cLabel);
 	}
 	
-	// Sub-Menu
-	CD_APPLET_ADD_SEPARATOR_IN_MENU (pSubMenu);
+	CD_APPLET_ADD_SEPARATOR_IN_MENU (CD_APPLET_MY_MENU);
 	CD_APPLET_ADD_IN_MENU_WITH_STOCK (D_("Refresh"),
 		GTK_STOCK_REFRESH,
 		_cd_switcher_refresh,
-		pSubMenu);
-	
-	CD_APPLET_ADD_SEPARATOR_IN_MENU (pSubMenu);
-	CD_APPLET_ADD_ABOUT_IN_MENU (pSubMenu);
+		CD_APPLET_MY_MENU);
 CD_APPLET_ON_BUILD_MENU_END
 
 
@@ -355,9 +350,9 @@ static gboolean _cd_switcher_redraw_main_icon_idle (CairoDockModuleInstance *myA
 	CD_APPLET_LEAVE (FALSE);
 	//return FALSE;
 }
-static void _cd_switcher_queue_draw (CairoDockModuleInstance *myApplet)
+static void _cd_switcher_trigger_redraw (CairoDockModuleInstance *myApplet)
 {
-	if (myData.iSidRedrawMainIconIdle == 0)
+	if (myData.iSidRedrawMainIconIdle == 0 && myData.iSidUpdateIdle == 0)
 	{
 		myData.iSidRedrawMainIconIdle = g_idle_add ((GSourceFunc) _cd_switcher_redraw_main_icon_idle, myApplet);
 	}
@@ -366,7 +361,7 @@ static void _cd_switcher_queue_draw (CairoDockModuleInstance *myApplet)
 gboolean on_change_active_window (CairoDockModuleInstance *myApplet, Window *XActiveWindow)
 {
 	CD_APPLET_ENTER;
-	_cd_switcher_queue_draw (myApplet);
+	_cd_switcher_trigger_redraw (myApplet);
 	CD_APPLET_LEAVE (CAIRO_DOCK_LET_PASS_NOTIFICATION);
 }
 
@@ -386,7 +381,7 @@ gboolean on_change_desktop (CairoDockModuleInstance *myApplet)
 	
 	if (myConfig.bCompactView)
 	{
-		_cd_switcher_queue_draw (myApplet);
+		_cd_switcher_trigger_redraw (myApplet);
 	}
 	else
 	{
@@ -435,7 +430,7 @@ gboolean on_change_screen_geometry (CairoDockModuleInstance *myApplet)
 {
 	CD_APPLET_ENTER;
 	cd_debug ("");
-	cd_switcher_update_from_screen_geometry ();
+	cd_switcher_trigger_update_from_screen_geometry (TRUE);  // TRUE = now
 	CD_APPLET_LEAVE (CAIRO_DOCK_LET_PASS_NOTIFICATION);
 }
 
@@ -443,7 +438,7 @@ gboolean on_window_configured (CairoDockModuleInstance *myApplet, Window Xid, XC
 {
 	CD_APPLET_ENTER;
 	cd_debug ("");
-	_cd_switcher_queue_draw (myApplet);
+	_cd_switcher_trigger_redraw (myApplet);
 	CD_APPLET_LEAVE (CAIRO_DOCK_LET_PASS_NOTIFICATION);
 }
 
